/* controller.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

/* #include "global.h" */ 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


/*--------------------------------------------CONTROLLER  TYPE 2--------------*/
int controller( unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
               qp_sum,tsim,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
{
/*  etc/xfig.awk

    TYPE 02
    PRIORITY  3          used by config routine: controller priority
    HYDRAULIC 99         used by config routine: non hydraulic

    
    PAR1: 1 [] mode: 1 On&Off, 2:unisol, 3:deltaT_matched 4:heating circle
               mode 6: heating circle thermostat valve
               mode 7: buffer control, mode 8: test              
    PAR2: 3 [C] dT_on,mode 
    PAR3: 3 [C] dT_off
    PAR4: 1 [] kP proportional factor
    PAR5: 300 [] mode1&2:mp_max for pump,mode 3:deltaT || Tout_set(if>20)
    PAR6: 1e-4 [] kI for mode 3,4, 2:LowFlowFlag, 7:Tset
    PAR7: 70 [] Tmax storage mode 1,3,4, mode3 Tcoll_max=Tmax+30
    PAR8: 0.3 [] mode3: ctr_min=0..1
    PAR9: 0   [] 0:standard control 1:nightflow if Tstorage>60 in mode 3

    INP1: 10 [C] mode 1 T_set, mode 2 Tkoll
    INP2: 10 [C] mode 1 T_Sp, mode 2 TR1
    INP3: 10 [C] TVL for mixer, mode 4 
    INP4: 10 [C] TRL, TStorageTop mode 1,3 
    INP5: 10 [C] Tamb

    OUT1: [] ctr 0..1
    OUT2: [] mode1: state 1/0,ON/OFF; mode3:integr_dT
    OUT3: [] no of swtch OFF/ON
    OUT4: [] ctr_mixer 
    OUT5: [s] t_switch_on
    
    OUT6: [Ws] system inlet energy 
    OUT7: [Ws] system outlet energy 
    OUT8: [Ws] internal energy 

*/

  int a=0; /*---------------------------analyse a=1--------------*/

    if (init_flag)
      {
        if (   ( (int) par[unit][1]<1)
	        || ( (int) par[unit][1]>8) )
	       { printf(" mode controller unit:%d error \n",unit); exit(-1);}
        if (    ((int) par[unit][1]==3)
	         && (      par[unit][8]>100 ) ) 
           {printf(" par8 controller unit:%d error \n",unit); exit(-1);}
		    
        /* defaults  */
	out_n[unit][1] =out_o[unit][1] =0; /*control state*/
	out_n[unit][2] =out_o[unit][2] =0; /*state;  reset integr_dT */
	out_n[unit][3] =out_o[unit][3] =0; /*count swtch_enrg*/
	out_n[unit][14] =out_o[unit][14] =0; /* swtch_2p */

	return 1; /*wait for outlet temperatures */
      }

 /*--------------------------------mode 1 bang-bang-control--------------------*/
  if ((int) par[unit][1]==1)
  {
  double TStorageTop=out_o[  in_u[unit][4] ]
                          [ in_nr[unit][4] ];     /* inp4*/
  double Tmax     =par[unit][7]; /* Tmax for TStorageTop */
  
    if (  (out_o[  in_u[unit][1] ]
                [ in_nr[unit][1] ] >110)  /* Tcoll */
        || (TStorageTop>Tmax) 
       ) 
       { /* pumpe OFF */
       out_n[unit][1]=0;
       out_n[unit][2]=0;
       return (1);
       }
       
    if ( (out_o[unit][2] ==0) &&                  /*if state=0&&dT>=dT_on*/
         ( (out_o[  in_u[unit][1] ]
	         [ in_nr[unit][1] ]-out_o[  in_u[unit][2] ]
		  		         [ in_nr[unit][2] ])>=par[unit][2]) )
      { /*switch ON */
	if (a)
	printf("controller:temp1:%.4f\n"
	       "          temp2:%.4f\n"
	       "out_o[%d][2]:%.4f\n\n"
	       ,out_o[  in_u[unit][1] ]
		     [ in_nr[unit][1] ]
               ,out_o[  in_u[unit][2] ]
		     [ in_nr[unit][2] ]
               ,unit
	       ,out_o[unit][2]
		     );

	qp_sum[unit][1][1]+=1;
	out_n[unit][1]=1;
	out_n[unit][2]=1;
      }
    else { 
	   if ( (out_o[unit][2]==1) &&                  /*if state=1&&dT<=dT_off*/
	      ( (out_o[  in_u[unit][1] ]
		        [ in_nr[unit][1] ]-out_o[  in_u[unit][2] ]
		  			        [ in_nr[unit][2] ])
                  <=par[unit][3]) ) 
             { /* pumpe OFF */
	       out_n[unit][1]=0;
	       out_n[unit][2]=0;
             }
           else { /*no change */
		  out_n[unit][1]=out_o[unit][1];
		  out_n[unit][2]=out_o[unit][2];
                };
	 };

	if (a)
	printf("controller:temp1:%.4f\n"
	       "          temp2:%.4f\n"
	       "out_o[%d][2]:%.4f\n\n"
	       ,out_o[  in_u[unit][1] ]
		     [ in_nr[unit][1] ]
               ,out_o[  in_u[unit][2] ]
		     [ in_nr[unit][2] ]
               ,unit
	       ,out_o[unit][2]
		     );
  } /*--------------------------------------------------------------------------*/
  
  
  /*----------------------mode 3 deltaT matched flow solar ---------------------*/
  if ((int) par[unit][1]==3) /*----------------------mode 3 matched ------------*/
  {
  int SWITCH_ON=0,SWITCH_OFF=0; /* check pump flipping */
  double Tcoll      =out_o[  in_u[unit][1] ]      /* Tcoll, inp1*/
                          [ in_nr[unit][1] ];
  double Tmess      =out_o[  in_u[unit][2] ]
                          [ in_nr[unit][2] ];     /* inp2*/
    /* double TColdOutWT =out_o[  in_u[unit][3] ]
                               [ in_nr[unit][3] ];
    */
  double TStorageTop=out_o[  in_u[unit][4] ]
                          [ in_nr[unit][4] ];     /* inp4*/
  double E40        =out_o[  in_u[unit][5] ]
                          [ in_nr[unit][5] ];             /* inp4*/
		    
  double hys_high =par[unit][2];
  double hys_low  =par[unit][3];             /* par2   */
  double kP       =par[unit][4];
  double deltaTset=par[unit][5]; /* if par5>20 Tset=par5 control dT , if -1 ,take optimal mp*/
  double Tset     =par[unit][5];
  double kI       =par[unit][6];
  double Tmax     =par[unit][7]; /* Tmax for TStorageTop -->ctr=0 ,also if Tcoll > Tmax+30  */
  double ctr_min  =par[unit][8]; /* minimum flow */


  double dT;                     /* Tcoll-Tmess-deltaTset */
  double integr_dT=out_o[unit][2];
  double ctr=0;
 
  if (ctr_min>1) { printf("controller par8 error \n");exit(-1); };

  if (Tset<20) Tset=0; /* use deltaT control, otherwise Tout fixed */
  if ( ( (Tcoll-Tmess)<hys_low ) 
      || ( ( Tcoll>Tmax+30) || ( TStorageTop>Tmax ) ) )  
    {
    ctr=0;
    out_n[unit][2] = out_o[unit][2]=0; /* reset integr_dT */
    out_n[unit][2] = integr_dT =0;
    if ( ( Tcoll>Tmax+30) || ( TStorageTop>Tmax ) )  ctr=0; 
    if ( (out_o[unit][1]>0) && ( ctr==0 ) ) SWITCH_OFF=1; 
    }	 
  else     
    {
    if (out_o[unit][1]>0) /* if pump was on */
      {
      if (Tset>0)  dT=Tcoll-Tset;           /* Tout fixed */
      else         dT=Tcoll-Tmess-deltaTset; /* deltaT control */
      /* discrete sum of the difference  */
      if (fabs (integr_dT)<2000 ) integr_dT+=dT*h;
      /* else integr_dT =0;        auto reset windup */
       
      /* ctr=ctr_old+kP*dT+kI*integr_dT; */
      ctr=kP*dT+kI*integr_dT;
      if (deltaTset==-1) { /* take optimal mp(E,Tin) */
                           ctr=(15+0.040*E40-0.188*Tmess)/50; /* /mp_max/m2 */
                         }
      if (ctr<0)  { ctr=0; integr_dT=0;  }
      if (ctr>1)    ctr=1;
      if ( ctr<ctr_min) ctr=ctr_min; /* mp_min=par8*mp_max */
      ctr=(double) ( (int) (10*ctr+0.5))/10.0; /* only 10 steps */

      /* if (   (ctr==0) && (Tcoll-Tmess>hys_low) ) ctr=ctr_min; */
      /*  printf("ctr:%.1f \n",ctr);  */   
      /* stop pump to prevent overheating of storage */
      /* if ( ( Tcoll>Tmax+30) || ( TStorageTop>Tmax ) )  { ctr=0; SWITCH_OFF=1; } */ 
      }
    else /* Pump was OFF */
      if (   ((Tcoll-Tmess)>hys_high) && (TStorageTop<Tmax-5)
          &&  (Tcoll<Tmax-4)          ) { ctr=ctr_min; SWITCH_ON=1; }
      else                                ctr=0.0;
    }
/*
  if ( (TStorageTop>60) && (nightflow>0) ) 
   { 
   if ( ( Tcoll <Tmax+30) && ( TStorageTop<Tmax ) ) ctr=1;
   else                                             ctr=0;
   }
*/
  out_n[unit][1] = ctr;
  out_n[unit][2] = integr_dT; /* save out2 non discrete state */
  if (SWITCH_ON) out_n[unit][5]=out_o[unit][5]=tsim;
  if (tsim<=h)   out_n[unit][5]=out_o[unit][5]=24*3600+1;
  if ( (SWITCH_OFF) && (tsim<=(out_o[unit][5]+h)))
    { printf("controller[%d] mode 3: flipping ctr Tmess:%.1f Tcoll:%.1f "
          "TStorageTop:%.1f ctr:%.2f hys_high:%.1f hys_low:%.1f \n"
           ,unit,Tmess,Tcoll,TStorageTop,ctr,hys_high,hys_low); }
  }
/*----------------------------------------------------------------------------*/
 
 return 1;
}

