/* load_control.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include "global.h"
#include <stdio.h>
#include <stdlib.h>

/*--------------------------------LOAD_CONTROL-----------*/
int load_control(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
               qp_sum,tsim,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
{
/*  etc/xfig.awk
    
    TYPE 15
    PRIORITY  3          used by config routine: controller priority
    HYDRAULIC 99         used by config routine: non hydraulic


    
    PAR1: 2 [] Nr of loads 1..4 if 0 autoselect of inputs to get par1
    PAR2: 5 [] kP proportionalfactor of PID
    PAR3: 0 [] kI integr-factor of PID
    PAR4: 0 [] kD diff-factor of PID
    PAR5: 0 [] mode 1 or 0:default mode 2:ctr1 static control 

    INP1: 10 [C] T1re, real temperature at water-drop-outlet load1
    INP2: 10 [C] T2re,  load2
    INP3: 10 [C] T3re,  load3
    INP4: 10 [C] T4re,  load4
    INP5: 0 [C] T_cw: cold water-inlet temperature
    INP6: 0 [C] Tset1: temperature-demand of load1
    INP7: 0 [kg/h] mp1: mass-flow of load1
    INP8: 0 [C] Tset2: temperature-demand of load2
    INP9: 0 [kg/h] mp2: mass-flow of load2
    INP10: 0 [C] Tset3: temperature-demand of load3
    INP11: 0 [kg/h] mp3: mass-flow of load3
    INP12: 0 [C] Tset4: temperature-demand of load4
    INP13: 0 [kg/h] mp4:  mass-flow of load4
    INP14: 0 [C] T_ci: set temperature of circulation pump
    INP15: 0 [kg/h] mpcirc: total mass-flow of circulation

    OUT1: [] c_cw: control of diverter KW_inlet
    OUT2: [] c_cw1: control of diverter KW load1
    OUT3: [] c_cw2: load2
    OUT4: [] c_cw3: load3
    OUT5: [] c_cw4: load4 (not used)
    OUT6: [Ws] system inlet energy 
    OUT7: [Ws] system outlet energy 
    OUT8: [Ws] internal energy 
    OUT9: [kg/s] mp-ci massflow of circulation pump
    OUT10: [kg/s] mp1 mass flow of load1
    OUT11: [kg/s] mp2 load2
    OUT12: [kg/s] mp3 load3
    OUT13: [kg/s] mp4 load4
    OUT14: [] c_ww1: control for diverter WW (warm water) load1
    OUT15: [] c_ww2: load2
    OUT16: [] c_ww3: load3
    OUT17: [] c_ww4: load4 (not used)
    OUT18: [] c_ci1: control of circ-diverter load1
    OUT19: [] c_ci2: load2
    OUT20: [] c_ci3: load3
    OUT21: [] c_ci4: load4 
*/

 int a=0; /*------------------------------------- ANALYSE with a=1---*/
 int b=0; /*----second level ANALYSE-------------------------*/
 int i; /* runner */
 int change=0; /* change of any massflow or set temperature */
 int mode=(int) par[unit][5]; 
 static int only_one_time=0; /* no second use of this unit ! */
 static int no_loads=0; /* number of loads (water-drops) in dek */

 static double kP,kI,kD;
 static double   dT[5],integr_dT[5],diff_dT[5];
 static double Tist[5],dT_old,mp_old; 
 static double Tsoll[5];
 double T_cw=out_n[  in_u [unit][5] ]
                  [  in_nr[unit][5] ];
 double discrete=10; /* steps of diverter controls */

 static double mp_KW[5]; /* remember the old value for PID */
 static double mp[5];  /* remember for proof of change */
 static double mp_circ[5];
 double mp_sum;
 double mp_Div_KW_WW0=0,mp_Div_KW_WW1=0;
 static double mp_WW[5],
               mp_Div_KW0[5],mp_Div_KW1[5],
               mp_Div_WW0[5],mp_Div_WW1[5];

 static double c_KW_WW,c_KW[5],c_WW[5],c_circ[5];  /* diverter control-variables */

 /* ----------------------init ------------------------*/
 if (init_flag)
   {
     if (only_one_time!=0)
      { printf("use load controller only one time \n"); exit (-1); }

     /* proof par1: Nr of load1 */
     if ( (int) par[unit][1] == 0 )
       {
         for (i=1;i<=4;i++)
           if ( in_u [unit][4+2*i] != 0 ) no_loads+=1;
       }
     else  no_loads=(int) par[unit][1];

     if (a) printf("no_loads:%d \n",no_loads);

     /* set init-values of static variables */
     for (i=1;i<=no_loads;i++)
       {
        /* static's should be stay at zero , if there is no use ! */
        dT[i]=integr_dT[i]=0;
        diff_dT[i]=0;
        mp_WW[i]=mp_KW[i]=0;
        mp_Div_KW0[i]=mp_Div_KW1[i]=0;
        mp_Div_WW0[i]=mp_Div_WW1[i]=0;
        c_KW_WW=c_KW[i]=c_WW[i]=c_circ[i]=0;
        Tsoll[i]=out_n [ in_u [unit][4+2*i] ]
                       [ in_nr[unit][4+2*i] ];
        mp[i]   =out_n [ in_u [unit][5+2*i] ]
                       [ in_nr[unit][5+2*i] ]/3600;
       }
         

     kP=par[unit][2]/h; /* this is buggy, due to variable timestep h  */
     kI=par[unit][3]/h;
     kD=par[unit][4]/h;

     only_one_time=1;

     change=1; /* set _controls */
     /* return 1; waiting for outlet temperatures */
   }
 if (a) printf(" load-control-procedure tsim:%.4f \n",tsim);

 /* ----------load inputs in variables */
 for (i=1;i<=no_loads;i++)
   {
                         /* proof change of Tsoll[i] */
     if ( out_n [ in_u [unit][4+2*i] ]
                [ in_nr[unit][4+2*i] ]
          != Tsoll[i] 
        )
        { 
          if  (a) printf(" there's a change Tsoll[%d]: \n"
                         "Tsoll_new:%.4f\n"
                         "Tsoll_old:%.4f\n"
                         ,i
                         ,out_n [ in_u [unit][4+2*i] ]
                                [ in_nr[unit][4+2*i] ]
                         ,Tsoll[i] );
          change=1;
        }

     Tsoll[i]=out_n [ in_u [unit][4+2*i] ]
                    [ in_nr[unit][4+2*i] ];

     /*                   proof change of mass flow mp[i]  */
     if ( (out_n [ in_u [unit][5+2*i] ]
                 [ in_nr[unit][5+2*i] ]/3600)
          != mp[i]
        )
        {
          if  (a) printf(" there's a mp different from 0: \n"
                         "mp_input:%.4f\n"
                         "mp[%d]:  %.4f\n"
                         ,out_n [ in_u [unit][5+2*i] ]
                                [ in_nr[unit][5+2*i] ]
                         ,i
                         ,mp[i]*3600
                         );
          change=1;
        }
     mp[i]   = out_n [ in_u [unit][5+2*i] ]
                     [ in_nr[unit][5+2*i] ]/3600;  /* kg/h-->kg/s */

     if ( mp[i] != 0 ) change=1; /* because there's sth to control !! */

     Tist [i]=out_o [ in_u [unit][    i] ]
                    [ in_nr[unit][    i] ];
      
   }


 /* ------------------------------------call controller really necessary?--*/
 if (out_o[unit][9] /* mp_circ_old */
     !=  (out_n[  in_u [unit][15] ]  /* set mp_circ */
               [  in_nr[unit][15] ]/3600)
    )
    { 
      change=1;
      if (a) printf(" there's a change of any mp_circ\n"
                    "mp_circ_old:     %.4f\n"
                    "last set mp_circ:%.4f\n"
                    ,out_o[unit][9]*3600
                    ,out_n[  in_u [unit][15] ]
                          [  in_nr[unit][15] ]
                   );
    }

 if (!change)    /* if no change of Tsolls & mp_load & mpcirc & mp[i]==0*/
                 /* then there is no sence of calculating again */
   {
     for (i=1;i<=no_loads;i++)
       dT[i]=integr_dT[i]=diff_dT[i]=mp_KW[i]=0;

     for (i=1;i<=21;i++)
        out_n[unit][i]=out_o[unit][i];  /* set outputs same as last timestep */
     return (change);
   }

      
 /* ------------------------------------controller---------*/

 change = 0; /* set to Zero for mp-check */
 if (init_flag) change = 1;
 for (i=1;i<=no_loads;i++)
   {
     /* evaluate mp_WW & mp_KW */
     dT_old=dT[i];
     dT[i]=Tsoll[i]-Tist[i];  /* difference from set temperature */
     integr_dT[i]+=dT[i];     /* discrete sum of the difference  */
     diff_dT[i]=dT[i]-dT_old; /* change of difference delta_T    */

     if (Tsoll[i]>Tist[i]) /* try to increase temperature as fast as possible */
       {
         mp_KW[i]=0;
         mp_WW[i]=mp[i];
       }
     else /* try to reach set temperature */
       {
         /* There's a PID */
         mp_old=mp_KW[i];
         mp_KW[i] = -kP*dT[i]-kI*integr_dT[i]-kD*diff_dT[i];

         if (mp_KW[i]>mp[i]) mp_KW[i]=mp[i]; /* limit mp_KW */
         if (mp_KW[i]<0    ) mp_KW[i]=0;
         if (mp[i]>0)
         mp_KW[i]=mp[i]*(double) ( (int) (discrete*(mp_KW[i]/mp[i]))) / discrete;
         if (mp_KW[i]!=mp_old) change=1;

         /*
         if (change!=0)
         printf("mp_old:%.4f  mp_KW:%.4f   Tist:%.4f "
                "kP*dT:%.4f ------------\ttsim:%.4f\n"
                ,mp_old
                ,mp_KW[i]*3600,Tist[i],kP*dT[i]*3600,tsim);
         */
         
         mp_WW[i]=mp[i]-mp_KW[i];
       }

         if (a)
         printf("  mp_KW[%d]:%.4f  mp_WW[%d]:%.4f "
                "  mp[%d]:%.4f \n"
                "Tsoll:%.4f   Tist:%.4f \n\n"
                ,i,mp_KW[i]*3600,i, mp_WW[i]*3600
                ,i,mp[i]*3600
                ,Tsoll[i],Tist[i]);

     /* circ_mass flow  kg/h--> kg/s  equal parts */
     mp_circ[i]=out_n[  in_u[unit][15] ]    
                     [ in_nr[unit][15] ] / (no_loads*3600); 
   }

  /*---------------- estimate mass flow for Diverter  */
  for (i=1;i<=no_loads;i++)
    {
      /* total diverting KW-WW at cold water inlet of the system */
      mp_Div_KW_WW0 += mp_KW[i];
      mp_Div_KW_WW1 += mp_WW[i];
     
      /* KW-diverter for each load */
      if (i != no_loads)
        {
          if (i==1) { mp_Div_KW1[i]=0;mp_Div_WW1[i]=0; }
          mp_Div_KW0[i] =  mp_KW[i];
          mp_Div_KW1[i] += mp_KW[i+1]; /* add mp of following KW-Div */
          
          /* WW_Diverter for each load */
          mp_Div_WW0[i] =  mp_WW[i]  +mp_circ[i];
          mp_Div_WW1[i] += mp_WW[i+1]+mp_circ[i+1];
        }    
      else
        {
          mp_Div_KW0[i]=mp_Div_KW1[i]=0;
          mp_Div_WW0[i]=mp_Div_WW1[i]=0;
        }
     
      /* circ-Diverter : mp0=mpcirc[i] mp1=mpload[i] */
     if (a)  printf(" mass flow for Diverter \n"
                    "mp_Div_KW_WW0:%.4f \n"
                    "mp_Div_KW_WW1:%.4f \n"
                    "mp_Div_KW0[%d]:%.4f \n"
                    "mp_Div_KW1[%d]:%.4f \n"
                    "mp_Div_WW0[%d]:%.4f \n"
                    "mp_Div_WW1[%d]:%.4f \n"
                    ,mp_Div_KW_WW0*3600
                    ,mp_Div_KW_WW1*3600
                    ,i
                    ,mp_Div_KW0[i]*3600
                    ,i
                    ,mp_Div_KW1[i]*3600
                    ,i
                    ,mp_Div_WW0[i]*3600
                    ,i
                    ,mp_Div_WW1[i]*3600
                    );
     }

   /*------------------  set Diverter-controls with known mp's */

   for (i=1;i<=no_loads;i++)
     {
       /* total diverting KW-WW at cold water inlet of the system */
       /* contr=0 --> out0=mp_in */
       mp_sum=mp_Div_KW_WW1+mp_Div_KW_WW0;
       if ( mp_sum != 0 )
         {
         c_KW_WW=mp_Div_KW_WW1/mp_sum;
         c_KW_WW=(int) (c_KW_WW*10) /10; /* only 10 steps */
         if (c_KW_WW>1) c_KW_WW=1;
         if (c_KW_WW<0) c_KW_WW=0;
         }
       else
         c_KW_WW=0;

       /* KW-diverter for each load */
       if (i != no_loads)
         {
           mp_sum=mp_Div_KW1[i]+mp_Div_KW0[i];
           if ( mp_sum != 0 )
             {
             c_KW[i]=mp_Div_KW1[i]/mp_sum;
             if (c_KW[i]>1) c_KW[i]=1;
             if (c_KW[i]<0) c_KW[i]=0;
             }
           else
             c_KW[i]=0;

           mp_sum=mp_Div_WW1[i]+mp_Div_WW0[i];
           if ( mp_sum != 0 )
             {
             c_WW[i]=mp_Div_WW1[i]/mp_sum;
             if (c_WW[i]>1) c_WW[i]=1;
             if (c_WW[i]<0) c_WW[i]=0;
             }
           else
             c_WW[i]=0;
         }
       /* circ-Diverter */
       mp_sum=mp_WW[i] + mp_circ[i];
       if ( mp_sum != 0 )
         {
         c_circ[i]= mp_WW[i] / mp_sum;
         if (c_circ[i]>1) c_circ[i]=1;
         if (c_circ[i]<0) c_circ[i]=0;
         }
       else 
         c_circ[i]=0;
       if (a)  printf("set Diverter-controls\n"
                      "c_KW_WW: %.4f \n"
                      "c_KW[%d]:%.4f \n"
                      "c_WW[%d]:%.4f \n"
                      "c_circ[%d]:%.4f \n"
                      ,c_KW_WW
                      ,i
                      ,c_KW[i]
                      ,i
                      ,c_WW[i]
                      ,i
                      ,c_circ[i]
                     );
     }

    /* mode 2 static control of ctr 1 */
    if (mode==2)
     {
     double mp1;
     if (Tist[1]>Tsoll[1]) { 
                           mp1=(mp[1]*(Tsoll[1]-T_cw)/(Tist[1]-T_cw) );
                           c_KW_WW=mp1/(mp[1]);
                           }
     else                  { 
                           mp1=mp[1];
                           c_KW_WW=0;
                           }
     if (c_KW_WW>1) c_KW_WW=1;
     if (c_KW_WW<0) c_KW_WW=0;
     discrete=1000;
     c_KW_WW=(double) ( (int) (c_KW_WW*discrete))/discrete;
     if ((a) && (mp[1]>0)) printf("mp1:%.1f mp[1]:%.1f T_WW:%.1f Tset:%.1f c_KW_WW:%.4f\n"
                                  ,mp1*3600,mp[1]*3600,Tist[1],Tsoll[1],c_KW_WW);
     mp[1]=mp[1]*3600;
     }

  /*--------------------set outputs --------------*/


  out_n[unit][1]=c_KW_WW; /*control of diverter KW_inlet */
  out_n[unit][2]=c_KW[1]; /*control of diverter KW load1 */
  out_n[unit][3]=c_KW[2]; /*control of diverter KW load2 */
  out_n[unit][4]=c_KW[3]; /*control of diverter KW load3 */
  out_n[unit][5]=c_KW[4]; /*control of diverter KW load4 */
  
  /* sys energies */
  /* out_n[unit][6]=0;  gains*/
  /* out_n[unit][7]=0;  losses  */
  /* out_n[unit][8]=0;  internal energy */

  out_n[unit][9]=out_n[  in_u[unit][15] ]
                      [ in_nr[unit][15] ]/3600; /* mp-circ massflow of 
                                                   circulation pump */
  out_n[unit][10]=mp[1]; /* mp1 mass flow of load1 */
  out_n[unit][11]=mp[2]; /* mp2 mass flow of load2 */
  out_n[unit][12]=mp[3]; /* mp3 mass flow of load3 */
  out_n[unit][13]=mp[4]; /* mp4 mass flow of load4 */

  out_n[unit][14]=c_WW[1]; /*control for diverter WW (warm water) load1 */
  out_n[unit][15]=c_WW[2]; /*control for diverter WW (warm water) load2 */
  out_n[unit][16]=c_WW[3]; /*control for diverter WW (warm water) load3 */
  out_n[unit][17]=c_WW[4]; /*control for diverter WW (warm water) load4 */

  out_n[unit][18]=c_circ[1]; /* control of circ-diverter load1 */
  out_n[unit][19]=c_circ[2]; /* control of circ-diverter load2 */
  out_n[unit][20]=c_circ[3]; /* control of circ-diverter load3 */
  out_n[unit][21]=c_circ[4]; /* control of circ-diverter load4 */

        if (b)
        if (    (out_o[unit][1 ] != out_n[unit][1 ])
             || (out_o[unit][18] != out_n[unit][18])
           )
        printf(
                      "mp_KW_WW:%.1f \t"
                      "mp_KW[1]:%.1f \t"
                      "mp_WW[1]:%.1f \t"
                      "mp_circ[1]:%.1f \n"
                      ,c_KW_WW*mp[1]*3600
                      ,(1-c_KW[1])*
                       mp_Div_KW_WW0*3600
                      ,(1-c_WW[1])*(mp_WW[1]+mp_circ[1]
                                            +mp_circ[2])*3600
                      ,(1-c_circ[1])*(mp_circ[1]+mp_WW[1])*3600
                     );
return (change);
}
/*---------------------------------------------------------------------*/

