#include "global.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*-----------------------------------------------energy_outcome---*/
/*
001025 check priority of types for out6,7,8 check ,priority[ type[i] ] >4 && <15
*/

double energy_outcome(int unit,double **out_n,double tsim,
		      int init_flag,int stop_flag,double h,int un_max,
		      double **init_energy,double **out_o,int *type,
		      char **type_def_str,double max_error,
		      double ***qp_sum,
		      double **deriv_o,double **deriv_n,
		      int **in_u,int **in_nr)
{
  /* 
     convention in each TYPE:
     output 6: system energy input
     output 7: system energy output
     output 8: system energy storage in heat capacity (related to 0 degrees)

     energy_outcome() proofs every timestep the energy balance of
     the system
     prints error-message if error>max_error
  */

  int a=0; /* debug */
  double Q_ges=0;          /* current energy of the system [Ws]  */
  double error;
  static double Q_sum=0;   /* input/output-energy of all components */
  static double Q_ges_0=0; /* initial energy of system */
  static double error_max=0;  /* max error of system energy Ws */
				               

  int i;
  int i1=0; /* store active units */

#include "unit_order_init.h" /* priority */
 if (a) printf("dummy-use hydraulic_type[max_types]:%d\n",hydraulic_type[1]);

  /* energy balance of ther system -------------------------*/
  for (i=1;i<=un_max;i++) 
    { 
    if ( ( priority[ type[i] ] >4) && ( priority[ type[i] ] <15) ) 
      /* compare unit_order_init.c PRIORITY 5...14 */
      {
      Q_sum+=  out_n[i][6]     /* Integral of system gains & losses [Ws]  */
              -out_n[i][7];  
      Q_ges+=  out_n[i][8];    /* energy of system at this timestep  */
      }
    };

  if (init_flag)   /*----------Init--*/
   {
      printf("\n System-Energy related to 0 degrees: %.4f [kWh]\n",
	      Q_ges/3.6e+6);
      Q_ges_0=Q_ges-Q_sum;  /* Q_init */
   }

  /* Error of Energy-balance  [Ws] */
  error=fabs(Q_ges_0+Q_sum-Q_ges);
  if (a) printf("energy_outcome: error:%.1f error_max:%.1f \n",error,error_max);

  if ( error > error_max ) error_max=error;

  /* Error limit ------------------------------------ERROR PRINTS ANALYSE---*/
  if (error > max_error)   /* > 1 Ws if h=1 sec */
    {
      static int active_unit[UNIT_MAX+2];
      FILE *error_script;
      if ((error_script=fopen("error_script","w"))==NULL)
         { printf("error open error_script!! \n\n");exit(-1); }
	    
      fprintf(error_script,"tsim:%7.4f h   System-Energy-Check:"
	     "   abs. Error:  %.4e [Ws]\n",tsim/3600,
	     Q_ges_0+Q_sum-Q_ges );

      for (i=1;i<=un_max;i++) /* all units */
	    {
	    /* Energy-Balance */
            if ( ( priority[ type[i] ]>4) && ( priority[ type[i] ]<15) ) 
              /* cmp unit_order_init.c PRIOR 5...14 */
              {
	      if ( fabs(-out_n[i][6]+out_n[i][7]+out_n[i][8]-out_o[i][8])>1e-1) {
	         active_unit[i1]=i;
	         i1++;
	         }
              }
	    }  
      fprintf(error_script," %d active_unit 's \n",i1);
    
      fprintf(error_script,"-TYPE--------UNIT--SysGains-SysLosses-"
                           "SysCapDiff-SysBalance-UnitIn---UnitOut---"
		                   "TotalEnergyBalance \n");
      for (i=1;i<i1;i++) /* only active units */
        {
	    int i3;
	    double SysBalance=0;
	    for (i3=0;i3<=12;i3++) fprintf(error_script,"%1c"
	         ,type_def_str[ type[active_unit[i]] ][i3]);
        fprintf(error_script,"[%d]  ",active_unit[i]);
		 
        if (out_n[ active_unit[i] ] [6]!=0)
	      fprintf(error_script,"%+3.1e  ",out_n[ active_unit[i] ] [6]);
        else fprintf(error_script,"........  ");

        if (out_n[ active_unit[i] ] [7]!=0)
           fprintf(error_script,"%+3.1e  ",out_n[ active_unit[i] ] [7]);
        else fprintf(error_script,"........  ");
	  
	    /* SysLoadDif: dQout:[8] */
        fprintf(error_script,"%+3.1e  "
	        ,out_n[ active_unit[i] ] [8]-out_o[ active_unit[i] ] [8]);
        /* SysBalance */
	    SysBalance=-out_n[ active_unit[i]] [6]+out_o[ active_unit[i] ] [7]
	               +out_n[ active_unit[i]] [8]-out_o[ active_unit[i] ] [8];
		     
	    fprintf(error_script,"%+3.1e  ",SysBalance);

       /* ---------------------------------------Check Storage----------*/
      if (type[active_unit[i]]==4) /* storage */
	    { 
	    double UnitIn=0,UnitOut=0;
	    UnitIn= out_n[ in_u [ active_unit[i] ] [1] ]
	                 [ in_nr[ active_unit[i] ] [1] ]
               +out_n[ in_u [ active_unit[i] ] [3] ]
		             [ in_nr[ active_unit[i] ] [3] ]
               +out_n[ in_u [ active_unit[i] ] [5] ]
		             [ in_nr[ active_unit[i] ] [5] ]
               +out_n[ in_u [ active_unit[i] ] [7] ]
		             [ in_nr[ active_unit[i] ] [7] ];
        UnitOut=out_n[active_unit[i]] [1]
	           +out_n[active_unit[i]] [3]
	           +out_n[active_unit[i]] [9]
	           +out_n[active_unit[i]] [17];
	    fprintf(error_script,"%+3.1e  ",UnitIn);
            fprintf(error_script,"%+3.1e  ",UnitOut);
            fprintf(error_script,"%+3.1e  ",UnitIn-UnitOut-SysBalance);
	    if (fabs(UnitIn-UnitOut-SysBalance)>1e0)
	       fprintf(error_script,"! ERROR !");
        else fprintf(error_script,"OK");
        }
						
      /* ---------------------------------------Check Collector----------*/
      if (type[active_unit[i]]==1) 
	    { 
	    double UnitIn=0,UnitOut=0;
	    UnitIn= out_n[ in_u [ active_unit[i] ] [1] ]
	                 [ in_nr[ active_unit[i] ] [1] ];
        UnitOut=out_n[active_unit[i]] [1];
	    fprintf(error_script,"%+3.1e  ",UnitIn);
            fprintf(error_script,"%+3.1e  ",UnitOut);
            fprintf(error_script,"%+3.1e  ",UnitIn-UnitOut-SysBalance);
	    if (fabs(UnitIn-UnitOut-SysBalance)>1e0)
	       fprintf(error_script,"! ERROR ! ");
        else fprintf(error_script,"OK");
        }

      fprintf(error_script,"\n");
          
        }
      for (i=1;i<i1;i++) /* only active units */
        {
	    int i3;
	    fprintf(error_script,"\n\nUNIT[%d]: ",active_unit[i]);
	    for (i3=1;i3<=OUT_MAX0;i3++) 
	      {
	      fprintf(error_script,"%d:%+3.1e ",i3,out_n[active_unit[i]] [i3] );
	      }
        fprintf(error_script,"\n");
        fprintf(error_script,"\nDERIV[%d]: ",active_unit[i]);
	    for (i3=1;i3<=DERIV_MAX;i3++)
	      {
	      fprintf(error_script,"%d:%.1f ",i3,deriv_n[active_unit[i]] [i3] );
	      }
        
        fprintf(error_script,"\n\n");
	    }

      fprintf(error_script,"---error=fabs(Q_ges_0+Q_sum-Q_ges)=%10.4f \n"
                           "Q_ges_0:%10.4f \n"
			               "Q_sum  :%10.4f \n"
			               "Q_ges:  %10.4f \n"
			               ,error,Q_ges_0,Q_sum,Q_ges);
      printf("\n Outputs_difference:\n");
      Ausgabe_Out_pr(out_n,out_o,un_max,type,type_def_str,1);
      fclose (error_script);
      
      printf("tsim:%7.4f h   System-Energy-Check:"
	     "   abs. Error:  %.4e [Ws] ,script file: ' more error_script ' \n ",
	     tsim/3600,
	     Q_ges_0+Q_sum-Q_ges
	    );
      printf("active_units: ");
      for (i=1;i<i1;i++) printf("%d ",active_unit[i]);
      printf("\n");
      /* DIRECT PRINTS ------------*/
      {
      double Q_gain=0,Q_cap =0,Q_gain_sum=0,Q_cap_sum=0;
      for (i=1;i<=un_max;i++) /* all units */
        {
        if ( (priority[ type[i] ]>4) && (priority[ type[i] ]<15) ) 
          /* cmp unit_order_init.c PRIOR 5...14 */
          {
          Q_gain =  out_n[i][6]                   /* sys gains and losses  [Ws]  */
                   -out_n[i][7]; 
          Q_cap  =  out_n[i][8]-out_o[i][8]  ;    /* energy-storage [Ws] */
          Q_gain_sum+=Q_gain;
          Q_cap_sum+=Q_cap;

          if (fabs(Q_gain)<1) printf("UNIT:%d gains-losses:........\t ",i);
          else                printf("UNIT:%d gains-losses:%+.1e \t ",
                                   i,Q_gain);
          if (fabs(Q_cap )<1) printf("Cap_diff:........\t");
          else                printf("Cap_diff:%+.1e \t",Q_cap );
          if (fabs(Q_cap -Q_gain)<1)
                              printf("UNIT_EXCHANGE:........\n");
          else                printf("UNIT_EXCHANGE:%+.1e \n",Q_cap -Q_gain);
         }
	}
      printf("\n");

      error=Q_gain_sum-Q_cap_sum;
      printf("error=Q_gain_sum-Q_cap_sum=%.1f Ws Q_gain_sum:%.1f Q_cap_sum:%.1f\n"
             ,error,Q_gain_sum,Q_cap_sum);
      }
					       
      
      exit (-1);                                      /* abort simulation */
    };
       
  if (stop_flag)
    {
      printf("System-Energy related to 0 degrees: %.4f [kWh]\n",
	      Q_ges/3.6e+6);
      printf("                                       -------------\n");
      printf("System-Energy-Check:  |Error_max|:%.4e [Ws]\n",
	      error_max);
      printf("tsim:%7.4f \n",tsim);
    };
  return ( error_max ); /*Energy-Check */

}		       
		       

