/*  global.h ColSim,Christof Wittwer FhG-ISE FBTA
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#define WIN32 0  /* 0: compile for UNIX 1: compile for win32  */

#define INPUTMAX  10     /* fuzzy_controller */

#define UNIT_MAX  200    /* Max units + DEAD_MAX */
#define PAR_MAX   40 
#define INPUT_MAX 40
#define OUT_MAX0  50 /* number of outputs (excluding init_values of inputs) */
#define OUT_MAX OUT_MAX0+INPUT_MAX  /* OUT_MAX0+INPUT_MAX
                                    OUT_MAX0+1: input initial 1
                                    OUT_MAX0+2: input initial 2 , ... etc...
                                          Input initials for out(>OUT_MAX0)*/

#define PATH_MAX 10            /* No of Energy Flows to Node */
#define MAX_POINTS 300        /* Max Number of values for t-Axes */

#define DEAD_TIME_MAX 10000     /* max no of layers (*h) */
#define DEAD_MAX 40           /* max no of dead_time-components */
                              /* Attention: collect. needs 4 comp's */
                              /*            storage:2 wt's          */
#define PUMP_NR 20    /*max number of pumps */
#define DIV_NR 30     /* max number of flow diverters */
#define NODE_MAX 101   /* max no of nodes in one TYPE, array size of **Qi  */
/* #define DERIV_MAX INPUT_MAX     earlier:15, bug in printer 10/97 */
#define DERIV_MAX NODE_MAX    /* earlier:15, bug in printer 10/97 */

#define IN 0
#define OUT 1                 /* point_dead[][OUT] at outlet temp  */

#define WEATHER_SIZE 10000  /* lines in weather[] */
#define LOAD_SIZE 5000       /* lines in load[]  */

#define NR_TETA_MAX 91 /* iam table  */
#define NR_PHI_MAX 361
#define XXX       -999

#define TYPES_STRING_LENGTH  13 /* name for the types */


/* Type Definitions:
  type es explicit defined in main(), init_type_def_str.c */

#define TypeAirRadNode 29   /* Now used in wall.c */ 



/* -------------malloc_call.c--------------------------------------*/
char**    malloc_2char(int x,int y);
double*   malloc_1double(int x);
double**  malloc_2double(int x,int y);
double*** malloc_3double(int x,int y,int z);
int**     malloc_2int(int x,int y);
int*      malloc_1int(int x);


double deadtime(int nr2,int t_dead,double Q_in,double **Q,
                int **point_dead,double mp_cp_h,double m_rest_cp
                ,double energy_dead, double mc);  

double deadtime_change(int nr3,int t_dead_old,int t_dead_new,double **Q
                       ,int **point_dead,double mc,double cp
                       ,double energy_dead
                       ,double mp_cp_h_old,double m_rest_cp_old
                       ,double mp_cp_h    ,double m_rest_cp    );

double energy_outcome(int unit,double **out_n,double tsim,
                      int init_flag,int stop_flag,double h,int un_max,
                      double **init_energy,double **out_o,int *type,
                      char **type_def_str,double max_error,
                      double ***qp_sum,
		      double **deriv_o,double **deriv_n,
		      int **in_u,int **in_nr);

/*-----------------------------------------------------------------*/
int init_set_mp(double **par,int **in_u,int **in_nr,double **out_o,
           double **out_n,int *type,int pump_nr,int *pump_unit,
           int coll_pipe_nr,int *coll_pipe_unit,double tsim,
           int un_max,char **type_def_str,
           int i_div,int *div_unit,int mp_change,int init_flag,
           int **active_circle);

int init_type_def_str(char **type_def_str);

/*-----------print_infos.c-----------------------------------------*/
int Ausgabe_Input(int **in_u,int **in_nr,int un_max,
                  int *type,char **type_def_str);

int Ausgabe_Out_pr(double **out_n,double **out_o,int un_max,
                   int *type,char **type_def_str, int difference );

int dek_reader(int *type,double **par,int **in_u,int **in_nr,
                double **out_n,double **out_o,
                double **deriv_o,double **deriv_n);

/*------------------------------weather.c--------------------------*/
int weather(
             double tsim,    /* simulation time in seconds of this day */
             int     unit,    /* current unit number */
             double **par,    /* parameter array par[unit][1..PARMAX] */
             double **out_o,  /* output array with privious state of out[unit][1..OUTMAX] */
             double **out_n,  /* output array with current state */
             int year_start,  /* filename generate */
             int month,
             int day,
             int day_flag,     /* if tsim==0 --> weather_init */
             double lenght,   /* return geographic lenght, due to solar noon */
             double latitude /* geographic latitude */
             );

double weather_init(int unit,double **out_o,double **out_n,
                    int spalten,double *weather,int year_start,int month,int day,
                    int unix_flag,int *weather_flag,double **par);

/*------------------------------------------------------------------*/

/*------------------------------load.c-TYPE 14 --------------------*/
double load_init(int unit,double **par,double **out_o,
                 double **out_n,double **load);
      /*returns cycle in sec*/
int load(int unit,double tsim,double **par,double **out_o,double **out_n,
                      int day_of_year,int day_of_week);
/*------------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
int rad_processor(double tsim,int day_of_year,double Ib_hor, double Id_hor
                  ,double rho_g,double E_40,double slope,double orientation
                  ,double *out,double lenght,double latitude,double Rd
                  ,double Rr);

/*-----------------angle_modifier-iam----- ------------------------*/
double inc_angl_mod(int init_flag, double b0, double teta,double phi
                    ,double **iam);

/*--------------------- defs for thermodynamics --------------------------------*/
double F_p_steam_s (double T_air);       /* [N/m2] pressure saturated air*/
double F_x (double p_steam, double p_air);
double F_rho_air_try (double T_air, double p_air);
double F_rho_humid (double T_air, double x, double p_air);
double F_h_air_try (double T_air);         /*[Ws/kg] enthalpie try air*/
double F_h_steam (double T_air);        /*[Ws/kg] enthalpie water steam*/
double F_h_water (double T_air);         /*[Ws/kg] enthalpie water*/


int type_call(
              char **type_def_str,
              int unit,
              int un_max,
              int *type,
              int init_flag,
              int day_of_year,
              int year_start,
              int month,
              int day,
              int hour,
              int min,
              int sec,
              int day_flag,
              int hour_flag,
              int week_flag,
              int day_of_week,
              int min10_flag,
              int stop_flag,
              int previous_unit,
              int **in_u,
              int **in_nr,
              int **point_dead,
              double **iam,
              double h,
              double tsim,
              double **par,
              double **out_n,
              double **out_o,
              double **deriv_o,
              double **deriv_n,
              double **init_energy,
              double **Qi,
              double **Q, /* pipe */
              double ***qp_sum,
              int a
             );

int AirRad_node(
           int unit,   /* current unit number */
           double **par,  /* parameter array par[unit][1..PARMAX] */
           int **in_u,    /* input unit array, input connections e.g. [3, 2] --> unit:3 */
           int **in_nr,   /* input number array                  e.g. [3, 2] --> no= 2 */
           double **out_o, /* output array with privious state of out[unit][1..OUTMAX] */
           double **out_n, /* output array with current state */
           double **deriv_o, /* derivatives: used for diffential equations */
           double **deriv_n, /*  new state of deriv */
           double h,         /* sim time step */
           double **init_energy, /* initial energy in RC-componentes [Ws] */
           double ***qp_sum, /* energy to one node qp[unit][n][m] */
           double tsim,      /* simulation time in [s] of this day */
           int init_flag,    /* set at initial loop of sim */
           int day_of_year,  /* 1..365 */
           int lenght,       /* geographic lenght for rad_processor */
           int latitude,     /* geographic latitude */
           int *type         /* type_number */       
);

int RC(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */
);

int collector(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int *i_dead_nr,   /* deadtime_component (history)          */
           double **Q,       /* enthalpie array                       */
           int **point_dead, /* deadtime component                    */
           int min10_flag,   /* each 10 minutes intervall set         */
           int day_of_year,  /* day_of_year                           */
           int un_max,       /* maximum number of units               */
           double lenght,    /* geographic lenght                     */
           double latitude,  /* geographic latitude                   */
           double **iam      /* incidence angle modifier array        */

);

int controller(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */
);

int controller_x(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */

);

int diverter_air(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           int    init_flag  /* set at initial loop of sim            */
);

int equation(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int    day_of_year 
);

int equation_x(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int    day_of_year 
);

int flow_div(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           int    init_flag  /* set at initial loop of sim            */
);

int fuzzy_controller(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double h,         /* sim time step                         */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */
);

int gnu_plotter(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double h,         /* sim time step                         */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int    stop_flag, /* set at simulation end                 */
           int    unix_flag, /* running on unix/linux                 */
           int    day_of_year 
);

int heater(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */

);

int heater_air(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */


);

int heatexchanger(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int previous_unit  /* get unit which is connected with inp i */
);

int heating(
         int unit,
         double **par,
         int **in_u,
         int **in_nr,
         double **out_n,
         double **out_o,
         double **deriv_o,
         double **deriv_n,
         double h,
         double **init_energy,
         double ***qp_sum,
         double tsim,
         int init_flag,
         int *i_dead_nr, 
         double **Q,
         int **point_dead,
         int un_max 

);

int heatpump(
	   int unit,   /* current unit number */
	   double **par,  /* parameter array par[unit][1..PARMAX] */
	   int **in_u,    /* input unit array, input connections e.g. [3, 2] --> unit:3 */
	   int **in_nr,   /* input number array                  e.g. [3, 2] --> no= 2 */
	   double **out_o, /* output array with privious state of out[unit][1..OUTMAX] */
	   double **out_n, /* output array with current state */
	   double h,         /* sim time step */
	   double ***qp_sum, /* energy to one node qp[unit][n][m] */
	   int init_flag     /* set at initial loop of sim */
);

int mass_element(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int *type,        /* type number array                     */
           int day_of_year 
);

int mixer(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double h,         /* sim time step                         */
           int    init_flag  /* set at initial loop of sim            */

);

int mixer_air(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double h,         /* simulation time step                  */
           int    init_flag  /* set at initial loop of sim            */
);

int pipe2(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */
);

int pipe_air(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int    *type      /* type array                            */


);

int printer(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int year_start,  /* filename generate */
           int day_of_year,
           int month,
           int day,
           int hour,
           int min,
           int sec,
           int day_flag,
           int week_flag,
           int hour_flag,
           int min10_flag,
           int stop_flag,
           int un_max 

);

int pump(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int    previous_unit  /* coming from this unit             */


);

int pump_air(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int    previous_unit  /* coming from this unit             */

);

int storage(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int    *type,     /* type number array                     */
           int    day_of_year,
           int    previous_unit,
           int    un_max     /* unit max */
);

int wall(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag, /* set at initial loop of sim            */
           int *type,        /* type number array                     */
           int day_of_year,
           double lenght,    /* geographic lenght                     */
           double latitude,  /* geographic latitude                   */
           double **iam      /* incidence angle modifier array        */

);

int water_drop(
           int    unit,      /* current unit number */
           double **par,     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u,    /* input unit array, input connections   */
           int    **in_nr,   /* input number array                    */
           double **out_o,   /* output array with privious state      */
           double **out_n,   /* output array with current state       */
           double **deriv_o, /* derivatives: used for diffential equ. */
           double **deriv_n, /*  new state of deriv                   */
           double h,         /* sim time step                         */
           double **init_energy, /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum, /* energy to one node qp[unit][n][m]     */
           double tsim,      /* simulation time in [s] of this day    */
           int    init_flag  /* set at initial loop of sim            */

);

int zone(
           int unit,   /* current unit number */
           double **par,  /* parameter array par[unit][1..PARMAX] */
           int **in_u,    /* input unit array, input connections e.g. [3, 2] --> unit:3 */
           int **in_nr,   /* input number array                  e.g. [3, 2] --> no= 2 */
           double **out_o, /* output array with privious state of out[unit][1..OUTMAX] */
           double **out_n, /* output array with current state */
           double **deriv_o, /* derivatives: used for diffential equations */
           double **deriv_n, /*  new state of deriv */
           double h,         /* sim time step */
           double **init_energy, /* initial energy in RC-componentes [Ws] */
           double ***qp_sum, /* energy to one node qp[unit][n][m] */
           double tsim,      /* simulation time in [s] of this day */
           int init_flag,    /* set at initial loop of sim */
           int day_of_year,  /* 1..365 */
           int lenght,       /* geographic lenght for rad_processor */
           int latitude,     /* geographic latitude */
           int *type         /* type_number */       

);
