/* collectorRadProc2.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/


/*
000607 pointer lenght,latitude
011108 copied to more_Types ;mode_radproc=IGH_IDH; rad_processor.h included
011128 labels update comment
020305 if (init_flag) radiation->iam_diff=tau_alpha_diff;
       tau_alpha_diff--> iam_diff
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h" 
#include "rad_processor.h" 


/*-------------------1xn MODEL -----------------------------COLLECTOR-------*/
int collector(info,interface,radiation,unit,par,in_u,in_nr,out_o,
              out_n,deriv_o,deriv_n,h,init_energy,
              qp_sum,tsim,init_flag,
              day_of_year,un_max,lenght,latitude,iam)
           struct sim_struct *info; /* see global.h */
           struct interface_struct *interface; /* see global.h */
           struct radiation_struct *radiation; /* see rad_processor.h */

           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int day_of_year;  /* day_of_year                           */
           int un_max;       /* maximum number of units               */
           double *lenght;   /* geographic lenght                     */
           double *latitude; /* geographic latitude                   */
           double **iam;     /* incidence angle modifier array        */

{
/*  etc/xfig.awk   
  
     TYPE 01
     PRIORITY  6          used by config routine
     HYDRAULIC 1          used by config routine, hydraulic 2 pole

    
     PAR1: 4.94 [m2] 'A' total absorber area
     PAR2: 0.83 [1] 'tau_alpha0' without F_str
     PAR3: 0.7229 [1] 'iam_diff'  -1:auto_calc 
     PAR4: 2.98 [W/m2K] 'k0' (k*F'=k0+k1*dT) , a_a0 apertur-related
     PAR5: 0.01 [W/m2K2] 'k1' , a_a1 apertur-related includind F_str     
     PAR6: 10 [1] 'n' number of nodes in 1xn-model                          
     PAR7: 3.8 [kJ/kgK] 'cp' specific capacity cp of fluid 
     PAR8: 45 [] 'slope' slope
     PAR9: 0 [] 'orient' orientation (0:south,90:west,-90:east)     
     PAR10: 7.78 [m] 'l' length of absorber tube
     PAR11: 6.8 [mm] 'd' inside diameter of tube
     PAR12: 30 [kJ/Km2] 'cap' capacity per squaremeter
     PAR13: 0.95 [1] 'F_str'  now used  for tau_alpha
     PAR14: -0.12 [1] 'b0' iam-factor:1-b0*(1/cos(teta)-1), if -1: take iam_table
     PAR15: 0 [1] 0:no horizon 1:horizon from './horizon/horizon.dat'

     INP1: 0 [Ws] 'dQin' deltaQ-inlet
     INP2: 0 [kg/s] 'mdot_in' mp-inlet
     INP3: 10.0 [C] 'Tamb' ambient temperature
     INP4: 0 [W/m^2] 'Igh' global horiz. radiation
     INP5: 0 [W/m^2] 'Idh' diffuse radiation horiz. 
     INP6: 0 [W/m^2] 'IgT' global rad. on tilted surf. (e.g. E_40)
     INP7: 0 [1] 'angle' incidence angle
     INP8: 0 [] 'slope_set' slope (if variabel)
     INP9: 0 [] 'orint_set' orientation (if variabel)

     OUT1: [Ws] 'dQout' deltaQ-outlet
     OUT2: [kg/s] 'mdot_out' mp-outlet
     OUT3: [C] 'T_n' last node n
     OUT4: [C] 'T_in'            
     OUT5: [W] 'P_use' 

     OUT6: [Ws] 'sys-gain' system inlet energy 
     OUT7: [Ws] 'sys-loss' system outlet energy 
     OUT8: [Ws] 'sys-ener' internal energy 

     OUT9:  [K] 'T_mean'
     OUT10: [K] 'dTambient'
     OUT11: [W] 'Ploss' thermal losses to ambient                             
     OUT12: [C] 'T1' temp node 1                              

     OUT13: [C] 'T2'                    
     OUT14: [C] 'T3'                            
     OUT15: [K/W] 'deltaTamb/E'

     OUT25: [1] 'eta' eta=Q_use/(E_ges)
     OUT26: [W] 'IgT' (IbT+IrT+IdT) 
     OUT27: [W/m^2] 'IbT' beam rad. on tilted surf.
     OUT28: [W/m^2] 'IrT+IdT' ground refl. on tilted surf.
     OUT29: [1] 'iam' angl_mod(teta)

     OUT30: [1] 'radproc1' rad proc
     OUT31: [1] 'radproc2' rad proc
     OUT32: [] 'teta' theta rad proc teta
     OUT33: [1] 'radproc2' rad proc phi
    
     DERIV1: 10 [C] 'T1' node1 in_tube

  */
  

#define N_MAX 100 

int a=0; /*----------debug flag --------------*/
int pedantic=1;
double energy [N_MAX+1] ;
double Qi[N_MAX+1][5]; 

double F_kA[N_MAX+1]; /* F_kA heatremoval from absorber to fluid */
double deltaT[N_MAX+1];
double A=par[unit][1];
double mc,mc_i;
/* double albedo=0.2; / * ambient reflection */
double cp=par[unit][7]*1e+3;

double tau_alpha0=par[unit][2];
double iam_diff=par[unit][3];
double F_k0=par[unit][4];
double F_k1=par[unit][5];
int n = (int) par[unit][6]; /* 1xn nodes */
double F_str=par[unit][13];
int mode_radproc=IGH_IDH; /* 4;  Igh & Ibn */


double mp_cp_h;
double slope=par[unit][8],orientation=par[unit][9];
/* double b0=par[unit][14]; / * default inc angl mod */
/* double horizon_nr=par[unit][15]; */
double dQ_out;
double Idt_trans,Ibt_trans,Ibn,Igt;
/* inputs */
double dQ_in   =out_n IN(1) ;
double mp      =out_n IN(2);
double Tamb    =out_n IN(3) ;
double Igh=Ibn =out_n IN(4) ;
/* double Idh     =out_n IN(5) ; */
/* double Igt_mess=out_n IN(6); */

int i=0;
double Tmean=0;
double T_tmp,energy_with_plug;
static int local_init_flag=1;
 

if (local_init_flag) { 
 local_init_flag=0; 
}

if (mode_radproc==4) Igh=0; /* in4 is now Ibn */

mc=A*par[unit][12]*1e+3; /*  capacity total fluid and plate and ... */ 
mc_i=mc/n; /* n>=1 !! */

 /* check inputs for variable positioning */
if (  ( in_u [unit][8] != unit ) && ( in_u [unit][8] != 0 ) ) /*inp connected*/ 
  slope=out_n IN(8); 
if (  ( in_u [unit][9] != unit ) && ( in_u [unit][8] != 0 ) )
  orientation=out_n IN(9);

mp_cp_h=mp*cp*h;

for (i=1;i<=n;i+=1) 
   { 
   deltaT[i]=deriv_o[unit][i]-Tamb;                      
   F_kA[i]=A*(F_k0+F_k1*deltaT[i])/( (double) n );
   };


/*----------------- Initialisation of init_energy */
if (init_flag) {
  int i;
  if (deriv_o[unit][1]==0) deriv_o[unit][1]=deriv_n[unit][1]=Tamb;

  for (i=1;i<=n;i++) {
    if (i>1) deriv_o[unit][i]=deriv_n[unit][i]=deriv_o[unit][1];
    init_energy[unit][i]=deriv_o[unit][i]*mc_i;
  }
};  /* end of init----------------------------------------------*/

radiation->mode=IGH_IDH; /* Igh & Ibn */
radiation->horizon_nr=par[unit][15];
if (init_flag) radiation->iam_diff=iam_diff;
radiation->b0=par[unit][14]; /* default inc angl mod */
radiation->Igh=out_n IN(4);
radiation->Idh=out_n IN(5);
radiation->rho_g=0.2;
radiation->slope=par[unit][8];
radiation->orientation=par[unit][9];
rad_proc2(info,interface,radiation);
Ibt_trans=radiation->Ibt_trans;
Idt_trans=radiation->Idt_trans;
Igt=radiation->Igt;
/* if (tsim>10*3600) printf("Igt:%.1f Ibt:%.1f Idt:%.1f Ibn:%.1f Idh:%.1f\n"
                 ,Igt,Ibt,Idt,Ibn,Idh); */

for (i=1;i<=n;i+=1) {   /* energy balance compare to pipe1.c */
 if (i==1)       Qi[i][3]=dQ_in;
 else            Qi[i][3]=mp_cp_h*T_tmp; /*energy plug from previous node*/ 

 energy_with_plug=Qi[i][3]+mc_i*deriv_o[unit][i]; /* get new mixed node */
 T_tmp=energy_with_plug/(mp_cp_h+mc_i);           /* get tmp-temp of node */
 if (a) printf("T_tmp1[%d]:%.2f mp_cp_h:%.2f mc_i:%.2f "
               "energy_with_plug:%.2f\n"
               ,i,T_tmp,mp_cp_h,mc_i,energy_with_plug);
 deltaT[i]=T_tmp-Tamb;
 Qi[i][1]=deltaT[i]*F_kA[i]*h;
   /* check time constant due to kA_i */
 T_tmp=(energy_with_plug-Qi[i][1])/(mp_cp_h+mc_i); /* with amb losses */
 if (   ( (Qi[i][1]>0) && (T_tmp<Tamb) ) /* instable */
     || ( (Qi[i][1]<0) && (T_tmp>Tamb) )
    )
    {
    if (pedantic) printf("coll[%d]:stabilization T_tmp:%.1f Tamb:%.1f "
       "Qi:%.1f \n", unit,T_tmp,Tamb,Qi[i][1]);
    Qi[i][1]=energy_with_plug-Tamb*(mp_cp_h+mc_i);/* set Tnode to Tamb */
 } 

 Qi[i][2]=A*F_str*tau_alpha0*(Ibt_trans+Idt_trans)*h/( (double) n ); /*Q_sun,Qgain*/

 energy_with_plug=energy_with_plug-Qi[i][1]+Qi[i][2]; /* 2nd. tmp-balance */
 T_tmp=energy_with_plug/(mp_cp_h+mc_i); /* 2nd. tmp-temp of node */
 if (a) printf("T_tmp2nd[%d]:%.2f energy_with_plug:%.1f\n" 
                   ,i,T_tmp,energy_with_plug);

 qp_sum[unit][i][1]+=Qi[i][1];
 qp_sum[unit][i][2]+=Qi[i][2];
 qp_sum[unit][i][3]+=Qi[i][3];
     /*first evaluation of absorber-temp, to accelerate 
       energy_flow node 1..2 ,etc. */
 if (a) printf("energy:%.1f Qi[%d][1]:%.2f Qi[%d][2]:%.2f "
               "Qi[%d][3]:%.2f Tamb:%.1f\n"
               ,init_energy[unit][i],i,Qi[i][1],i,Qi[i][2],i,Qi[i][3],Tamb);
 if (a) printf("deriv_o[unit][%d]:%.2f deriv_o[unit][]:%.2f mp_cp_h:%.2f \n"
                  ,i,deriv_o[unit][i],deriv_o[unit][i-1],mp_cp_h);
} /* for (i=1;i<=n;i+=1) */

Qi[n+1][3]=dQ_out=mp_cp_h*T_tmp; /* last element */
qp_sum[unit][n+1][3]+=Qi[n+1][3];
   
for (i=1;i<=n;i+=1) {
  energy[i]=init_energy[unit][i]
                -qp_sum[unit][i  ][1]       /* Q_loss node 2 ambient */
                +qp_sum[unit][i  ][2]       /* Q_sun */
                +qp_sum[unit][i  ][3]       /* Q_in fluid */
                -qp_sum[unit][i+1][3];      /* Q_out fluid */
  deriv_n[unit][i]=energy[i]/mc_i;
  if (deriv_n[unit][i]<(Tamb-10)) printf("coll[%d] T[%d]:%.1f<Tamb:%.1f\n"
                                       ,unit,i,deriv_n[unit][i],Tamb);
  Tmean+=deriv_n[unit][i]/( (double) n);
  if (a) printf("deriv_n[unit][%d]:%.2f\n",i,deriv_n[unit][i]);
};
                   
   if (a) printf("dQ_in:%.4f mp_cp_h:%.4f dQ_out:%.4f\n",dQ_in,mp_cp_h,dQ_out);



/*-------------- set outputs -----------------------------*/

out_n[unit][1]=dQ_out;  /* dQ_out */
out_n[unit][2]=mp;
out_n[unit][3]=deriv_n[unit][n];  /* T_n last node n */
if ( fabs (deriv_n[unit][n])>300) 
   { printf("collector gau unit:%d Tabs:%.1f init_energy:%.2f,P71:%.1f, "
            "P81:%.1f,P82:%.1f \n"
             ,unit,deriv_n[unit][8],init_energy[unit][8],Qi[7][1]/h,
             Qi[8][1]/h,Qi[8][2]/h);  exit (-1); }
if (mp_cp_h>0) out_n[unit][4]=dQ_in/mp_cp_h; /* T_in */                   
else           out_n[unit][4]=deriv_n[unit][1];

out_n[unit][5]=(dQ_out-dQ_in)/h; /* P_use [W] */

  /*System Energy Balance--------------------------------------------- */
out_n[unit][6]=0; for (i=1;i<=n;i++) out_n[unit][6]+=Qi[i][2]; /*sys gains*/
out_n[unit][7]=0; for (i=1;i<=n;i++) out_n[unit][7]+=Qi[i][1]; /* losses */
out_n[unit][8]=0; for (i=1;i<=n;i++) out_n[unit][8]+=energy[i]; /* cap */

   /*  T_mean  */
out_n[unit][9]=Tmean;                   
   /*  deltaTambient */
out_n[unit][10]=Tmean-Tamb;
out_n[unit][11] = out_n[unit][7]/h; /* thermal losses to ambient */
out_n[unit][12] = deriv_n[unit][1]; /* T1 */
out_n[unit][13] = deriv_n[unit][2]; /* T2 */
out_n[unit][14] = deriv_n[unit][3]; /* T3 */
    /* [K/W] deltaTamb/E */
if (mp>0) out_n[unit][15] = (Tmean-Tamb)/(Igt+1e-12); /* T3 */
else      out_n[unit][15] =0; 
if (Igt<10) out_n[unit][15] =0;
if ( (Tmean-Tamb) <0) out_n[unit][15] =0;

/* if (mp>0) 
 printf("tsim:%.1f dT/Igt:%.2f \n",tsim/3600,(Tmean-Tamb)/(Igt+1e-12));
*/

   /*out25: eta=Q_use/(E_ges), out26: A*IgT*/
if (Igt>10) out_n[unit][25] = out_n[unit][5]/(Igt+1e-12);
else        out_n[unit][25] = 0;


   /*  out27: IbT */
out_n[unit][26]=Igt; /* A*(I_beam*(1-shadow)+I_diff); */
out_n[unit][27]=Ibt_trans;
out_n[unit][28]=Idt_trans;

if (1) { /* print irradation(theta) */
 static double Qgt[91],Qbt[91],Qdt[91],Qbt_trans[91],Qdt_trans[91],Qgt_trans[91],Qcoll[91];
 int i;
 double pi=3.14159265;
 double rad=pi/180;


 if (init_flag) 
   for (i=0;i<=90;i++) Qgt[i]=Qbt[i]=Qdt[i]=Qbt_trans[i]=Qdt_trans[i]=Qgt_trans[i]=Qcoll[i]=0;
 i=(int) (radiation->theta/rad+0.5);
 if (i>90) i=90;
 if (i<0) i=-1;
 
 if (radiation->theta/rad<90) {
  Qgt[i]+=Igt*h;
  Qbt[i]+=radiation->Ibt*h;
  Qdt[i]+=radiation->Idt*h;
  Qbt_trans[i]+=Ibt_trans*h; 
  Qdt_trans[i]+=Idt_trans*h;
  Qgt_trans[i]+=h*F_str*tau_alpha0*(Ibt_trans+Idt_trans);
  Qcoll[i]+=(dQ_out-dQ_in);
 }
 
 if ( (info->day_of_year==365) && (info->tsim==(24*3600-h)) ) { /* last time step */
  FILE *file,*file2;
  char filename[20]="irradiation.dat";
  char filename2[30]="irradiation_integral.dat";
  double Qgt_int=0,Qbt_int=0,Qdt_int=0,Qbt_trans_int=0,Qdt_trans_int=0,Qgt_trans_int=0,Qcoll_int=0;

  if ( (file =fopen(filename,"w"))==NULL) { printf("file open error %s \n",filename ); exit (-1); };
  if ( (file2=fopen(filename2,"w"))==NULL) { printf("file open error %s \n",filename2); exit (-1); };
  fprintf(file,"#teta Qgt Qbt Qdt Qbt_trans Qdt_trans Qgt_trans integral\n");
  for (i=0;i<=90;i++) { 
    Qgt[i]/=(3600*1000); Qgt_int+=Qgt[i]; /* kWh */
    Qbt[i]/=(3600*1000); Qbt_int+=Qbt[i];
    Qdt[i]/=(3600*1000); Qdt_int+=Qdt[i];
    Qbt_trans[i]/=(3600*1000); Qbt_trans_int+=Qbt_trans[i];
    Qdt_trans[i]/=(3600*1000); Qdt_trans_int+=Qdt_trans[i];
    Qgt_trans[i]/=(3600*1000); Qgt_trans_int+=Qgt_trans[i];
    Qcoll[i]/=(3600*1000); Qcoll_int+=Qcoll[i];
    fprintf(file,"%d %.2f %.2f %.2f %.2f %.2f %.2f  \n"
    ,i,Qgt[i],Qbt[i],Qdt[i],Qbt_trans[i],Qdt_trans[i],Qgt_trans[i]); 
    fprintf(file2,"%d %.2f %.2f %.2f %.2f %.2f %.2f  \n"
    ,i,Qgt_int,Qbt_int,Qdt_int,Qbt_trans_int,Qdt_trans_int,Qgt_trans_int); 
  }
  fclose(file);
  fclose(file2);
 }
}



return (1);
} /* end collector() */
/*-------------------------------------------------------------------*/

