/* controller_x.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

/* #include "global.h"  */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


/*--------------------------------------------CONTROLLER_X  TYPE 26-----------*/
int controller_x( unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
               qp_sum,tsim,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */

{
/*  xfig.awk

    TYPE 26
    PRIORITY  3          used by config routine: controller priority
    HYDRAULIC 99         used by config routine: non hydraulic

    
    PAR1: 1 [] mode: 1 On&Off, 2:unisol, 3:deltaT_matched 4:CPCnew 
    PAR2: 3 [C] dT_on
    PAR3: 3 [C] dT_off
    PAR4: 1 [] kP proportional factor
    PAR5: 300 [] mode1&2:mp_max for pump,mode 3:deltaT || Tout_set(if>20)
    PAR6: 1e-4 [] kI for mode 3,4, 2:LowFlowFlag
    PAR7: 70 [] Tmax mode 4
    PAR8: 0.3 [] mode3: mp_min=par8*mp_max
    PAR9: 30 [C] mode 4:min. Temp. TSpo zum Heizen Zone

    INP1: 10 [C] mode 1,7 T_set, mode 2 Tkoll 
    INP2: 10 [C] mode 1 T_Sp, mode 2 TR1
    INP3: 10 [C] TVL for mixer, mode 4 , mode 8: On/Off
    INP4: 10 [C] TRL 
    INP5: 10 [C] Tamb

    OUT1: [] ctr 0..1 
    OUT2: [] mode1: state 1/0,ON/OFF; mode3:integr_dT ; 
    OUT3: [] no of swtch OFF/ON Mode 7: Heizen / Nicht heizen
    OUT4: [] ctr_mixer 
    
    OUT6: [Ws] system inlet energy 
    OUT7: [Ws] system outlet energy 
    OUT8: [Ws] internal energy 

*/

  int a=0; /*---------------------------analyse a=1--------------*/

    if (init_flag)
      {
        if (   ( (int) par[unit][1]<1)
	        || ( (int) par[unit][1]>4 ))
	      { printf(" mode controller u:%d wrong\n",unit); exit(-1);}
        if (    ((int) par[unit][1]==3)
	         && (      par[unit][8]>1 ) ) 
           { printf(" par8 controller u:%d wrong\n",unit); exit(-1);}

	    return 1; /*wait for outlet temperatures */
      }

if ((int) par[unit][1]==1)
  {
    if (  (out_o[  in_u[unit][1] ]   /* Sicherheitsvorkehrung */
                [ in_nr[unit][1] ] >110)  /* Tcoll */
        || (out_o[  in_u[unit][2] ]
	         [ in_nr[unit][2] ]>90)   /* Tstorage */
       )  
       { /* pumpe OFF */
       out_n[unit][1]=0;
       out_n[unit][2]=0;
       return (1);
       }
       
    if ( (out_o[unit][2] ==0) &&                  /*if state=0&&dT>=dT_on*/
         ( (out_o[  in_u[unit][1] ]
	         [ in_nr[unit][1] ]-out_o[  in_u[unit][2] ]
		  		         [ in_nr[unit][2] ])>=par[unit][2]) )
      { /*switch ON */
	if (a)
	printf("controller:temp1:%.4f\n"
	       "          temp2:%.4f\n"
	       "out_o[%d][2]:%.4f\n\n"
	       ,out_o[  in_u[unit][1] ]
		     [ in_nr[unit][1] ]
               ,out_o[  in_u[unit][2] ]
		     [ in_nr[unit][2] ]
               ,unit
	       ,out_o[unit][2]
		     );

	qp_sum[unit][1][1]+=1;
	out_n[unit][1]=1;
	out_n[unit][2]=1;
      }
    else { 
	   if ( (out_o[unit][2]==1) &&                  /*if state=1&&dT<=dT_off*/
	      ( (out_o[  in_u[unit][1] ]
		        [ in_nr[unit][1] ]-out_o[  in_u[unit][2] ]
		  			        [ in_nr[unit][2] ])
                  <=par[unit][3]) ) 
             { /* pumpe OFF */
	       out_n[unit][1]=0;
	       out_n[unit][2]=0;
             }
           else { /*no change */
		  out_n[unit][1]=out_o[unit][1];
		  out_n[unit][2]=out_o[unit][2];
                };
	 };

	if (a)
	printf("controller:temp1:%.4f\n"
	       "          temp2:%.4f\n"
	       "out_o[%d][2]:%.4f\n\n"
	       ,out_o[  in_u[unit][1] ]
		     [ in_nr[unit][1] ]
               ,out_o[  in_u[unit][2] ]
		     [ in_nr[unit][2] ]
               ,unit
	       ,out_o[unit][2]
		     );
  } /*--------------*/
  
  
  if ((int) par[unit][1]==2) /*----------------------mode 2 unisol ------------*/
  {
   double Tset_HW =out_o[  in_u[unit][1] ]      /* hot water 1*/
                        [ in_nr[unit][1] ];
   double Tmess_HW=out_o[  in_u[unit][2] ]      /* inp 2*/
                        [ in_nr[unit][2] ];     
   double TPO=     out_o[  in_u[unit][3] ]      /* inp3 top heating buffer */
                        [ in_nr[unit][3] ];
   double TPU=     out_o[  in_u[unit][4] ]      /* inp4: button buffer */
                        [ in_nr[unit][4] ];    
   double T_amb=   out_o[  in_u[unit][5] ]      /* inp5*/
                        [ in_nr[unit][5] ];   
   double T_CollVL=out_o[  in_u[unit][6] ]      /* inp6*/
                        [ in_nr[unit][6] ];   

   double hys_HW           =par[unit][2]; /* hot water hysterese */        
   double hys_heating      =par[unit][3]; /* heating hys */
   double T_heating_max    =par[unit][4]; /* T_VL max at -10 C */
   double T_heating_min    =par[unit][5]; /* T_VL_min if Tamb > 15 C*/
   int LowFlowFlag         =(int) par[unit][6]; /* =1 --> LowFlow */
   double Tset_modula;
   double ctr_unisol=2;
   double ctr_pump,ctr_heating,ctr_HW,T_heating_set;

   ctr_HW     =out_o[unit][4] ; /* get old state */
   ctr_heating=out_o[unit][5] ;
  
   T_heating_set=(T_heating_min-T_heating_max)*0.04*T_amb /* lin. in Tamb [-10,15] */
                 +T_heating_min*0.4+0.6*T_heating_max;
		 
   if (T_heating_set>T_heating_max)  T_heating_set=T_heating_max;
   if (T_heating_set<T_heating_min)  T_heating_set=10;
    
   if   (TPO < T_heating_set-hys_heating)              ctr_heating=1;
   if ( (TPO > T_heating_set) && (TPU>T_heating_set) ) ctr_heating=0;
   
   if   (Tmess_HW < Tset_HW-hys_HW)                  ctr_HW=1;
   if   (Tmess_HW > Tset_HW )                        ctr_HW=0;
   /* if ( (TPO > Tset_HW) && (Tmess_HW> Tset_HW) )    ctr_HW=0; */
   
   /* if (ctr_HW==1) printf("Tmess_HW:%.1f Tset_HW:%.1f TPO:%.1f\n"
                         ,Tmess_HW,Tset_HW,TPO);
    */
   
   if (ctr_HW==1) {                                  /* HW priority */ 
                  Tset_modula=Tset_HW+hys_HW; 
		  ctr_unisol=2;
		  ctr_pump=1;
		  }   
   else           { 
                  ctr_pump=0;
                  Tset_modula=10; 
		  ctr_unisol=4;  /* no change of inlet nodes */
		  if (ctr_heating==1) { 
		                      ctr_unisol=3; /* heating zone */
		                      ctr_pump=1;
		                      Tset_modula=T_heating_set+hys_heating;;
                                      }
		  }
   /* Low Flow control */
   if ( (LowFlowFlag) && (T_CollVL>TPO) ) ctr_unisol+=0.1; 

   out_n[unit][1]  = ctr_pump;
   out_n[unit][2]  = Tset_modula;
   out_n[unit][3]  = ctr_unisol;
   out_n[unit][4]  = ctr_HW;
   out_n[unit][5]  = ctr_heating;
   out_n[unit][9]  = T_heating_set;
   
   }
   
  /*----------------------mode 3 deltaT matched flow solar, CPC PARADIGMA old concet---*/
  if ((int) par[unit][1]==3) /*----------------------mode 3 matched ------------*/
  {
  double Tcoll     =out_o[  in_u[unit][1] ]      /* Tcoll, inp1*/
                         [ in_nr[unit][1] ];
  double Tmess     =out_o[  in_u[unit][2] ]
                         [ in_nr[unit][2] ];     /* inp2*/
  double TColdOutWT=out_o[  in_u[unit][3] ]
                         [ in_nr[unit][3] ];
		    
  double hys_high =par[unit][2];
  double hys_low  =par[unit][3];             /* par2   */
  double kP       =par[unit][4];
  double deltaTset=par[unit][5]; /* if par5>20 Tset=par5 control dT */
  double Tset     =par[unit][5];
  double kI       =par[unit][6];
  /* double Tmax     =par[unit][7]; / * Tmax Storage */
  double ctr_min  =par[unit][8]; /* 0..1  */
  double dT;                     /* Tcoll-Tmess-deltaTset */
  double integr_dT=out_o[unit][2];
  double ctr=0;
 
  if (Tset<20) Tset=0; /* use deltaT control, otherwise Tout fixed */
  
    if ( ( (Tcoll-Tmess)<hys_low ) || ((Tcoll-TColdOutWT)<hys_low ) )  
       {
       ctr=0;
	   out_n[unit][2] = out_o[unit][2]=0; /* reset integr_dT */
	   out_n[unit][2] = integr_dT =0;
       }	 
    else     
       {
       if (out_o[unit][1]>0) /* if pump was on */
         {
         if (Tset>0)  dT=Tcoll-Tset;           /* Tout fixed */
         else         dT=Tcoll-Tmess-deltaTset; /* deltaT control */
         if (fabs (integr_dT)<2000 ) 
              integr_dT+=dT*h;     /* discrete sum of the difference  */
         /* else integr_dT =0;        auto reset windup */
       
         /* ctr=ctr_old+kP*dT+kI*integr_dT; */
         ctr=kP*dT+kI*integr_dT;
         if (ctr<0)  { ctr=0;
                      integr_dT=0;
                     }
         if (ctr>1)             ctr=1;
	     if ( ctr<ctr_min) ctr=ctr_min; /* mp_min=par8*mp_max */
         if ( (ctr_min==0) || (ctr_min>1) )
            { printf("par8 controller unit:%d error\n",unit); exit(-1); }
         ctr=(double) ( (int) (10*ctr+0.5))/10; /* only 10 steps */

         if (   (ctr==0) && (Tcoll-Tmess>hys_low) ) ctr=ctr_min;
       
         /* stop pump to prevent overheating of storage */
         if ( ( Tcoll >110) || ( Tmess>90 ) )  ctr=0;   
       
          }
	else /* Pump was OFF */
	     if ((Tcoll-Tmess)>hys_high) ctr=ctr_min;
	     else                        ctr=0.0;
        }

   out_n[unit][1]  = ctr;
   out_n[unit][2] = integr_dT; /* save out2 non discrete state */
   out_n[unit][3]  = ctr; /* sek & prim pump , same control */
  }

   
  /*----------------------mode 4 PARADIGMA CPC CONTROL ---------------------*/
  if ((int) par[unit][1]==4) /*------------*/
  {
  int a=0; /* debug flag */
  double TSA=out_o[  in_u[unit][1] ]     
                  [ in_nr[unit][1] ];   /* Solar Austritt */
  double TWU=out_o[  in_u[unit][2] ]
                  [ in_nr[unit][2] ];   /* WW unten */
  double TSV=out_o[  in_u[unit][3] ]
                  [ in_nr[unit][3] ];   /* Solar Vorlauf WTsek */
  double TSE=out_o[  in_u[unit][4] ]
                  [ in_nr[unit][4] ];   /* Solar Eintritt  */

  double hys_ON    =  par[unit][2];     /* Hysterese , Einschaltbedingung */
  double kP        =  par[unit][4];
  double Tset      =  par[unit][5];
  double kI        =  par[unit][6];
  double Tmax      =  par[unit][7];  /*Tmax Storage */
  double ctr_min   =  par[unit][8];  /* 0..1 */

  double ctr_prim       =out_o[unit][1];
  double integr_dT      =out_o[unit][2];
  double ctr_sek        =out_o[unit][3];
  double x              =out_o[unit][4];
  double time_to_stay_ON=out_o[unit][5];

  double EinschaltKriterium;
  int primON,sekON;
  
  if (ctr_prim>0) primON=1;
  else            primON=0;

  if (ctr_sek >0)  sekON=1;
  else             sekON=0;
if ( sekON || primON) a=0;

  EinschaltKriterium=TSA-(TWU+hys_ON);

  if ( (TSA>30) && !primON && (x==0) ) 
     {
     time_to_stay_ON=20+h; /* 20 sec */
     primON=1;
     }

  if (time_to_stay_ON > 0) 
     { 
     double m=-30*60/30; /* steigung: time_to_stay_OFF/EinschaltKriterium */
     if ( TSA                  >= 30 )   x=30*60; /* time_to_stay_OFF in sec  */
     if (  (m*EinschaltKriterium<  x )  
         &&(  EinschaltKriterium<  0 ) ) x=m*EinschaltKriterium; 
     }
     
  if ( !primON && (x>0) ) x-=h; /* count down time_to_stay_OFF */
  if (             x<0)   x =0;
  if ( primON && (time_to_stay_ON>0) ) time_to_stay_ON-=h;
  if (            time_to_stay_ON<0)   time_to_stay_ON =0;
 
  /* absolutes EinschaltKriterium */
  if ( (TSA>20) && (TSA>(TWU+10)) ) {
                                    primON=1; 
                                    x=0;
                                    time_to_stay_ON=0;
                                    }

  /* absolutes EinschaltKriterium Sek Pumpe */
  if ( (TSE>(TWU+5)) && (!sekON) )          { sekON=1 ; if (a) printf("\n\nsekON !!!!\n"); }
  if ( (TSV<(TWU+5)) && ( out_o[unit][3]>0)  
                     && ( TSE<(TWU+8)     ) 
                     && (TSE<(TWU+5)      ) ) sekON=0;


  /* PI Control for sek Pump */
  if (sekON)
    {
    double dT; /* temperature difference */
    if (TWU>Tset) Tset=Tmax; /* switch Tset to Tmax */

    if (out_o[unit][3]>0) /* if pump sek was on */
      {
      dT=TSV-Tset;           /* Tout fixed */
      if (fabs (integr_dT)<2000 ) integr_dT+=dT*h; 
      /* else integr_dT =0;        auto reset windup */

      ctr_sek=kP*dT+kI*integr_dT;
      if (ctr_sek<0)  { ctr_sek=0;
                      integr_dT=0;
                    }
      if ( ctr_sek>1)       ctr_sek=1;
      if ( ctr_sek<ctr_min) ctr_sek=ctr_min; 
      if ( (ctr_min==0) || (ctr_min>1) )
         { printf("par8 controller unit:%d error\n",unit); exit(-1); }
      ctr_sek=(double) ( (int) (10*ctr_sek+0.5))/10; /* only 10 steps */
      }
     else ctr_sek=ctr_min;
    }
  else
    {
    ctr_sek=0; 
    out_n[unit][2] = out_o[unit][2]=0; /* reset integr_dT */
    out_n[unit][2] = integr_dT =0;
    }

  /* prim pump switch OFF */
       /* if sek switched off &&!EinschaltKrit then prim off  */
  if (    (!sekON) 
       && (out_o[unit][3]>0) 
       && (EinschaltKriterium<0) ) primON=0; 
  if (    (time_to_stay_ON==0) 
       && primON 
       && (EinschaltKriterium<0) ) primON=0;

  if (primON) ctr_prim=1;
  else        ctr_prim=0;
  /* stop pump to prevent overheating of storage */
  if (   (TSA>120)
      || (TSV>Tmax+5)
      || (TWU>Tmax)
     )
     ctr_prim=0;
if ( sekON || primON || x ) a=0;
  if (a) printf("primON:%d sekON:%d x:%.2f dT+hys:%.2f ti_to_st_ON:%.2f "
                "TSA:%.1f TWU:%.1f TSE:%.1f TSV:%.1f\n"
                ,primON,sekON,x,EinschaltKriterium,time_to_stay_ON,TSA,TWU,TSE,TSV);
  
  out_n[unit][1]  = ctr_prim;
  out_n[unit][2]  = integr_dT; /* save out2 non discrete state */
  out_n[unit][3]  = ctr_sek;
  out_n[unit][4]  = x; /* keep time_to_stay_OFF */
  out_n[unit][5]  = time_to_stay_ON;
  

  return 1;
  }


  
/*-----------------------------------outputs-------------------------------*/
 /*
   outputs are set in each mode !! 
   IMPORTANT: don't use out_n[unit][6] , 7 ,8 : 
              they are used for system-energy 
 */

 return (-1);
}

