/* diverter_air.c */
/*       10        20        30        40        50        60        70      */
/*345678901234567890123456789012345678901234567890123456789012345678901234567*/
/*---------------------------------------------------------------------------*/


#include "global.h"
#include "thermodynamics.h"

#include <stdio.h>
#include <stdlib.h>

/* 
000411 max includes hydraulic/pneumatic functions
000906 out_n[unit][20]=out_o[unit][20]=out_o[  in_u[unit][1] ][20];
010427 chriwi: some syntax mod
010627 include thermodynamics.h

*/

/* ----------------------------------DIVERTER_AIR--TYPE 47  ---------------*/

int diverter_air(unit,par,in_u,in_nr,out_o,out_n,init_flag,select_unit)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           int    init_flag; /* set at initial loop of sim            */
           int    *select_unit; /* number of calls in this timestep   */
{
/*  etc/xfig.awk 
  
    TYPE      47
    PRIORITY  6          used by config routine
    HYDRAULIC 47         diverter air              
     
  
    PAR1: 1 [] if 1 then ctr from airdraulic calculation
    PAR2: 1e5 [] cetaA2 DivIn
    PAR3: 1e5 [] cetaA2 DivO
    PAR4: -1  [] a1 quadratic DivIn0
    PAR5: 0   [] a2 linear DivIn0
    PAR6: 1   [] a3 offset DivIn0
    PAR7: 1   [] b1
    PAR8: 0   [] b2
    PAR9: 0   [] b3
 
    INP1: 0 [Ws]   `Q_in' input heat capacity 
    INP2: 0 [kg/s] `mp_dry_in' mass stream dry air
    INP3: 0 [kg/s] `mp_steam_in'  mass stream steam
    INP4: 0 [kg/s] `mp_water_in'  mass stream water
    INP5: 0 [kg/s] ` mp_co2_in'   mass stream co2
    INP6: 0 [] `ctr': Control signal:mp1_out=ctr*mp_in
    INP7: 0 [] Hub 0:mp0_out =mp 1:mp1_out
  
    OUT1: [Ws] `Q1_out' output heat capacity 
    OUT2: [kg/s] `mp1_dry_out' mass stream dry air into branch 1
    OUT3: [kg/s] `mp1_steam_out' mass stream steam into branch 1
    OUT4: [kg/s] `mp1_water_out' mass stream water into branch 1
    OUT5: [kg/s] `mp2_co2_out'   mass stream co2 into branch 1
 
    OUT6: [Ws] system inlet energy 
    OUT7: [Ws] system outlet energy 
    OUT8: [Ws] internal energy 

    OUT9: [Ws] `Q2_out' output heat capacity 
    OUT10: [kg/s] `mp2_dry_out' mass stream dry air into branch 2
    OUT11: [kg/s] `mp2_steam_out' mass stream steam into branch 2
    OUT12: [kg/s] `mp2_water_out' mass stream water into branch 2
    OUT13: []     `mp2_co2'       mass stream co2   into branch 2
    OUT14: [] `hydr_ctr' hand over hydr_in to mixer
    OUT21: [] `ceta0' Ceta0(ctr)
    OUT22: [] `ceta1' Ceta1(ctr) 
*/
int a=0;  /*debug with a=1*/
int DEBUG=0; /*0:off 1:check values*/
/*___ ZUORDNUNGEN __________________________________________________________*/
double Q1_out, mp1_dry_out, mp1_steam_out, mp1_water_out,
       Q2_out, mp2_dry_out, mp2_steam_out, mp2_water_out;

/*___ SET PARAMETER ________________________________________________________*/

/*_____ AIRDRAULIC ________________________________________________________*/
double hub,a1,a2,a3,b1,b2,b3,hilfa,hilfb;
/*_____ SET INPUTS _________________________________________________________*/
double Q_in        = out_n IN (1);          /*[Ws] input heat capacity*/
double mp_dry_in   = out_n IN (2);          /*[kg/s] mass stream dry air*/
double mp_steam_in = out_n IN (3);          /*[kg/s] mass stream steam*/
double mp_water_in = out_n IN (4);          /*[kg/s] mass stream water*/
double mp_co2_in   = out_n IN (5);          /*[kg/s] mass stream co2 */
double mode_div    = out_n IN (6);          /* Control signal */
                          
/*_____ CONTROL INPUT AND SET ERRORS ______________________________________*/
if( DEBUG )
  { if( (mp_dry_in<0) || (mp_steam_in<0) || (mp_water_in<0) )
      { printf("DIVERTER:%d error mp_in negativ: %.1f %.1f %.1f\n",unit,
        mp_dry_in ,mp_steam_in,mp_water_in);  exit (-1); }    
  } 

/*_____ INIT ______________________________________________________________*/
if(init_flag)
  { if( in_u[unit][1] ==unit )     /*control for hydraulic circle*/
       printf("MIXER unit%d out:1 no connection for hydraulic calculation\n",
               unit);
    
    if( (mode_div <0) || (mode_div >1) ) 
      { printf("DIV:%d PAR1 mode_div =%f must be between 0<= mode_div <=1\n",
                unit ,mode_div);    exit (-1); }
  }

/* check first time call in this timestep */
if (select_unit[unit]==0) { select_unit[unit]++; 
                           if (a) printf("diverter_air[%d] 1st time \n",unit); 
                          }
else   { if (a) printf("diverter_air[%d] 2nd time select_unit:%d\n",
                        unit,select_unit[unit]); return (1);   }

/*_____ START CALCULATION __________________________________________________*/

/*_____ CALCULATION OF OUTPUTS 1 __________________________________________ */
Q1_out =mode_div *Q_in;                      /*[Ws] output 1 heat capacity*/
mp1_dry_out =mode_div *mp_dry_in;            /*[kg/s] mass stream 1 dry air*/
mp1_steam_out =mode_div *mp_steam_in;        /*[kg/s] mass strea m1 steam*/
mp1_water_out =mode_div *mp_water_in;        /*[kg/s] mass stream 1 water*/

/*_____  CALCULATION OUTPUTS 2 ______________________________________________*/
Q2_out =Q_in -Q1_out;                        /*[Ws] output 2 heat capacity*/
mp2_dry_out =mp_dry_in -mp1_dry_out;         /*[kg/s] mass stream 2 dry air*/
mp2_steam_out =mp_steam_in -mp1_steam_out;   /*[kg/s] mass stream 2 steam*/
mp2_water_out =mp_water_in -mp1_water_out;   /*[kg/s] mass stream 2 water*/

/*----- END CALCULATION ----------------------------------------------------*/

if (a) printf("div_air:%d Q_in:%.2f Q_out1:%.2f Q2_out:%.2f\n"
       ,unit,Q_in,Q1_out,Q2_out);
/*_____ SET OUTPUTS ________________________________________________________*/

OUT (1)  = Q1_out;                   /*[Ws] output 1 heat capacity*/
OUT (2)  = mp1_dry_out;              /*[kg/s] mass stream 1 dry air*/
OUT (3)  = mp1_steam_out;            /*[kg/s] mass stream 1 steam*/
OUT (4)  = mp1_water_out;            /*[kg/s] mass stream 1 water*/
OUT (5)  = mp_co2_in*mode_div;       /*[kg/s] mass stream 1 co2 */
OUT (9)  = Q2_out;                   /*[Ws] output 2 heat capacity*/
OUT (10) = mp2_dry_out;              /*[kg/s] mass stream 2 dry air*/
OUT (11) = mp2_steam_out;            /*[kg/s] mass stream 2 steam*/
OUT (12) = mp2_water_out;            /*[kg/s] mass stream 2 water*/
OUT (13) = mp_co2_in-OUT (5); /*[kg/s] mass stream 2 co2*/    
OUT (14) = mp1_steam_out;            /*hydr_ctr; */


/*_____ CONTROL OUTPUT AND SET ERRORS ______________________________________*/
if( DEBUG )
  {if( (mp1_dry_out<0) || (mp1_steam_out<0) || (mp1_water_out<0) )
      { printf("DIVERTER:%d  mp1_out error:%.1e %.1e %.1e \n",
                unit,mp1_dry_out,mp1_steam_out,mp1_water_out); exit(-1); }
   if( (mp2_dry_out<0) || (mp2_steam_out<0) || (mp2_water_out<0) || (Q2_out<0))
      { printf("DIVERTER:%d  mp2_out error %.1e %.1e %.1e\n",
                unit,mp2_dry_out,mp2_steam_out,mp2_water_out); exit(-1); }
  }
 
if(a) printf("DIVERTER   mp_dry_in:%.1f      mp_steam_in:%.1f      Q_in:%.1f\n"
       "          mp1_dry_out:%.1f    mp1_steam_out:%.1f    Q1_out:%.1f\n"
       "          mp2_dry_out:%.1f    mp2_steam_out:%.1f     Q2_out:%.1f\n\n",
      mp_dry_in*3600,mp_steam_in*3600,Q_in,mp1_dry_out*3600,mp1_steam_out*3600,
       Q1_out, mp2_dry_out*3600,mp2_steam_out*3600,Q2_out);  

/*_________________________________________________________*/
if (PAR (1)==1) /*hydraulic solve*/ 
 {
    a1=PAR (4);
    a2=PAR (5);
    a3=PAR (6);
    b1=PAR (7);
    b2=PAR (8);
    b3=PAR (9);
    hub=out_n IN (7);
  hilfb=b1*hub*hub+b2*hub+b3;
 OUT (21)=PAR (3)/((hilfb*hilfb)+1.0e-7);/*hub=1 ceta_mp1=infinity 
                                            hub=0  mp0=mp_in    */

 hilfa=(a1*hub*hub+a2*hub+a3);
 OUT (22)=PAR (3)/((hilfa*hilfa)+1.0e-7);/*hub=0 ceta_mp0=infinity 
                                            hub=1  mp1=mp_in    */
 printf("hub von unit[%d][%d] \n",in_u[unit][6],in_nr[unit][6]);
 printf(
 "\nAirdraulicSolve in diverter_Air hub%2.2f ceta0 %6.1f ceta1 %6.1f ctr %3.3f"
 ,hub,OUT (21),OUT (22),mode_div);

 }
return 1;
} /*end diverter_air */



