/* equation.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

/* #include "global.h" */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define IN_MAX 10

/* -----------------------------------------------------TYPE 27 ----------------*/
int equation_x(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
             qp_sum,tsim,init_flag,day_of_year)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int    day_of_year;
{
/*  
    TYPE 27
    PRIORITY  2          used by config routine
    HYDRAULIC 99         used by config routine

  
    PAR1:  [] equation-set 0...99
    PAR2: 4.19 [kJ/kgK] mode 8
    PAR4: 10000 [W] mode 6:P Zusatzheizung
    PAR5: 0.7 [] Steilheit 10/-15
    PAR6: 0.8 [] etaWT
    PAR7: 20 [] Tset Zone

    INP1: 0 [] inp1 mode 6: ctr Pzuheiz ,mode 7: Tamb, mode 8: Tset [C]
    INP2: 1000 [W] inp2 mode6:  zustzliche Wrmequellen (Licht...), mode 8: Tist [C]
    INP3: 0 [Ws] inp3 mode 6: Qsol, mode 8: mp [kg/h]
    INP4: 0 [W] inp4 
  
    OUT1: [] equation1 mode 6: Pheiz. mode 7: Tamb_mean, mode 8: Paux[kW]
    OUT2: [] equation2
    OUT3: [] equation3
    OUT4: [] equation4
    OUT6: [Ws] system inlet energy 
    OUT7: [Ws] system outlet energy 
    OUT8: [Ws] internal energy 
    OUT9: []  equation5
    
*/
  int i;
  double input[IN_MAX+1];
  
  if (init_flag)
    {
      for (i=1;i<=IN_MAX;i++) out_n[unit][i]=out_o[unit][i]=0;
    }


  for (i=1;i<=IN_MAX;i++) input[i]=out_n[  in_u[unit][i] ]
                                        [ in_nr[unit][i] ];

					
  /*--EQUATIONS--------------------------------------------------------*/
  
  switch ( (int) par[unit][1] ) /* equation-set 0...99*/
   {
   case 6:    
      {
        double Psum=0;  /* Summe der Leistungen*/

        if (input[1] == 1) Psum += par[unit][4];
        Psum += input[2];    /* P von Waermequelle(Licht etc) */
        Psum += input[3]/h;    /* P von SolarHeizung, inp 3 ist Energie [Ws] */
        Psum += input[4];    /* P Speicherverluste */
        out_n[unit][1]=Psum/1000; /*Pheiz in kW */
        break;
      } 
   case 7:     /* calculates mean of input  of day before */
      {
        static double count=0;
        static double sum=0, mean_o;
        static int day_of_year_old;
        int a = 0;


        if (init_flag)
        {
          mean_o  = input[1];     /* Initialisierung mit aktuellem Wert */
          day_of_year_old=day_of_year;
        }
        if (day_of_year_old != day_of_year)     /* New day */
        {
           day_of_year_old = day_of_year;
           mean_o  = sum/count;
           if (a) printf ("mean of last day: %f, Summe: %f, count: %f\n\n", mean_o, sum, count);
           sum=0;
           count=0;
        }
        sum+=input[1];
        count++;
        if (a) printf ("sum = %f, count = %f, input[1]= %f, mean: %f\n", sum, count, input[1], sum/count);

        out_n[unit][1] = mean_o;
        break;
      }
      case 8:     /* required Paux to heat HW to Tset (Durchlauferhitzer) */
      {
        double Paux;
        int a=0;   /* analayse with a=1 */

        if (input[2]>=input[1])
           Paux = 0;  /*no heating */
        else
           Paux=input[3]*par[unit][2]*
               (input[1]-input[2]); /* P=mp*cp*(Tset-Tist) */
        out_n[unit][1]=Paux/3600;               /* Paux in kW */
       if (a) printf (" Tset: %f, Tist: %f, mp: %f, Paux: %f\n"
                        ,input[1],input[2], input[3], out_n[unit][1]);
       break;
      }
   default: 
      printf("equation-set unit:%d not valid\n",unit);
      exit (-1);
   }
  
  return 1;
}


