/* heating.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include "global.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*
020528 werners import ...

*/

/*-----------HEATING --TYPE8--------------------------------------*/

int heating(unit,par,in_u,in_nr,out_n,out_o,deriv_o,deriv_n,h,
            init_energy,qp_sum,tsim,init_flag,Q,
            un_max)
         int unit;
         double **par;
         int **in_u;
         int **in_nr;
         double **out_n;
         double **out_o;
         double **deriv_o;
         double **deriv_n;
         double h;
         double **init_energy;
         double ***qp_sum;
         double tsim;
         int init_flag;
         double **Q;
         int un_max;

{
/* 

   TYPE 08
   PRIORITY  6          used by config routine
   HYDRAULIC 1          1-pole fluid

   
   PAR1: 1 [] mode: -1:make fit; 0:use fit; 1:typ.Roehrenradiator; 2:typ.Konvektor; 3:typ.Fussbodenheizung; 4:typ.Plattenheizung(z.B Deckenstrahlplatten) 
   PAR2: 3 [m^2] area of heating
   PAR3: 36 [kg/m^2] Gewicht des Heizkoerpers pro m^2 (Leergewicht) 
   PAR4: 21 [l/m^2] Volumeninhalt Wasser pro m^2 Heizflaeche
   PAR5: 4.19 [kJ/kgK] cp of fluid
   PAR6: 0.456 [kJ/kgK] cp of iron
   PAR7: 1624 [W] Qn pro m^2 (need for mode -1) (Nennleistungsabgabe bei Nennbetrieb dTn (PAR7))
   PAR8: 50 [K]  dTn (need for mode -1) (Nennbetrieb (Bsp.:75/65/20)) 
   PAR9: 1.3 [m] (need for mode -1) Heizkoerperpotenzfaktor n
   PAR10: 10 [] n nodes
   PAR11: 20.381 [] fit a (need for mode 0) Qp=a*dT+b*dT*dT(Kollektorkennlinie) 
   PAR12: 0.234 [] fit b (need for mode 0)

   INP1: 0 [Ws] deltaQ-inlet
   INP2: 0 [kg/s] mp-inlet 
   INP3: 20.0 [C] ambient temperature for convection
   INP4: 18.0 [C] ambient temperature for radiation 
   INP5: 0.8 [] IR-emissivity of walls (not yet used)

   OUT1: [Ws] deltaQ-outlet
   OUT2: [kg/s] mp-outlet
   OUT3: [C] temperature outlet
   OUT4: [Ws] outlet energy convective
   OUT5: [Ws] outlet energy radiative
   OUT6: [Ws] system inlet energy 
   OUT7: [Ws] system outlet energy 
   OUT8: [Ws] internal energy 
   OUT9: [] i_dead_nr 
   OUT10: [W] P heat
   OUT11: [] T_sensor par17 
   OUT15: [] Tmean nodes 
   OUT16: [] kA_node_conv
   OUT17: [] kA_node_rad
   OUT18: [] T_inlet

   out(OUTMAX0+1): T_KW_in, if input1=[0,0] input initial
   out(OUTMAX0+2): mp_KW  , if input1=[0,0] input initial
   ..

   DERIV1: 10.0 [C] temperature of first node
                    init_energy 1=deriv1/mc
   DERIV5: [] node 5 lowest
   .
   deriv(un_max+nr): wt1 temp

*/
int a=0; /*-------------ANALYSE a=1 ---------*/

double cp_f= PAR (5)*1e+3; /* cp of fluid*/
double cp_i= PAR (6)*1e+3; /* cp of iron */
double mc_total; 
double Qout; 
double Tout;
double mp_cp_h;  
  
double Tin,deltaTin;
int i,ret;

double k_node_conv;      /* 'a' Qp=a*dT+b*dT*dT */
double k_node_rad ;      /* 'b' Qp=a*dT+b*dT*dT */
double kA_node_conv;      /* 'a' Qp=a*dT+b*dT*dT */
double kA_node_rad ;      /* 'b' Qp=a*dT+b*dT*dT */
double energy[NODE_MAX];
double Qi[NODE_MAX][10];  

double Qin         =out_n IN(1);  /* current timestep */
double mp          =out_n IN(2); /* [kg/s] */
double Tamb        =out_o IN(3); /* air temperature of connecting room */
double Tstar       =out_o IN(4); /* star temperature of connecting room*/
/* werner -> */
int n = (int) PAR (10); /* n nodes */
double mc_n,Tmean,T_tmp=10;
double energy_with_plug;
double kA_node_conv_n,kA_node_rad_n;
double Qn,dTn,hoch_n,erg;
int mode= PAR (1);
FILE *datFILE;
FILE *gnuFILE;
/* <- werner */

mc_total=(PAR (4)*cp_f+PAR (3)*cp_i)*PAR (2); 
        if (a) printf("mc_total: %.4f \n",mc_total);
mc_n=mc_total/n;
mp_cp_h=mp*cp_f*h; /* input-plug */
 if (mp_cp_h!=0) Tin=Qin/mp_cp_h;
    else            Tin=deriv_o[unit][1];
deltaTin=Tin-Tamb;   /* temp convect */
  

switch (mode)
{
 case -1:
  Qn     = PAR (7);
  dTn    = PAR (8);
  hoch_n = PAR (9);
  datFILE=fopen("heizkoerperfit.dat","w" );
  for (i=0;i<7;i++)
   {
    erg=pow((i*10)/dTn,hoch_n)*Qn;
    fprintf(datFILE, "%d  %.1f \n",i*10,erg); 
   }
  gnuFILE=fopen("heizkoerperfit.gnu","w" );
  fprintf(gnuFILE,"g(x)=a*x+b*x*x\n");
  fprintf(gnuFILE,"fit g(x) 'heizkoerperfit.dat'us 1:2 via a,b\n");
  fprintf(gnuFILE,"set grid\nset key below\nset nolabel\n"
                  "set xlabel 'Uebertemperatur zum Raum in [C]  \n"
                  "set ylabel 'Heizleistung pro m^2 Heizflche in [W] \n");
  fprintf(gnuFILE,"plot g(x) ti 'fit a*x+b*x*x (Kollektorkennlinie)',"
                  "'heizkoerperfit.dat' us 1:2 ti 'nach Recknagel' w l,"
                  "g(x)-(a*x) ti 'only radiation' \n");
  fprintf(gnuFILE,"show variables\n"
           "print'now take Variable a for (PAR11) and Variable b for (PAR12)'\n"
           "print'and put it in your info_edit window of heating,'\n"
           "print'then change PAR1-PAR4 to right values.'\n"
           "set out 'heizkoerperfit.ps'\n"
           "set term post color solid\n" 
           "rep\n");

  ret=system("gnuplot -persist heizkoerperfit.gnu & \n");


  printf("heating[%d] mode %d: make fit; not aktive yet  exit\n",unit,mode);  
  exit(-1);
 break;


 case 0:
  k_node_conv = PAR (11); /* value due to one squaremeter*/
  kA_node_conv=k_node_conv*PAR (2);
  kA_node_conv_n=kA_node_conv/n;

  k_node_rad = PAR (12); /* value due to one squaremeter*/
  kA_node_rad =k_node_rad *PAR (2);
  kA_node_rad_n=kA_node_rad/n;
 break;


 case 1:
  /* Typ Roehrenradiator von Zehnder 2Saeuler Modell Charlston        */
  /* siehe Datei auch "./fit_roehrenradiator/heizkct.txt"             */
  /* Modell  H    N   T  A/El.  M/El.  V/El.  SK  qms  Exp. P (EN442) */
  /*         mm  mm  mm   m2      kg    dm3    %  kg/h  n   W bei 50K */
  /* 2075   742 684  62 0.113   1.23   0.72    22 4.8  1.3  56        */
  /* Elementabtsand : 46mm                                            */
  /* -->Umschliessungsflaeche dieses elementes 0,742*0,046=0.0341m^2  */
  /*    -> FAKTOR=29.33                                               */
  /* k-Werte bestimmt aus fit siehe fitting.gnu                       */
  /* Werte fr fit: PAR3:    36                                       */
  /*                PAR4:    21                                       */
  /*                PAR7:  1624                                       */
  /*                PAR8:    50                                       */
  /*                PAR9:     1.3                                     */


  k_node_conv=20.3811; /* value due to one squaremeter*/
  kA_node_conv=k_node_conv*PAR (2);
  kA_node_conv_n=kA_node_conv/n;

  k_node_rad =0.234766; /* value due to one squaremeter*/
  kA_node_rad =k_node_rad *PAR (2);
  kA_node_rad_n=kA_node_rad/n;
 break;


 case 2:
  /* Typ Konvektor von Zehnder 2Saeuler Modell Radiavektor            */
  /* siehe Datei auch "./fit_konvektor/heizkktd.txt                   */
  /* werte fr laufenden Meter                                        */
  /*Modell H   N(1/2+3/8") T  A/El. M/El. V/El. SK qms  Exp. P(EN442) */
  /*       mm  mm          mm m2    kg    dm3   %  kg/h  n   W bei 50K*/
  /* 213  210 174         73  3.28  17.25  3.6  14  60  1.3  696      */
  /*                                                                  */
  /* --> Umschliessungsflaeche 0,21*1,000=0.21m^2   -> FAKTOR=4.762   */
  /* k-Werte bestimmt aus fit siehe heizkoerperfit.gnu                */
  /* Werte fr fit: PAR3:   82.14                                     */
  /*                PAR4:    3.43                                     */
  /*                PAR7: 3314.29                                     */
  /*                PAR8:   50                                        */
  /*                PAR9:    1.3                                      */
  k_node_conv=42.2989; /* value due to one squaremeter*/
  kA_node_conv=k_node_conv*PAR (2);
  kA_node_conv_n=kA_node_conv/n;

  k_node_rad =0.470378; /* value due to one squaremeter*/
  kA_node_rad =k_node_rad *PAR (2);
  kA_node_rad_n=kA_node_rad/n;
 break;


 case 3:
  printf("case Fussbodenheizung; not aktive yet ");exit(-1);
 break;


 case 4:
  printf("case Plattenheizung; not aktive yet ");exit(-1);
 break;
}
	      

/*---------- Initialisation------------------------ */
if (init_flag)
 {

 if(mc_total<=0)
   {
    printf("exit heating,caused by wrong Parameters\n");
    exit(-1);
   }
 if(deriv_o[unit][1]==0)
    {
     printf(" set deriv1 of heating unit:%d",unit);
     exit(-1);
    }
 init_energy[unit][1]=deriv_n[unit][1]*mc_total;
 printf("heating unit: %d  init_energy: %f \n",unit,init_energy[unit][1]);
/* werner -> */
 for (i=1;i<=n;i++)
     {
     deriv_o[unit][i]=deriv_n[unit][i]=deriv_o[unit][1];
     init_energy[unit][i]=deriv_n[unit][i]*mc_n;
     }
 if(Tamb<=0)  Tamb=20;  
 if(Tstar<=0) Tstar=18;      
 printf(" use mode %d in heating unit %d\n",mode,unit); 
/* <- werner */
 };
/*---------- end of init ------------------------ */

/* werner -> */
/* calc heattransfer through n nodes */
for (i=1;i<=n;i+=1)
     {
     if (i==1)     Qi[i][1]=Qin;
     else          Qi[i][1]=mp_cp_h*T_tmp; /* energy plug from previous node */
     energy_with_plug=Qi[i][1]+mc_n*deriv_o[unit][i];  /* get new mixed node */
     T_tmp=energy_with_plug/(mp_cp_h+mc_n);    /* get temperary temp of node */
     if (a) printf("energy_with_plug:%.2f\n",energy_with_plug);
     if (a) printf("1: T_tmp:%.2f  Tair:%.2f Tstar:%.2f"
                   "   Qin:%.2f  mp:%.2f\n"
                   ,T_tmp,Tamb,Tstar,Qin,mp);
     Qi[i][2]=-kA_node_conv_n*h*(T_tmp-Tamb);
     if(T_tmp>=Tstar) Qi[i][3]=-kA_node_rad_n*h*(T_tmp-Tstar)*(T_tmp-Tstar);
       else  Qi[i][3]=kA_node_rad_n*h*(T_tmp-Tstar)*(T_tmp-Tstar); 
     T_tmp=(energy_with_plug+Qi[i][2]+Qi[i][3])/(mp_cp_h+mc_n);
     if (a) printf("2: T_tmp:%.2f\n",T_tmp);
     qp_sum[unit][i][1]+=Qi[i][1];
     qp_sum[unit][i][2]+=Qi[i][2];
     qp_sum[unit][i][3]+=Qi[i][3];
     if (a) printf("energy:%.1f Qi[%d][1]:%.2f Qi[%d][2]:%.2f Qi[%d][3]:%.2f\n"
                   ,init_energy[unit][i],i,Qi[i][1],i,Qi[i][2],i,Qi[i][3]);
     if (a) printf("deriv_o[unit][%d]:%.2f deriv_o[unit][]:%.2f mp_cp_h:%.2f \n"
                  ,i,deriv_o[unit][i],deriv_o[unit][i-1],mp_cp_h);

     }
  Qi[n+1][1]=Qout=mp_cp_h*T_tmp; /* last element */
  qp_sum[unit][n+1][1]+=Qi[n+1][1];
  Tmean=0;
   for (i=1;i<=n;i+=1)
     {
      energy[i]=init_energy[unit][i]
                    +qp_sum[unit][i  ][1]       /* Q_in fluid */
                    +qp_sum[unit][i  ][2]       /* Q_conv to zone */
                    +qp_sum[unit][i  ][3]       /* Q_rad  to zone */
                    -qp_sum[unit][i+1][1];      /* Q_out fluid */
      deriv_n[unit][i]=energy[i]/mc_n;
      Tmean+=deriv_n[unit][i]/( (double) n);
      if (a) printf("deriv_n[unit][%d]:%.2f\n",i,deriv_n[unit][i]);
     }
    Tout=deriv_n[unit][n];   
    if (a) printf("Qin:%.4f mp_cp_h:%.4f Qout:%.4f\n",Qin,mp_cp_h,Qout); 
/* <- werner */

/*internal energy check */
/*  if (1) printf(init_energy
    init_energy[unit][i]=deriv_n[unit][i]*mc_n;
*/
  /*--------------set outputs----------------------*/
  out_n[unit][1]=Qout; 
if (a) printf("#######tsim:%.0f heating[%d]: Qin:%.1f Qout:%.1f"
              " Tin:%.1f Tout:%.1f\n"
              ,tsim,unit,Qin,Qout,Tin,Tout);
  out_n[unit][2]=mp;  

  out_n[unit][3]=Tout; 

  out_n[unit][4]=out_n[unit][5]=0;
  for (i=1;i<=n;i++) out_n[unit][4]-=Qi[i][2];   /* Q_conv to zone */
  for (i=1;i<=n;i++) out_n[unit][5]-=Qi[i][3];   /* Q_rad  to zone */
  if (a) printf("(heating) out4,5 of unit %d  Q_conv:%.4f  Q_rad:%.4f  "
                "Q_conv+Q_rad:%.4f\n",unit ,out_n[unit][4] 
                ,out_n[unit][5],out_n[unit][4]+out_n[unit][5]);
  /*System Energy Balance */
  out_n[unit][6]=0; /* gains */
  out_n[unit][7]=0;/* losses *//*out_n[unit][7]=out_n[unit][4]+out_n[unit][5];*/
  out_n[unit][8]=0; for (i=1;i<=n;i++) out_n[unit][8]+=energy[i]; /* cap */
#if 0
  out_n[unit][8]=out_n[unit][8]+out_n[unit][4]  /* Q_amb_conv */
                               +out_n[unit][5]; /* Q_amb_rad  */
#endif
  if (a) printf("(heating) out6,7,8 of unit %d  :%.4f  :%.4f  %.4f\n"
         ,unit ,out_n[unit][6] ,out_n[unit][7] ,out_n[unit][8]);
  /* P_heat */
  out_n[unit][10]=(Qin-Qout)/h;
  /* T_WW */
  out_n[unit][11]=deriv_n[unit][ (int) par[unit][12] ]; /* Tsensor1: node 1 T_WW */
  out_n[unit][15]=Tmean;

  /* kA_node_rad,kA_node_conv */
  out_n[unit][16]=kA_node_conv;
  out_n[unit][17]=kA_node_rad ;

  out_n[unit][18]=Tin ;
/*  if (tsim>70) exit (-1);*/
  return 1;

}



