/* horizon.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

/*
000108 modified syntax check reading empty line
010823 modified syntax check, reading char-data
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*------------------------------------- returns shadow factor 0..1 */
double  horizon(int init_flag,double teta_z,double azimut /*gamma_s */
           ,double *Rd,/*ratio_diffus*/ double *Rr /* ration reflect */ 
           ,double albedo,double slope,double orientation)
  {
  int a=0; /* debug flag */
  #define NR_AZIMUT_MAX 361  
  #define XXX          -999
  #define LINE_LETTERS  1000
  static double horizon_above [NR_AZIMUT_MAX];
  static double horizon_beyond[NR_AZIMUT_MAX];
  double shadow;
/*
reads from horizon/horizon_table.dat table of teta_z azimut in
init_step. 
if not init horizon() checks if current sun-pos
 teta_z,azimut is shadowed thru table.dat--> return double
*/

  if (init_flag) /* read horizon calculate Rd and Rr */
    {
    FILE *horizon_file;
    char filename[]="horizon/horizon_table.dat";
    char line_array[1000];
    char *line=line_array;
    char *rest=line_array;
    double fill=0;
    static double azimut_table[NR_AZIMUT_MAX];
    double azimut;
    int i_azimut=0,i_azimut_max;

    if (a) printf("start horizon init \n");
#ifdef WIN32
   sprintf(filename,"horizon/h_table.dat"); /* limit 8 characters */
   filename[7]='\\'; 
#endif
    if ((horizon_file=fopen(filename,"r"))==NULL)
        { printf("no '%s' !! \n\n",filename); exit (-1); };

    do {               /* search '-180' in 'horizon_table.dat' */
       if (fgets(line,LINE_LETTERS,horizon_file)==NULL) 
          { printf("no '-180' in 'horizon_table.dat'\n"); exit(-1); };
       if (a) printf(" read 'horizon_table.dat':%s \n",line);
       }
    while (strtod(line,&rest)!=-180);

    rest=line;
    if (a) printf(" scan all gamma_s \n");
    do {   /* scan all gamma_s from headline */
       line= rest;
       if (a) printf("line:%s \n",line);
       azimut_table[i_azimut] =strtod(line,&rest);
       if (a) printf("azimut_table[%d]:%.2f \n",i_azimut,azimut_table[i_azimut]);
       if ( fabs(azimut_table[i_azimut]) >180) 
          { printf("error azimut:%.2f in 'horizon_table.dat'"
                   "\n",azimut_table[i_azimut]); exit (-1); };
       i_azimut_max=i_azimut;
       i_azimut++;
       }
    while ( (azimut_table[i_azimut-1]<180) && (i_azimut<NR_AZIMUT_MAX) );
    if (a) printf("headline ready\n");

    line=line_array;
    fgets(line,LINE_LETTERS,horizon_file);  /* skip empty line */
    if ( (strlen(line)>1) && (line[0]!='#') ) {
      printf("horizon_init:no empty line behind azimut entries\n:line[%d]:%s"
            ,strlen(line),line);
      exit (-1);
    }
     

    /*----------------------------- fill the horizon_array with a dummy value */
    for   (i_azimut=0;i_azimut<NR_AZIMUT_MAX;i_azimut++)
      horizon_above [i_azimut]=horizon_beyond[i_azimut]=XXX;

    rest=line=line_array; /* reset pointers */
    fgets(line,LINE_LETTERS,horizon_file); /* read horizon_above line */
    for (i_azimut=0;i_azimut<=i_azimut_max;i_azimut++)
       {
       int i;
       line= rest;
       azimut=azimut_table[i_azimut];
       horizon_above [(int) azimut+180]=strtod(line,&rest);
       if (a) printf("line:%s\n",line);
       for (i=0;i<strlen(line)-1;i++) { /* check chars of line */
        if ( (line[i]!=' ') 
            && ( (line[i]>0x39)||(line[i]<0x2d) )
           ) { printf("horizon: table error column:%d line[%d]:%c\n",
                              i_azimut,i,line[i]); exit (-1); }
       }
       if (a) printf("horizon_above read: azimut_table[%d]:%.2f "
                     "horizon_above:%.2f \n"
                     ,i_azimut,azimut_table[i_azimut],
                      horizon_above [(int) azimut+180]);
       }

    /* same procedure with horizon beyond */
    rest=line=line_array; /* reset pointers */
    fgets(line,LINE_LETTERS,horizon_file); /* read horizon_beyond line */
    for (i_azimut=0;i_azimut<=i_azimut_max;i_azimut++)
       {
       line= rest;
       azimut=azimut_table[i_azimut];
       horizon_beyond [(int) azimut+180]=strtod(line,&rest);
       if (a) printf("horizon_beyond read: azimut_table[%d]:%.2f "
                     "horizon_beyond:%.2f \n"
                     ,i_azimut,azimut_table[i_azimut],
                      horizon_beyond [(int) azimut+180]);
       }
     fclose (horizon_file);
     
     /*---------------------------------------------------make 'horizon.dat'-------------*/
     sprintf(filename,"horizon/horizon.dat");
     if ((horizon_file=fopen(filename,"w"))==NULL)
         { printf("no '%s' !! \n\n",filename); exit (-1); };
#ifdef WIN32
     filename[7]='\\';
#endif


     /* now fill the rest of the horizon-elements */
     for (i_azimut=0;i_azimut<NR_AZIMUT_MAX;i_azimut++) {
      if (horizon_above [i_azimut]!=XXX) {   fill=horizon_above [i_azimut];}
      else                               {        horizon_above [i_azimut]=fill;}
      }
     fill=0;
     for (i_azimut=0;i_azimut<NR_AZIMUT_MAX;i_azimut++) {
      if (horizon_beyond[i_azimut]!=XXX) {   fill=horizon_beyond [i_azimut];}
      else                               {        horizon_beyond [i_azimut]=fill;}
      fprintf(horizon_file,"%d %.3f %.3f\n",i_azimut-180,horizon_above [i_azimut]
                                                       ,horizon_beyond[i_azimut] );
      }
     fclose (horizon_file);

     /* get Rd and Rr for diffus reduction with horizon */
     if (1)
       {
       double pi=3.14159265;
       double rad=pi/180;
       double phi; /* elevation */
       double d_phi=1,phi_front,phi_back;
       double A_korr=0; /* correctur factor for Idif due to inc surface */
       double teta;
       double alpha=0,alpha1=0;
       double c,y;
     
       for (i_azimut=0;i_azimut<NR_AZIMUT_MAX;i_azimut++) {
         if (a) printf("horizon_beyond[azi:%d] :%.2f \n",i_azimut,horizon_beyond [i_azimut]);
         }
       /* get Rd for the Halfroom of the surface */

       phi_front=orientation-90;
       phi_back =orientation+90;

       if (phi_front<-180) phi_front=180-fabs(phi_front+180);
       if (phi_back >180 ) phi_back =phi_back-180;

       /* check range */
       if (  (phi_front>180) || (phi_front<-180) 
              ||(phi_back>180) || (phi_back<-180)
          )
          {
          printf("error range : phi_front:%.1f phi_back:%.1f\n"
                 ,phi_front,phi_back);
          exit(-1);
          }
       for (phi=-180;phi<=180;phi+=d_phi) /*  sphere */
         {

         /* phi in the back range */
         if ( (  (phi_back>phi_front) /* usually the case, orient >-90 */
               &&(phi>=phi_front)&& (phi<=phi_back)
              )  
            ||(  (phi_back<=phi_front) /* orient <-90 */
               &&(phi>=phi_back)&& (phi<=phi_front)
              )
            )
            {
              if (alpha1>90) alpha=90-(alpha1-90) ; 
              else           alpha=alpha1;
              alpha1+=d_phi;
/*slope=70;*/
              /* first calc background diffus light; check horizon */
              if (slope>89) A_korr+=0;
              else
                {
                alpha=alpha*rad;
                c=tan(alpha)*tan(alpha);
                y=cos(slope*rad)*sqrt(c/(c+cos(slope*rad)*cos(slope*rad)));
                teta=acos(sqrt(c*cos(slope*rad)*cos(slope*rad)
                             /(c+cos(slope*rad)*cos(slope*rad))
                            +1-y*y/(cos(slope*rad)*cos(slope*rad)) 
                             )
                         );
                A_korr+=  0.5*( sin(pi/2)*sin(pi/2)
                         -sin(teta)*sin(teta))*d_phi*rad;
                }
               if (a) printf("slope:%.1f phi:%.1f teta:%.1f A_korr:%.2f\n"
                ,slope,phi,teta/rad,A_korr);
              /* second: calc front light */
              teta=horizon_beyond [(int) phi+180]*rad;
              A_korr+=  0.5*( sin(pi/2)*sin(pi/2)
                       -sin(teta)*sin(teta))*d_phi*rad;
              
             }

         /* if (phi>=180) teta=0; */
         
         }
         *Rd=A_korr/pi;
         *Rr=albedo*(1-*Rd);
        /* printf("without Idiff-correction of horizonline from phi=-180..-90 & 90..180 !!\n");
         if (a) printf("Rd:%.2f slope*rad:%.4f pi/2-slope*rad:%.4f\n"
               ,*Rd,slope*rad,pi/2-slope*rad);
        */
       }

     } /* end of init horizon table */

   /* get shadow on surface -------------------------------------------------*/
   shadow=0;

   /* calculate teta_z  */
   if (teta_z <= 90) teta_z=90-teta_z;
   if (teta_z >  90) teta_z=0;
   if ( (int) teta_z > horizon_above [(int) azimut+180] ) shadow=1;
   if ( (int) teta_z < horizon_beyond[(int) azimut+180] ) shadow=1;
  
   if (a) printf("horizon: azimut:%d zenit:%d "
                 "horizon_above:%.1f horizon_beyond:%.1f shadow:%.2f \n"
                 ,(int) azimut,(int) teta_z,horizon_above [(int) azimut+180]
                              ,horizon_beyond[(int) azimut+180],shadow);

   return (shadow);
 }
     
     
     
