/* load_control.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/
/* 
010223 1e-12 added to mp=f(x/dT)
020418 load_control updated
020526 T_amb,T_ambient_HtgOff,mode erased
*/

#include "global.h"
#include <stdio.h>
#include <stdlib.h>

/*--------------------------------LOAD_CONTROL-----------*/
int load_control(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
               qp_sum,tsim,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
{
/*  etc/xfig.awk
    
    TYPE 15
    PRIORITY  3          used by config routine: controller priority
    HYDRAULIC 99         used by config routine: non hydraulic
    
    PAR1: 1 [] mode
    PAR2: 10 [] T_CW
    PAR3: 10 [] T_set_return_WW
    PAR4: 19 [] T_ambient_HtgOff

    INP1: 10 [C] T_messure
    INP2: 10 [C] mp1_set
    INP3: 10 [C] Tset1  
    INP4: 10 [W] Pset_WW
    INP5: 10 [C] T_messure_WW
    INP6: 10 [W] Pset_htg
    INP7: 10 [C] T_messure_Htg_in
    INP8: 10 [C] T_set_Htg_out
    INP9: 10 [C] T_amb 

    OUT1: [] 'mpWW' ctr of diverter KW_inlet
    OUT2: [] 'mpWW1' pump_WW
    OUT3: [] 'mpHtg' pump_Htg
    OUT4: [] 'TsetHtgOut' T_set_Htg_out

*/

int a=0; /*------------------------------------- ANALYSE with a=1---*/

/* int    mode     =(int)  par[unit][1]; */
double T_CW            =par[unit][2];
double T_set_return_WW =par[unit][3];
/* double T_ambient_HtgOff=par[unit][4]; */

double T_messure    =out_o IN(1);
double mp1_set      =out_n IN(2)/3600; /* load_profile kg/h --> kg/s */
double Tset1        =out_n IN(3);
double Pset_WW      =out_n IN(4);
double T_messure_WW =out_o IN(5);
double Pset_htg     =out_o IN(6);
double T_messHtg_in =out_o IN(7);
double T_set_Htg_out=out_o IN(8);
/* double T_amb        =out_o IN(9); */

double cp=4190.0;
double P_WW;
double mp1=0,mp2=0,mp3=0; 

if (init_flag) {
}
if (a) printf(" load-control-procedure tsim:%.4f \n",tsim);
if (T_messure<Tset1) { /* increase mp */
 /* P_WW=mp*cp*dT */
 P_WW=mp1_set*cp*(Tset1-T_CW);
}
 P_WW=mp1_set*cp*(Tset1-T_CW);

mp1=P_WW/(cp*(T_messure-T_CW)+1e-12); 
if ( (T_messure<Tset1) && (mp1_set>0) )
 printf("load_control:[%d]:T_messure:%.1f<Tset1:%.1f\n",unit,T_messure,Tset1);

if (a) {
 if (mp1_set>0) 
     printf("tsim:%.1f P_WW:%.1f mp1_set:%.1f Tset:%.1f T_messure:%.1f mp1:%.1f\n"
            ,tsim,P_WW,mp1_set*3600,Tset1,T_messure,mp1*3600);
 }
if (T_messure_WW<=0) { printf("load_control:T_messure_WW:%.1f\n",T_messure_WW); /*exit (-1);*/ }
mp2=Pset_WW/(cp*(T_messure_WW-T_set_return_WW)+1e-12);
if (a) {
 if (mp2>0) printf("tsim:%.1f P_WW:%.1f T_messure_WW:%.1f mp2:%.1f\n"
            ,tsim,Pset_WW,T_messure_WW,mp2*3600);
 }
/* calculate T_set_Htg_out from Tamb =f(Tamb); */

if ( (T_set_Htg_out>T_messHtg_in-10) && (!init_flag) ) T_set_Htg_out=T_messHtg_in-10;
if (T_set_Htg_out<10) {
 printf("load_control[%d]: T_set_Htg_out:%.1f\n",unit,T_set_Htg_out); exit (-1);
}
mp3=Pset_htg/(cp*(T_messHtg_in-T_set_Htg_out)+1e-12);
if (a) {
 if (mp3>0) printf("tsim:%.1f Phtg:%.1f Tin:%.1f T_set_Htg_out:%.1f mp3:%.1f\n"
            ,tsim,Pset_htg,T_messHtg_in,T_set_Htg_out,mp3*3600);
 }
if ( (T_messHtg_in<T_set_Htg_out) && (!init_flag) ) {
  printf("load_control[%d]: T_messHtg_in:%.1f <T_set_Htg_out:%.1f \n",
     unit,T_messHtg_in,T_set_Htg_out); exit (-1);
}
if ( (mp1<0) || (mp2<0) || (mp2<0) ) {
 printf("load_control[%d]: mp<0!! exiting \n"
        "mp1:%.1f mp2:%.1f mp3:%.1f \n",unit,mp1*3600,mp2*3600,mp3*3600);
 exit (-1);
}

/* ------------------------------------outputs ----------------------*/
out_n[unit][1]=mp1*3600; /* mp kg/h */
out_n[unit][2]=mp2*3600; /* mp kg/h */
out_n[unit][3]=mp3*3600; /* mp kg/h */
out_n[unit][4]=T_set_Htg_out;            

  /* sys energies */
  /* out_n[unit][6]=0;  gains*/
  /* out_n[unit][7]=0;  losses  */
  /* out_n[unit][8]=0;  internal energy */

return (1);
}
/*---------------------------------------------------------------------*/

