/* mixer.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/


#include "global.h" 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*
020109 update comment and source 
020325 exit on mode!=1
020531 bug fix ctr_out: mdot_in0+mdot_in1==0 and 1-mp0/(sum_mp) 

*/
/*-------------------------------------------MIXER TYPE 13---------*/
int mixer(unit,par,in_u,in_nr,out_o,out_n,h,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double h;         /* sim time step                         */
           int    init_flag; /* set at initial loop of sim            */

{
/*  mixer_src2fig.awk
    
    TYPE 13
    PRIORITY  6          used by config routine
    HYDRAULIC 13         mixer fluid               

    
    PAR1: 1 [] 'mode' mode 1=> MIXER
    PAR2: 1 [1] 'ceta0' ceta0 hydraulic port 0 in
    PAR3: 1 [1] 'ceta1' ceta1 hydraulic port 1 in

    INP1: 0 [Ws] 'dQin0' deltaQ0-inlet
    INP2: 0 [kg/s] 'mp0in' mp0-inlet 
    INP3: 0 [Ws] 'dQin1' deltaQ1-inlet
    INP4: 0 [kg/s] 'mp1in' mp1-inlet

    OUT1: [Ws] 'dQout' deltaQ-outlet
    OUT2: [kg/s] 'mpout' mp-outlet =mp0+mp1
    OUT3: [1] 'ctr-out' = 1-mp0/(mp0+mp1) (collecting pipe)
    OUT4: [kg/h] 'mdot_out' view mdot

    OUT6: [Ws] 'sys-gain' system gains 
    OUT7: [Ws] 'sys-loss' system losses
    OUT8: [Ws] 'sys-ener' system capacity to to 0 degrees   
    OUT9: [1] 'CetaOut'
    OUT10: [Pa] 'deltaP0' written by hydraulic type
    OUT11: [Pa] 'deltaP1' 
*/
int a=0;
int pedantic=1;
int mode       =(int) par[unit][1];

double Qin0    =out_n IN(1);
double mdot_in0=out_n IN(2);
double Qin1    =out_n IN(3);
double mdot_in1=out_n IN(4);

double Qout=0,mdot_out=0,ctr_out=0;

/* dQ_out=dQ0_in+dQ1_in*/
if (init_flag) { 
  if ( (int) par[unit][1]!=1) { printf("mode mixer unit%d\n" ,unit);exit (-1); }
  if (mode!=1) {
    printf("mixer[%d] mode:%d error \n",unit,mode); exit (-1);
  }
}
if (a) printf("mixer[%d]\n",unit);

Qout=Qin0+Qin1;
mdot_out=mdot_in0+mdot_in1;
 /* set ctr for 'mixer before diverter situation */
if ((mdot_in0+mdot_in1)==0) ctr_out=0; /* no plugs */
else ctr_out=1-mdot_in0/(mdot_in0+mdot_in1);
if (ctr_out>1) ctr_out=1; /* limit ctr_:out 0..1 */
if (ctr_out<0) ctr_out=0;

if ( (mdot_in0<0) || (mdot_in1<0) || (mdot_out<0) ) {
 printf("mixer[%d] neg mdot's mdot_in0:%.4fkg/h mdot_in1:%.4f mdot_out:%.4f\n",
        unit,mdot_in0*3600,mdot_in1*3600,mdot_out*3600);
}
if (pedantic) {
 if ( (Qin0<0) || (Qin1<0) || (Qout<0) ) {
    printf("mixer[%d] neg dQ's Qin0:%.4fWs Qin1:%.4f Qout:%.4f\n",
           unit,Qin0,Qin1,Qout);
 }
}
/* set outputs */
out_n[unit][1]=Qout;
out_n[unit][2]=mdot_out;
out_n[unit][3]=ctr_out;
out_n[unit][4]=mdot_out*3600; /* kg/h */
return 1;
}


