/* pump.static.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include "global.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* 000411 update headline pump.static.c 

*/

/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/

/*-------------------------------------------PUMP TYPE 3------------*/
int pump(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,qp_sum,
         tsim,init_flag,previous_unit)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int    previous_unit; /* coming from this unit             */
{
/*  

    TYPE 03
    PRIORITY  5          used by config routine: pump priority
    HYDRAULIC 3          pump fluid


    PAR1: 300 [kg/h] mp_max
    PAR2: 4.19 [kJ/kgK] cp
    PAR3: 360 [W] Pmax_el Maximum power consumption
    PAR4: 0.2 [] factor kmp : P-el = kmp*mp (kg/h) 
    PAR5: 0 [] mode if=0 ctr=inp3 , if 1  mp_in=inp4 [kg/h] 

    INP1: 0 [Ws] deltaQ-inlet
    INP2: 0 [kg/s] mp-inlet
    INP3: 0 [] controller signal
    INP4: 0 [kg/s] mp, set by controller
    INP5: 20.0 [C] ambient temperature for starting pump

    OUT1: [Ws] deltaQ-outlet
    OUT2: [kg/s] mp-outlet
    OUT3: [W] P_el
    OUT4: [kg/h] mp-outlet
    OUT5: [C] temp of pump
    OUT19: first_execute_time
*/

int a=0; /* analyse */
double Qgain=0,Q_out=0,Told=0,Qin=0,Tin=1;
double mp_new=0,mp_old=0;
int mp_change=0;
double cp=par[unit][2]*1e+3;
int first_execute_time=out_n[unit][19];
int first_time_flag=1; /* first call of unit in timestep */
/* double mp_cp=par[unit][2]*1e+3*out_n[  in_u[unit][2] ]
                                       [ in_nr[unit][2] ];
*/

if   (first_execute_time!=tsim) {
                                out_n[unit][19]=tsim;
                                first_time_flag=1;
                                }
else                            first_time_flag=0;




/*----------------------------------------init---------------*/
  if (init_flag)
    {
    /*set default T_amb */
    out_o[unit][5]=out_n[unit][5]=out_n[unit][OUT_MAX0+5];
    if (out_o[unit][5]==0) {
         printf("default ambient temperature for pump unit:%d error\n"
	        ,unit); }
    }

  /*--------------------------------------mode 1 mpset by inp3 -----*/
  if ( (int) par[unit][5]==1 )  /* mode 1 */
    {
    mp_new = out_n[  in_u[unit][4] ]
                  [ in_nr[unit][4] ]/3600; /* set mp kg/s */
    if ( (mp_new>0) && (mp_new<2.7e-4) )
       {
       printf("mp_set pump:%d %.3f kg/h exit tsim:%.4fh\n"
              ,unit,out_n[unit][2]*3600,tsim/3600);
       exit (-1);
       }
    }
  else /* take ctr */
    mp_new=par[unit][1]/3600*out_n[  in_u[unit][3] ]
				  [ in_nr[unit][3] ];
  if (init_flag) mp_new=0;
				  
  if (a) printf("mp_set PUMP:%d =%.1f kg/h\n",unit,out_n[unit][2]*3600);

  mp_old=out_o[unit][2];
  if (  mp_old!=mp_new) mp_change=1;
  else                  mp_change=0;

 /* -----------------------first call of pump- fst loop -------------*/
  if (first_time_flag) /* bug 5/97 if previous unit of pump is heatexch for fst time */
                       /* maybe mp out is not ready previous_unit!=in_u[unit][1])  */
    {
       Qin   =out_o[unit][8]; /* get old energy from last timestep */
       Told  =out_o[unit][5]; /* get old temp   from last timestep */
       
       if (mp_change)
         {
         if (a) printf("PUMP:%d first loop: mp_change mp_old:%.2f mp_new:%.2f\n"
	               ,unit,mp_old*3600,mp_new*3600);
         if (mp_old==0) /* if mp_old == 0 take Qamb*/
           {
	   if (Qin!=0) { printf("PUMP:%d first loop Qin!=0 \n",unit); exit (-1); }
	   Qgain=Q_out=Told*mp_new*cp*h;
	   if (a) printf("PUMP:%d first loop Told:%.1f Qout:%.1f\n",unit,Told,Q_out);
           }
         else /* if mp_old != 0 */
	   { 
	   if (Told>200) printf("Told pump:%d : %.1f \n",unit,Told);
	   Q_out=mp_new*cp*h*Told;
	   Qgain=Q_out-Qin;
	   if (a) printf("PUMP:%d first loop: mp_change mp_old:%.2f mp_new:%.2f "
	          "Qgain:%.2f Q_out:%.1f Qin:%.1f \n"
	          ,unit,mp_old*3600,mp_new*3600,Qgain,Q_out,Qin);
	   }
          }
         else  /* if !mp_change */
           Q_out=Qin; 
    
	 out_n[unit][1]=Q_out;
	 out_n[unit][2]=mp_new;
/*
	 if (mp_new!=0) out_n[unit][5]=out_o[unit][5]=Q_out/(mp_new*cp*h); / * Temp * /
         if ( (out_n[unit][5]>160) || (out_n[unit][5]<-10)  ) 
            { printf("unit:%d temp pump:%.1f ..set to 90\n ",unit,out_n[unit][5]);
              out_n[unit][5]=out_o[unit][5]=90; }
*/
         out_n[unit][6]=Qgain;  /* gains from ambient */
         out_n[unit][9]=out_o[unit][9]+=Qgain; /* error sum due to illegal gains */
	 if (    ( fabs(out_n[unit][9])>1e8) 
              || ( fabs(Qgain          >1e8) ) )
            printf("unit:%d Qgain_sum:%.0f Ws Q_out:%.0f Qin:%.1f Qgain:%.0f mp_new:%.1f Told:%.1f\n"
                  ,unit,out_n[unit][9],Q_out,Qin,Qgain,mp_new*3600,Told );
	 return (1);
      }
    else /*-------------------- second loop ----------------------- */
      {
      if (a) printf("PUMP:%d second loop: mp_change mp_old:%.2f mp_new:%.2f\n"
                    ,unit,mp_old*3600,mp_new*3600);

      Qin=out_n[  in_u[unit][1] ]    /* get dQ in from closed loop */
               [ in_nr[unit][1] ];
      Qgain=     out_n[unit][6];         /* get back */
      if (Qin>0) Tin=out_n[unit][5]=out_o[unit][5]=Qin/(mp_new*cp*h); /* Temp */
      if ( (Tin>160) || (Tin<-10)  )
	  printf("unit:%d temp pump:%.1f \n ",unit,out_n[unit][5]);
           

      
      
      if  (mp_new==0) 
         { 
          if (Qin  !=0) { printf("PUMP:%d second loop mp_new==0 && Qin!=0\n"
	                  ,unit); }
         }
      out_n[unit][8]=Qin;    /* store Qin [Ws] */	   
      	 	 
      if ( (Qin==0) && (mp_new!=0)
         )			     
         { printf("pump:%d  second loop Qin=0! mp_new:%.1f kg/h \n",unit,mp_new*3600);
           exit (-1);
         }
      if (a) printf("PUMP:%d second loop: Qin:%.2f\n ",unit,Qin);

       /*----------------------------check mp in&out---------------------*/
       if (  ( (fabs(mp_new-out_n[  in_u[unit][2] ]
                                 [ in_nr[unit][2] ]))>1e-10  
             )						   
             && (!init_flag)
          )
          {
          printf("pump:%d  mp-balance error diff:mp_in-mp_out:%e\n"
                ,unit,out_n[  in_u[unit][2] ]
	                   [ in_nr[unit][2] ]*3600-out_n[unit][2]*3600);
          exit (-1);
          }
       }     


   if (a) printf("UNIT:%d mp_change:%d mp_old:%.1f mp_new:%.1f"
                 " Qgain:%.2f Qout:%.2f Qin:%.1f tsim:=%.5f \n"
		 ,unit,mp_change,mp_old*3600,mp_new*3600
		 ,Qgain,Q_out,Qin,tsim/3600);
		 
  /* -------------------------set outs----------------------------*/ 
  /* 1 & 2 first loop */
  
  /* out 3 P_el = Pmax*mp/mp_max */
  out_n[unit][3]=par[unit][3]*out_n[unit][2]*3600/par[unit][1];

  /* mp=contr*mp_max kg/h */
  out_n[unit][4]=out_n[unit][2]*3600;

  /* out5 Told first loop */
  /* out6 Qgain first loop */

  return 1;
 }

