
/*       10        20        30        40        50        60        70      */
/*345678901234567890123456789012345678901234567890123456789012345678901234567*/
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"
#include "rad_processor.h" /*werner*/


/* Multilayer,multi-node: the uniformly distributed capacities of each node was
   moved to a multi-node construction, that forces the node-distribution
   to place extended nodes inside one layer, which has the largest capacity
   mc in relation to the other node.
   calculate_nodes() routine is used to define nodes and associated 
   kA-values from node to adjacent node.
   Make sure not to use to many nodes (PAR8) => destabilizes calculation.
   About 2 nodes for thin and light inside walls,about 5 nodes for outside wall

991212 pcm layer included. malloc new wall-specific data to create
       new storage , instead of storing in out-array
       model of pcm in cnv/sys/lib/model.pcm.fig
000224 type included insode walls check
1.4.00 malloc routine was buggy , kA values to small
000607 inp11:  Tinside, if AirRadNode not in use standalone simulation
       inp12:  Tset_shadowing, close rollerblinds if T_AirRadNode>Tset
       mode_shadowing included
000609 pointer length
000709 if (Tnode<-30) exit instead of Tnode<-10 (Tamb: try10/27.01.)
001012 christiane: In GetQRadOutside: Changed sign of Qsol. Replaced fit by
       expression for black body radiation again, with new iteration loop.
       Floor is identified by alpha_a_conv=0, k_earth is used instead (par40). 
001104 chriwi update GetQrad_outside2 created with linear surface
       out20,21 created for Tsurface and T_test valid_twd
001206 christiane: Insert transmission factor for diffuse radiation.
       Send wall surface temperatures via OUT1 and OUT10 to AirRadNode instead
       of first and last node temp.
010528 pcm feature , see doc/waa.pcm.model.simple.fig
       calculate_nodes: *pcm_node added, pcm just 1 layer!
010824 chriwi:exit on (alpha_a_conv<=0) 
       !! if (init) k_earth=2.0; buggy : no static !!!
       old wall_types : no layer4 ; move par alpha_i and appha_a to par37,38
       GetTsurfaceInside debug -a added
       GetQrad_outside: argument unit added
       update from wall.pcm.010623.c
       update rad_proc2
       GetQrad_outside2() active
010827 info struct added, rad_proc2 modified
010920 rad_proc2 timezone added
       Tsurface_outside re-read from out9 not out16 
       Tsurface_inside re-read from out1
       Q_lw_win=h*A_win*((Ibt*(g_win-trans_win))+Idt*trans_win); -->
         trans_win --> g_win substituted (energy balance correct )
       Rd,Rr erased
011001 stebus: GetQrad_outside(1) is active again. Tsurface_outside was not 
       read from right input. Now only one or two iteration steps are needed.
       Results are different from GetQrad_outside2!!
       Tsurface_inside now is calculated by energy balance at inner surface,
       not by potential devider anymore.  
       new call for Get_Alpha_inside with Tsurface, not Tnode anymore.
       Get_Alpha_inside now is also called for second surface of Inside_wall.
       New Get_Alpha_inside with mode for horizontal and tilted walls,
       floors must have slope=180 instead of 0.
011002 stebus: Added 'int wall_unit' and 'int mode_shading' to the struct 
       radiation in the global.orig.h .
011008 stebus: new calculation of beta_luft in Get_Alpha_inside.
011016 stebus: Tsurface_outside re-read from out9 not out16 in the init, too. 
011025 stebus: new more dynamic version of energy-balance at the inside 
       surfaces. Newton-methode like at the outside-surface used. Also used for
       the determitation of Tsurf_ins_win. New window_model that balances 
       Q_sw_beam_win(transmited radiation to floor), Q_sw_diff_win(transmited
       radiation to Tstar) and Q_2ndary_win (combined heatflow from 
       surf_ins_win to Tair/Tstar), instead of old Q_sw_win(to floor) and 
       Q_lw_win(to Tstar). Q_2ndary_win is added up to Qloss_win. The sum of 
       both is devided into radiativ (lw) and convectiv portion by 
       energy-balance at the glazing's inside surface. Old losses were only 
       convectiv, gains only radiativ.
020123 stebus: Tenvironment with Rd and (Rd-1)
020131 stebus: Added fframe -> less gains.Used in AirRad_node with u_frame,too.
020131 stebus: call GetTsurfaceInside for the window with A_win, not A!
       call GetTsurfaceInside for 2nd surface of Insde_wall with Tstar2 
       instead of Tstar, 0 instead of Qsw (Qsw if connected is coupled into
       the 1st surface).
020212 stebus: include Q_sw_diff_towall in energy-balance at inner wall-surface
020222 stebus: Q_sw_diff_towall for inside_wall
020222 werner/stebus: pcm-bug in the init: pcm_node[wall_index]=0, 
       not pcm_node[i]=0
020515 stebus: if (Stationary_ambient_conditions && floor) t_amb= 10.00
*/

#define LAYER_MAX    4      /* max. number of layers for arrays */
#define NODES_MAX    50     /* max number of nodes in wall */
#define APPROX_ERROR 1e-8   /* approximitty error */
#define WALL_MAX 200        /* maximum number of walls, used for malloc */

#define SIGMA 5.67e-8            /* Stefan-Boltzmann constant [W/m2K4]  */
#define  Stationary_ambient_conditions 0 /* set this also in AirRadNode */

int calculate_nodes(int unit,double **par,double A,int pcm_layer,int *pcm_node,
                    int wall_index,double h,
                    int init_flag,int layer_max,int nodes_max,double **kA,
                    double **mc,double *k_wall_intern,double *mc_intern);


int Get_Alpha_inside(double Tsurface,double Tair,double H,double L,double slope,
                     double *alpha_i_conv);

int GetNodeTemp(double tsim,int unit,int wall_index,double **deriv_n,
                double **deriv_o,
                int i, double energy[NODE_MAX+1],double ***qp_sum,
                double Qi[NODE_MAX+2][3],
                int i1,double **init_energy,double *mc[WALL_MAX],
                int nodes_max,double Tsurface_outside,double h ,
                int pcm_node,double T_melt,double H_melt_upper,double mcp2 );

int GetQrad_outside(int unit, double *Qrad_outside,double *Tsurface_outside,
                    double *Qconva,double *Q_kA,double T_amb,double Tnode,
                    double eps_a_lw,double alpha_a_conv,double h,double A
                    ,double kA,int mode_ti,double g_win, double k_earth,
                     double eps_a_sw,double Igt,double cc,int init_flag,double Rd);
/* GetQrad_outside2 is linear calculation of Tsurface */
int GetQrad_outside2(int unit,double *Qrad_outside,double *Tsurface_outside,
                    double *Qconva,double *Q_kA,double T_amb,double Tnode,
                    double eps_a_lw,double alpha_a_conv,double h,double A
                     ,double kA,int mode_ti,double g_win,double k_earth,
                     double eps_a_sw,double Igt,double cc,int init_flag,double Rd);


/*int GetTsurfaceInside(double *Tsurface_inside,double *Qconvi,
                      double *Qrad,double *Q_sw_inside,
                      double A,double alpha_i_conv,int wall_index,
                      double kA,double Tinside,double Tnode,
                      int mode_wall_standalone,double h);*/
int GetTsurfaceInside(double *Tsurface_inside,double *Qconvi,
                      double *Qrad,double Q_gain,
                      double A,double alpha_i_conv,int wall_index,
                      double kA,double Tinside,double Tnode,
                      int mode_wall_standalone,double h,double Tstar,
                      double eps_i_lw);

double GetNodeTempPcm(double T,double energy,double T_melt,double mcp1,
                      double H_melt_upper,double mcp2);


/*-----------------------------------------WALL TYPE 28---------------------*/
int wall( info,interface,radiation,unit,par,in_u,in_nr,out_n,out_o,
          deriv_o,deriv_n,h,
          init_energy,qp_sum,tsim,init_flag,type,
          day_of_year,timezone,lenght,latitude,iam,type_index
         )
           struct sim_struct *info; /* see global.h */
           struct interface_struct *interface; /* see global.h */
           struct radiation_struct *radiation; /* see rad_processor.h */
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int *type;        /* type number array                     */
           int day_of_year;
           double *timezone;
           double *lenght;    /* geographic lenght                     */
           double *latitude;  /* geographic latitude                   */
           double **iam;     /* incidence angle modifier array        */
           int    *type_index; /* index of calling this type          */
{
  /*   comment is used by etc/xfig.awk
  
       TYPE 28
       PRIORITY  12         used by config routine
       HYDRAULIC 99         used by config routine

  
       PAR1: 8 [m] 'L_wall' length of wall   !! read by AirRadNode !!
       PAR2: 2.7 [m] 'H_wall' height of wall !! read by AirRadNode !!
       PAR3: 0.9 [1] 'eps_i_lw' epsilon inside wall IR
       PAR4: 0.9 [1] 'eps_a_lw' epsilon outside wall IR (set to 0 if floor)
       PAR5: 0.6 [1] 'eps_a_sw' epsilon outside wall shortwave (set 0 if floor)
       PAR6: 0   [] 'orient' orientation -90: east 0:south 90:west
       PAR7: 90  [] 'slope' 0:horizontal roof 90:vertical 180:horizontal floor
       PAR8: 5   [1] 'n_nodes' number of nodes(ca.5 for thick wall,3 for thin) 
       PAR9: 0  [m2] 'A_win' area of window !! read by AirRadNode !!
       PAR10: 0.787 [1] 'g_win' g-value of window, energy_flow (g > trans)
       PAR11: 0.748 [1] 'trans_win' transmission of window
       PAR12: 3.0 [W/m2k] 'U_win' U-value window !! read by AirRadNode !!
       PAR13: -0.12 [1] 'b0_win' iam=1-b0*(1/cos(teta)-1),If -1:iam_table 

       PAR14: 3 [1] 'n_layer' number of layers
       PAR15: 0 [1] 'hor_flag' 1:horizon from './horizon/horizon.dat' 0: no hor

       PAR16: 0.033 [m] 's_1' thickness of layer1 from inside 1 to outside n
       PAR17: 1.00  [kJ/kgK] 'cp_1' capacity of layer1
       PAR18: 0.51  [W/mK] 'lambda_1' Lambda of layer1
       PAR19: 1400  [Kg/m3] 'rho_1' density of layer1

       PAR20: 0.033 [m] 's_2' thickness of layer2
       PAR21: 1.00  [kJ/kgK] 'cp_2' capacity of layer2
       PAR22: 0.51  [W/mK] 'lambda_2' Lambda of layer2
       PAR23: 1400  [Kg/m3] 'rho_2' density of layer2
   
       PAR24: 0.033 [m] 's_3' thickness of layer3
       PAR25: 1.00  [kJ/kgK] 'cp_3' capacity of layer3
       PAR26: 0.51  [W/mK] 'lambda_3' Lambda of layer3
       PAR27: 1400  [Kg/m3] 'rho_3' density of layer3

       PAR28: 0.033 [m] 's_4' thickness of layer4 , 
       PAR29: 1.00  [kJ/kgK] 'cp_4' capacity of layer4
       PAR30: 0.51  [W/mK] 'lambda_4' Lambda of layer4
       PAR31: 1400  [Kg/m3] 'rho_4' density of layer4

       PAR32: 3   [1] 'pcm_layer' phase change layer number (pcm), if:0 no pcm
       PAR33: 3.2 [kJ/kgK] 'cp2' spec capacity upper temp range 
       PAR34: 300 [KJ/kg] 'dh_melt' melting enthalpy per kg
       PAR35: 30  [C] 'T_melt' melting temperature 
       PAR36: 5   [K]  'melt_hys' hysteresis of melting (undercooling)
       PAR37: 3.16  [W/m2K] 'alpha_i_conv' If -1: accord.VDI. If 0:floor 
       PAR38: 16.37 [W/m2K] 'alpha_a_conv' If -1: accord.Feist
       PAR39: 0 [1] 'mode_ti' use TWD model with par 41 and 42  
       PAR40: 1.7 [W/m2K] k_earth for floor. Set PAR38=0 then.

       PAR41: 0   [1] 'g_win' g-value of TWD
       PAR42: 0   [1] 'b0_TWD' iam factor b0 of TWD
       PAR43: 0   [1] 'mode_shading' if 1: use shading units con. to inp 14, 15 
       PAR44: 2   [m] 'H_win' hight of window If 0 default=2/3 H_wall
       PAR45: 0.84  [1] 'eps_ins_win' IR If 0 default=0.84
       PAR46: 3.16  [W/m2K] 'alpha_i_conv_win' If -1 accord.VDI If 0 default=-1
       PAR47: 0.15  [1] 'fframe' windows part of frame
       PAR48: 3.16  [W/m2K] 'u_frame' U-value frame

       INP1: 0 [Ws] 'Qrad' from Tstar to wall
       INP2: 0 [Ws] 'Qconv' from wall to air
       INP3: 10 [C] 'T_amb' ambient temperature      
       INP4: 0 [W/m^2] 'Igh' or Qconv from AirRadNode, Igh_Qconv2
       INP5: 0 [W/m^2] 'Idh' or Qrad  from AirRadNode, Idh_rad2
       INP6: 0 [W/m^2] 'Igt_sw_in' glob.incidence or shortw transm from adj zone
       INP7: 0 [m/s] 'v_wind' speed of wind
       INP8: 1 [] 'cloud_cover' cc 0-1, if -1 no data
       INP9: 0 [W] 'Q_sw_inside' sw inside coupling from AirRadnode
       INP10: 0 [Ws] 'Qloss_win' combined loss through window
       INP11: 20 [C] 'Tinside' if AirRadNode not in use
       INP12: 23 [C] 'Tset_shadowing' if T_Air>Tset_shadowing close blinds
       INP13: 0 [W] 'Pheat' heat_power to inside layer 
       INP14: 0 [1] 'shadow_win' shadow for window
       INP15: 0 [1] 'shadow_TWD' shadow for TWD

       OUT1: [C] 'Tsurf_ins' T_inside surface
       OUT2: [W/Km2] 'alpha_i_conv' alpha_conv_inside, !! read by AirRadNode !!
       OUT3: [1] 'eps_i_lw' eps_i_lw !! read by AirRadNode !!
       OUT4: [] 'slope' slope
       OUT5: [] 'orient' orientation
       OUT6: [Ws] 'sys_gains' system inlet energy 
       OUT7: [Ws] 'system_loss' system outlet energy 
       OUT8: [Ws] 'system_energy' internal energy 
       OUT9: [C] 'Tsurf_outs' Tsurface_outside
       OUT10: [C] 'T_last_node' T of last node
       OUT11: [W/Km2] 'k-wall' K-value wall 
       OUT12: [W] 'Igt' Igt on wall
       OUT13: [W] 'Ibt' Ibt on wall     
       OUT14: [Ws] 'Q_sw_beam_win' Q_sw_beam_win from window to floor
       OUT15: [Ws] 'Q_sw_diff_win' Q_sw_diff_win from Window to Tstar
       OUT16: [Ws] 'Q_2ndary_win' Q_2ndary_win devided into conv. and rad. Part

       OUT18: [W/m2K] 'k_node[0]'  k_node[0] directly read by AirRad_node 
       OUT19: [W] 'P_node_max' P_node_max
       OUT20: [C] 'T_first_node' T_first_node
       OUT21: [C] 'T_test' node_max-1 & node_max-2
       OUT22: [W/Km2] 'alpha_a_conv' !! if Inside_Wall read by AirRadNode !!
       OUT23: [1] 'eps_i_lw_win' epsilon inside window IR 
       OUT24: [W/Km2] 'alpha_i_conv_win' !! read by AirRadNode !!

       OUT47: [W/m2K] 'kA-wall' kA-Wall [W/K] read directly by AirRad_node 
       OUT48: [C] 'T_n_minus_1' T last node -1

       DERIV1: 10 [C] 'T1' temperature of node 1
       DERIV2:  0 [] 'dummy' dummy

  */

  int a=0; /*-------------------analyse a=1--------------*/
  int i=0;
  int test_heat_flow=0; /* Tamb=0,Tinside=20,Igt=0,standalone wall */
  int no_radiation=0; /* for testing only !! */
  int InsideWallFlag=0;
  double Qi    [NODE_MAX+2][3]; /* heat flow to nodes */
  double energy[NODE_MAX+1];

  double kA_wall=0;              /* kA value for one wall */
  double k_wall_intern=0;        /* k-value between 1. and last node (W/m2K) */
  double mc_intern=0;
  double iam_diff_win=0.872;/* inc. angle modifier diffus = const= 0.7 */
  double iam_diff_TWD=0.7; /* better would be an integration of b0 in the init*/
  double L           =      PAR (1);    /* [m] lenght of wall */ 
  double H           =      PAR (2);    /* [m] Height of wall */

  double eps_i_lw    =      PAR (3);    /* Inside of wall epsilon */
  double eps_a_lw    =      PAR (4);    /* Outside of wall eps.longwave*/
  double eps_a_sw    =      PAR (5);    /* Outside of wall eps.shortwave*/
  double orientation =      PAR (6);
  double slope       =      PAR (7);
  int    nodes_max   =(int) PAR (8); 
  double A_win       =      PAR (9);       /* area of window [m2] */
  double g_win       =      PAR (10);         /* g-value window [] */
  double trans_win   =      PAR (11);     /* trans window  [] */
  double u_win       =      PAR (12);/*u-value window[W/m2K]read AirRad!*/
  double b0_win      =      PAR (13);    /* b0 iam factor  */  
  int    layer_max   =(int) PAR (14);
  int    horizon_nr  =(int) PAR (15);         
                                       /* par16...par31 layer physics */
  int    pcm_layer   =(int) PAR (32); /* phase change layer number (pcm) */
  double cp2_pcm     =      PAR (33)*1e3; /*[Ws/kgK] cp upper temp range */
  double dh_melt     =      PAR (34)*1e3; /* [Ws/kg] melting enthalpy per kg */
  double T_melt      =      PAR (35); /* melting temp */
  /* double melt_hys = PAR (36); / * not yet used */

  double alpha_i_conv=      PAR (37);  /* alpha_i_conv [W/m2K] */
  double alpha_a_conv=      PAR (38);  /* alpha_a_conv [W/m2K] */
  int    mode_ti     =(int) PAR (39);  /* use twd with g_win */
  double k_earth     =      PAR (40);  /* k_earth, for floor */   

  double g_TWD       =      PAR (41);
  double b0_TWD      =      PAR (42);
  int    mode_shading=(int) PAR (43);
  double H_win       =      PAR (44);
  double eps_i_lw_win=      PAR (45);
  double alpha_i_conv_win=  PAR (46);
  double fframe      =      PAR (47);

  double A=L*H-A_win;
  double cp1_pcm=PAR (13+pcm_layer*4)*1e3;
  double m_pcm=A*PAR (12+pcm_layer*4)*PAR (15+pcm_layer*4); /* A*s*rho */
  double mcp2_pcm=m_pcm*cp2_pcm;
 
  double Q_sw_beam_win;      /* beam shortwave win to floor or inside wall */
  double Q_sw_diff_win;      /* diffus shortwave win to Tstar */
  double Q_2ndary_win;	     /* added up to Q_loss_win, then devided into 
                                convectiv and radiativ part */

  double sw_lost_ratio;      /* sw_lost_ratio set to  (1-0.035) */

  double Ibt,Idt,Igt,Igh,Idh,Igt_mess; /* Ibn */
  double Igt_trans,Ibt_trans,Idt_trans;

  double Qrad          =out_n IN (1); /*get current Qrad from AirRadnode */
  double Qconvi        =out_n IN (2); /*get current Qconv from AirRadnode*/
  double T_amb         =out_n IN (3); /* Ambient temperature */
  /* Inputs 4..6 read in later */
  double v             =out_n IN (7);  /* m/s velocity of wind */
  double cc            =out_n IN (8);  /* cloud cover (0=clear sky) */
  double Q_sw_inside   =out_n IN (9);  /*shortwave coupled to floor [W]*/
  double Qloss_win     =out_n IN(10);  /* Qloss_win for system balance */
  double Tinside       =out_n IN(11);  /* if wall is used standalone */
  double Tset_shadowing=out_n IN(12); 
  double heat_power    =out_n IN(13);  /* [W]  heating_power */

  double shadow_win    =in_u[unit][14];  /* unit number of shading-element */
  double shadow_TWD    =in_u[unit][15];

  double T_AirRadNode;
  double Tstar;
  double Tstar2;
  double Tsurface_inside     =out_o[unit][1];  /* reRead stored temp */
  double Tsurface_outside    =out_o[unit][9];
  double Tsurface_inside_win =out_o[unit][11];                   
  double Qconva; /* convective heat flow outsid wall */
  double albedo =0.2; /* ground reflexion */
/* longwave radiation to sky (Clarke,S.194) */

  double Qrad_outside=0;      /* lw radiation between wall and sky */ 
  static double     *kA[WALL_MAX]; /*k_node_malloc[wall_index][NODE_MAX] */
  static double     *mc[WALL_MAX]; /*    mc_malloc[wall_index][NODE_MAX] */
  static int pcm_node[WALL_MAX]; /* get node for pcm_layer */
  static double H_melt_upper[WALL_MAX]; /* store m*cp1*Tmelt+dHmelt */
  int wall_index=type_index[unit];
  int mode_wall_standalone=0; /* no air_rad_node is connected */
  int mode_shadowing=0;
  int mode_radproc=1; /* Igh & Idh */
  double QkAn=0; /* node_max */
  double Qconv2_i=0,Qrad2_i=0,Qsw_direct=0,Tinside2=0; /* if inside wall */
  /* ports from 2nd AirRadNode Qsw_direct from sw_out2*/
  double Qgain_node=0; /* ti and wall heating */
  double keff;
  double Rd=0.5;
  double A_glazing=0;
  double Q_sw_diff_towall=0;
  double Q_sw_diff_towall2=0;

if (a) printf("wall%d: interface par1:%.2f\n",
               unit,interface[unit].parameter[1]);

  Igh=0,Idh=0; /* if outside wall */
 
  if (test_heat_flow) { T_amb=0; Tinside=20; }

  /* arrange modes of this wall */
  if (in_u[unit][1]==unit) mode_wall_standalone=1;/*if input1 not connected */
  if (test_heat_flow) mode_wall_standalone=1;
  if ( (!mode_wall_standalone) && (Tset_shadowing!=0) ) mode_shadowing=1;
  /* check connection input 4 to set in- or outside wall */
  if (type[ in_u [unit][4] ]==TypeAirRadNode) InsideWallFlag=1;
  if (a) printf("wall[%d]: InsideWallFlag:%d \n",unit,InsideWallFlag);

  T_AirRadNode=deriv_n[in_u[unit][1]][1]; /* Temp. of AirRadNode */
  if (!mode_wall_standalone) 
    {
     Tinside =T_AirRadNode; /*AirRadNode*/
     Tstar=out_n[in_u[unit][1]][37];
     Q_sw_diff_towall=out_n[in_u[unit][2]][in_nr[unit][2]+29];
    }
  else Tstar=deriv_n[in_u[unit][1]][1]; /* Tstar=Tair */
  if (!InsideWallFlag) {
    Igh     =out_n IN (4);
    Idh     =out_n IN (5);
    Igt_mess=out_n IN (6);
    if ( (mode_shadowing) && (T_AirRadNode>Tset_shadowing) ) Igh=Idh=0;
  }
  else /* if InsideWallFlag */ {
     Qconv2_i  =out_n IN (4); /* direct import from AirRadNode */
     Qrad2_i   =out_n IN (5);  
     Qsw_direct=out_n IN(6);  /*  shortwave from a window */
     Tstar2=out_n[in_u[unit][5]][37];
     Tinside2=out_n[in_u[unit][5]][36];
     Q_sw_diff_towall2=out_n[in_u[unit][4]][in_nr[unit][4]+29];
  }

#if Stationary_ambient_conditions
if (alpha_a_conv!=0) T_amb=-10.00;/* normal outside wall. task26_S/MFH_heating:-10*/
else                 T_amb= 10.00;/* floor */
v =4.0/*3.1*/;
cc=0;
#endif

  if ((cc<0)||(cc>1)) {printf("Cloud cover value cc out of range, 
 set to cc=0.5"); cc=0.5;}
  if (no_radiation) Igh=Idh=Igt=0;
  /* chriwi if (no_radiation) heat_power=v=cc=Igh=Idh=IgT=0; */
  if (test_heat_flow) heat_power=0;

/* default window properties */
  if ((A_win>0)&&(H_win==0)) H_win=0.667*H; /*default: H_win= 2/3 H_wall */
  if ((A_win>0)&&(eps_i_lw_win==0)) eps_i_lw_win=0.84;
  if (A_win>0) A_glazing=(A_win*(1.0-fframe));

/* convective alpha air to wall  VDI Fa2 (Orginal  Churchill) */
/*--- inside surface of window ---*/
  if ((!mode_wall_standalone)&&(A_win>0) /* take par alpha_i_conv [W/Km2] */
      &&((alpha_i_conv_win==-1)||(alpha_i_conv_win==0))) {
    if (a)printf("wall unit[%d] window coupled to AirRad_node unit[%d]\n"
                 "Tsurface_inside_win:%.4f Tinside:%.4f",
                 unit,in_u [unit][1],Tsurface_inside_win,Tinside);
    Get_Alpha_inside(Tsurface_inside_win,Tinside,H_win,
                      (A_win/H_win),slope,&alpha_i_conv_win);
  }
/*--- (first) inside surface of wall ---*/
  if ((!mode_wall_standalone)&& /* take par alpha_i_conv [W/Km2] */
      (alpha_i_conv==-1)) {     /* no alpha_i_conv given */
    if (a)printf("wall unit[%d] 1st surface coupled to AirRad_node unit[%d]\n"
                 "Tsurface_inside:%.4f Tinside:%.4f",
                 unit,in_u [unit][1],Tsurface_inside,Tinside);
    Get_Alpha_inside(Tsurface_inside,Tinside,H,L,slope,&alpha_i_conv);
  }
/*--- outside surface of wall ---*/
  if ((alpha_a_conv==-1)&&(!InsideWallFlag)) alpha_a_conv=2.8+3*v;
/*--- second inside surface of inside wall ---*/
  if ((alpha_a_conv==-1)&&(InsideWallFlag)) {
    if (a)printf("wall unit[%d] 2nd surface coupled to AirRad_node unit[%d]\n"
                 "Tsurface_outside:%.4f Tinside2:%.4f",
                 unit,in_u [unit][5],Tsurface_outside,Tinside2);
     Get_Alpha_inside(Tsurface_outside,Tinside2,H,L,(180-slope),&alpha_a_conv);
  }
  if (a) printf("mode_wall_standalone:%d alpha_i_conv:%.1f inp1:%d unit:%d\n",
                mode_wall_standalone,alpha_i_conv,in_u [unit][1],unit);
 
/*-----------INIT-----------------------------------------------------------*/
  if (init_flag)  
   { 
   if ((u_win<=0.3) && (A_win>0)) { printf("u_win:%.2f \n",u_win); exit (-1); }

   if ( (alpha_a_conv<=0) && (k_earth<=0) && ( (int) alpha_a_conv!=-1) ) {
     printf("wall[%d]: !! alpha_a_conv:%.1f k_earth:%.1f !!\n"
          ,unit,alpha_a_conv,k_earth);
     exit (-1);}
   if ( (alpha_i_conv<=0) || (alpha_i_conv>100) ) {
       printf("wall[%d]!! alpha_i_conv:%.1f  !!\n",unit,alpha_i_conv); 
       exit (-1);}
   if (a) printf("wall[%d]:type(inp4):%d\n",unit,type[ in_u [unit][4] ]);
   if (A<0) { printf("wall[%d]: A:%.1f \n",unit,A); exit (-1); }
 
   
   kA[wall_index]=malloc_1double(NODE_MAX+2);
   mc[wall_index]=malloc_1double(NODE_MAX+1);
   pcm_node[wall_index]=0; /*init */

   calculate_nodes(unit,par,A,pcm_layer,&pcm_node[wall_index],wall_index,h,
                   init_flag,layer_max,nodes_max ,kA,mc, &k_wall_intern,
                   &mc_intern);
   if (pcm_node[wall_index]>0) {  /* calc H_melt_upper */
      H_melt_upper[wall_index]= 
        T_melt*mc[wall_index][pcm_node[wall_index]  ]
        +dh_melt*mc[wall_index][pcm_node[wall_index] ]/cp1_pcm;
      printf("wall[%d] H_melt_upper:%.1fkWh Tmelt:%.1f m:%.1f kg A:%.1f "
             "d:%.1fmm dh_melt:%.2f cp1:%.2f \n\n"
           ,unit,H_melt_upper[wall_index]/3.6e6,T_melt,m_pcm
           ,A,1e3*PAR (12+pcm_layer*4),dh_melt,cp2_pcm );
   }

   if (kA[wall_index][nodes_max+1]!=k_wall_intern)
     { 
     printf("k_wall_intern:%.1f != kA:%.1f\n",k_wall_intern
            ,kA[wall_index][nodes_max+1]); exit(-1);
     }

   if (0) for (i=0;i<=nodes_max;i++) printf("kA[%d]:%.2f mc[%d]:%.2f\n",
             i,kA[wall_index][i],i,mc[wall_index][i]);

   if (  ( alpha_i_conv==0  ) || ( (alpha_a_conv==0) && (k_earth==0) ) )
      { printf(" wall[%d]:par 37,38 not set :%.4f, %.4f \n"
               ,unit,alpha_i_conv,alpha_a_conv ) ;
        exit(-1);  }
           
   for(i=1;i<=nodes_max;i++) deriv_n[unit][i]=deriv_o[unit][i]=deriv_o[unit][1];
     Tsurface_outside=out_n[unit][9]=out_o[unit][9]=deriv_o[unit][nodes_max];
     Tsurface_inside=Tsurface_inside_win=deriv_n[unit][1];
      /* parameter-check */
   if (nodes_max>DERIV_MAX ) 
      {printf("nodes of unit:%d > DERIV_MAX",unit);exit(-1);}
   if (pcm_layer>4) { printf("wall[%d] pcm_layer:%d error\n"
          ,unit,pcm_layer); exit (-1); }
   for (i=1;i<=nodes_max;i++)
        {
        if (mc[wall_index][i]<=0) 
           { printf("mc of unit:%d i:%d<=0 mc:%.3f "
                     ,unit,i,mc[wall_index][i]);exit(-1);}
        if (i!=pcm_node[wall_index]) init_energy[unit][i]
                           =deriv_o[unit][1]*mc[wall_index][i];
        else { /* this node is a pcm-layer */
            if (deriv_o[unit][1]<T_melt) 
                  init_energy[unit][i]=deriv_o[unit][1]*mc[wall_index][i];
            else  init_energy[unit][i]=T_melt*mc[wall_index][i]
                                +dh_melt*mc[wall_index][i]/
                  /*cp_layer[pcm_layer]*/ (PAR (13+pcm_layer*4)*1e3)
                                +(deriv_o[unit][1]-T_melt)*mcp2_pcm;
        }
        if (i>1) deriv_n[unit][i]=deriv_o[unit][i]=deriv_o[unit][1];
        if (a) printf("T_init[%d]:%.1f energy:%.4f \n"
                        ,i,deriv_o[unit][1],init_energy[unit][1]);
        } /* i <= nodes_max */
   Q_sw_diff_towall=0;
   }; /* init_flag */
  /*------------------------------------------------------------------------*/
  /*-----END---INIT---------------------------------------------------------*/

  k_wall_intern=kA[wall_index][nodes_max+1];

  if (alpha_a_conv!=0) /* This must be a normal wall or the roof */
    kA_wall=1/(1/(alpha_i_conv*A)+(1/k_wall_intern)+1/(alpha_a_conv*A));
  else /* This must be the floor */
    {
     if (k_earth==0)
      {
        if (init_flag) 
        {printf("Wall unit %d:If alpha_a_conv=0 k_earth must be set!",unit);
         printf("Default value of 2 W/m^2K is assumed!!\n");}
        PAR (40)=k_earth=2.0;
      }
      kA_wall=1/(1/(alpha_i_conv*A)+(1/k_wall_intern)+1/(k_earth*A+1e-12));
     if (slope==0)
      {
       printf("Wall unit %d: for the floor slope has to be 180 instead of 0!\n",
             unit);
       exit(-1);
      }
    }

  if (init_flag) { printf("kA_wall[%d]:%.3f A:%.3f kA_wall_intern:%.3f "
                  "alpha_i_conv:%.3f alpha_a_conv:%.3f Pwall_init:%.1f\n"
                  ,unit,kA_wall,A,k_wall_intern
        ,alpha_i_conv,alpha_a_conv,kA_wall*(Tinside-T_amb) );  }

  if (a) printf("wall[%d]:kA_wall:%.2f kwall:%.2f k_wall_intern:%.2f "
                "alpha_i_conv:%.1f alpha_a_conv:%.1f A:%.1f\n"
        ,unit,kA_wall,kA_wall/A,k_wall_intern,alpha_i_conv,alpha_a_conv,A);

  if (!InsideWallFlag) /* calculate outside heatflows */
    {
      if ((in_u[unit][6]!=unit)&&(in_u[unit][4]==unit))  mode_radproc=IGT_MES;
      if ((in_u[unit][4]!=unit)&&(in_u[unit][5]!=unit))  mode_radproc=IGH_IDH;
      if ((in_u[unit][4]!=unit)&&(in_u[unit][5]==unit))  mode_radproc=IGH_MES;
      if ((in_u[unit][6]!=unit)&&(in_u[unit][4]!=unit))  mode_radproc=IGT_IGH;

      radiation->mode=mode_radproc;
      radiation->horizon_nr=horizon_nr;
      radiation->Igh=Igh;
      radiation->Idh=Idh;
      radiation->rho_g=albedo;
      radiation->slope=slope;
      radiation->orientation=orientation;
      radiation->shading_unit=shadow_win;
      radiation->wall_unit=unit;  
      radiation->mode_shading=mode_shading;

      if ((!mode_ti)||(A_win>0)) 
        { /* normal wall or window */
         radiation->iam_diff=iam_diff_win;
         radiation->b0=b0_win;
         radiation->g=g_win;
         radiation->tau=trans_win;
         radiation->shading_unit=shadow_win;

         rad_proc2(info,interface,radiation);
 
         Ibt_trans=radiation->Ibt_trans;/*window*/
         Idt_trans=radiation->Idt_trans;/*window*/
         Igt_trans=radiation->Igt_trans;/*window*/
         Igt=radiation->Igt;            /*wall*/
         Rd=radiation->Rd;
        }

#if Stationary_ambient_conditions
Ibt_trans=0/*200*/;
Idt_trans=0/*30*/;
Igt_trans=0/*230*/;
Igt=0/*300*/;
#endif

      /* 0.965; sw_lost_ratio=0.965forBestcase600-900eps_i_sw=0.6*/
      sw_lost_ratio=0.965;

      /* Q_sw_win and Q_lw_win are short and long wave transmitted heatflows
      through window to AirRadNode, theyre not balanced in this wall(), but
      later coupled e.g. to the floor. */
/*    Q_sw_win=h*Ibt_trans*trans_win*A_win*sw_lost_ratio; 
      Q_lw_win=h*A_win*((Ibt_trans*(g_win-trans_win))+Idt_trans*g_win); */
       /* [Ws] */  /* radiation->Ibt_trans & radiation->Idt_trans */

/* Q_sw_beam_win and Q_sw_diff_win are radiativ heatflow to floor and Tstar,
   Q_2ndary_win is combined heatflow to Tstar and Tair, theyre not balanced 
   in this wall(), but later in the AirRad_node                          to  */
      Q_sw_beam_win=h*Ibt_trans*       trans_win *A_glazing*sw_lost_ratio;/*floor*/
      Q_sw_diff_win=h*Idt_trans*       trans_win *A_glazing*sw_lost_ratio;/*Tstar*/
      Q_2ndary_win =h*Igt_trans*(g_win-trans_win)*A_glazing;     /*Tstar & Tair*/
      if (a)printf("wall unit[%d]:Q_sw_beam_win=%.2f,Q_sw_diff_win=%.2f,"
                   "Q_2ndary_win=%.2f\n",unit,Q_sw_beam_win,Q_sw_diff_win,
                   Q_2ndary_win);
      if (mode_ti) 
        {/* TWD */
         radiation->iam_diff=iam_diff_TWD;
         radiation->b0=b0_TWD;
         radiation->shading_unit=shadow_TWD;
         radiation->g=g_TWD;
         radiation->tau=0; 

         rad_proc2(info,interface,radiation);

         Igt_trans=radiation->Igt_trans;
         Rd=radiation->Rd;
/*         radiation->shading_unit=0;*/
        } 

      radiation->wall_unit=0;

      if (deriv_o[unit][nodes_max]<-30)
        printf("T10:%.1f T9:%.1f \n",deriv_o[unit][nodes_max],
               deriv_o[unit][nodes_max-1]);
      if (test_heat_flow) Igt=eps_a_sw=eps_a_lw=0;

      if(a)printf("day_of_year:%d,tsim:%.0f,unit:%d",day_of_year,tsim,unit);

      GetQrad_outside/*2*/(unit,&Qrad_outside,&Tsurface_outside,&Qconva,&QkAn
                      ,T_amb
                      ,deriv_o[unit][nodes_max], eps_a_lw,alpha_a_conv,h,A
                      ,kA[wall_index][nodes_max],mode_ti,g_win,k_earth,
                      eps_a_sw,Igt,cc,init_flag,Rd); 

      if (a) printf("Qconva:%.1f[W/m2] alpha_a_conv:%.1f T_amb:%.1f "
                    "T10:%.1f Qrad_outside:%.1f \n",
    Qconva/(A*h),alpha_a_conv,T_amb,deriv_o[unit][nodes_max],Qrad_outside);

    } /* end !InsideWallFlag */
  else /* InsideWallFlag is set */
    {
      Qrad_outside=Qconva=0;
      QkAn=Qrad2_i-Qconv2_i; /* QkAn=energy flow to last node from surface2 */
      Ibt=0;Idt=0;Igt=0; 

 /* for second side of inside wall*/     
 /*   For inside walls: Tinside2 = temperature of second zone connected to
      wall. Tsurface_outside is temperature of second inner wall surface */
/*      GetTsurfaceInside(&Tsurface_outside,&Qconv2_i,&Qrad2_i,&Q_sw_inside,
                      A,alpha_a_conv,wall_index,kA[wall_index][nodes_max],
                      T_amb,deriv_o[unit][nodes_max],mode_wall_standalone,h);*/
      GetTsurfaceInside(&Tsurface_outside,&Qconv2_i,&Qrad2_i,Q_sw_diff_towall2,
                        A,alpha_a_conv,wall_index,kA[wall_index][nodes_max],
                        Tinside2,deriv_o[unit][nodes_max],
                        mode_wall_standalone,h,Tstar2,eps_i_lw);
      if (a) printf("GetTsurfaceInside:T_inside_surface2:%.1f "
                    "Tinside2:%.1f Tnode:%.1f\n",
                    Tsurface_outside,Tinside2,deriv_o[unit][nodes_max]);

      Q_sw_beam_win=0; /* no win to ambient */
      Q_sw_diff_win=0;
      Q_2ndary_win=0;
    }
/*--------------------------------------------------------------------------*/

/*  GetTsurfaceInside(&Tsurface_inside,&Qconvi,&Qrad,&Q_sw_inside,
                    A,alpha_i_conv,wall_index,kA[wall_index][0],
                    Tinside,deriv_o[unit][1],mode_wall_standalone,h);*/
  if (a) printf ("Qconvi=%.1f Qrad=%.1f Q_sw_diff_towall=%.1f\n",
                 Qconvi,Qrad,Q_sw_diff_towall);
  GetTsurfaceInside(&Tsurface_inside,&Qconvi,&Qrad,
                    (Q_sw_inside+Q_sw_diff_towall),
                    A,alpha_i_conv,wall_index,kA[wall_index][0],
                    Tinside,deriv_o[unit][1],mode_wall_standalone,h,
                    Tstar,eps_i_lw);

  if (mode_wall_standalone) Q_sw_inside=0;

  if (a) printf("GetTsurfaceInside:Tsurface_inside:%.1f Tinside:%.1f "
                "Tnode:%.1f\n",
                Tsurface_inside,Tinside,deriv_o[unit][1]);

  if (A_win>0)/* window */
    {
      GetTsurfaceInside(&Tsurface_inside_win,&Qconvi,&Qrad,
                        (Q_2ndary_win-Qloss_win),A_win,alpha_i_conv_win,
                        wall_index,
                        0,Tinside,T_amb,mode_wall_standalone,h,
                        Tstar,eps_i_lw_win);

      if (a) printf("GetTsurfaceInside:Tsurface_inside_win:%.1f Tinside:%.1f "
                    "Tamb:%.1f\n",
                    Tsurface_inside_win,Tinside,T_amb);
    }

/*---------energy balance energy flow --------------------------------------*/

  /* printf("wall%d Q_sw_inside:%.1f \n",unit,Q_sw_inside); !!!!!!!! */


  for (i=1;i<=nodes_max;i++) /* run through wall */
    {        
      if (i==1) { /* inside node */
        Qi[1][1]= Q_sw_inside+Qrad-Qconvi; /* inside node */
        Qi[1][2]=heat_power*h; /* no gains except through wall heating */
        Qi[1+1][1]=Qi[1+1][2]=0; /* also used in GetNodeTemp() */
        GetNodeTemp(tsim,unit,wall_index,deriv_n,deriv_o,1,energy,qp_sum,
                    Qi,2,init_energy,mc,nodes_max,Tsurface_outside,h,
                    pcm_node[wall_index],T_melt,H_melt_upper[wall_index],
                    m_pcm*cp2_pcm );
      }
      else /* i>1 */
        {
          Qi[i][1]=h*kA[wall_index][i-1]*(deriv_o[unit][i-1]-deriv_o[unit][i]);
          if ( (mode_ti) && (i==nodes_max-1) )
            {
             Qgain_node=h*A*Igt_trans*g_TWD;  
             Qi[i  ][2]=Qgain_node;
            }
          else Qi[i  ][2]=0; /* no gains */
          Qi[i+1][1]=Qi[i+1][2]=0; /* also used in GetNodeTemp() */
          GetNodeTemp(tsim,unit,wall_index,deriv_n,deriv_o,i,energy,qp_sum,Qi
                      ,2,init_energy,mc,nodes_max,Tsurface_outside,h,
                      pcm_node[wall_index],T_melt,H_melt_upper[wall_index],
                      m_pcm*cp2_pcm );
  
          if (a) printf("tsim:%.1f kA[%d]=%.2f, T[%d]=%.2f, "
                        " P_xy:%.1fW mc:%.1f \n",
                        tsim,i,kA[wall_index][i],i-1,deriv_o[unit][i-1],
                        Qi[i][1]/h,mc[wall_index][i]);
        }
    } 
  Qi[nodes_max+1][1]= -1*QkAn; /* (Qrad_outside+Qconva); */
  Qi[nodes_max+1][2]=0;
  qp_sum[unit][nodes_max+1][1]+=Qi[nodes_max+1][1];
  if (a) { printf("wall[%d] inside tsim:%.1f Q_sw_beam_win:%.1f Qrad:%.1f "
                  "Qconvi:%.1f Qconva:%.1f "
                  "Qsw_direct:%.1f \n",unit,tsim
                  , Q_sw_beam_win,Qrad,Qconvi,Qconva,Qsw_direct); }

  for (i=1;i<=nodes_max;i++) 
    {                                  
      energy[i]= init_energy[unit][i]
        +qp_sum[unit][i  ][1]  /* this one to the last node  */
        -qp_sum[unit][i+1][1]  /* this one to the next node i+1 */
        +qp_sum[unit][i  ][2]; /* gains */
      deriv_n[unit][i]=energy[i]/mc[wall_index][i];
      if (pcm_node[wall_index]==i) 
         deriv_n[unit][i]=GetNodeTempPcm(deriv_n[unit][i],energy[i],T_melt,
                            mc[wall_index][i], H_melt_upper[wall_index],
                            m_pcm*cp2_pcm);
#if 0
      if (pcm_node[wall_index]==i)  printf("wall[%d] tsim:%.1f Tpcm:%.2f\n"
        ,unit,tsim,deriv_n[unit][i] );
#endif
/*
        H_melt_upper[wall_index]=
                      T_melt*mc[wall_index][pcm_node[wall_index]  ]
                     +dh_melt*mc[wall_index][pcm_node[wall_index] ]/cp1_pcm;
*/
   if (a) printf("Qi[%d][1]:%.4e  Qi[%d][1]:%.4e dT[%d]:%.4e energy[i]:%.1f \n"
  ,i,Qi[i][1],i+1,Qi[i+1][1],i,deriv_o[unit][i]-deriv_o[unit][i-1],energy[i]); 
   if (a) printf("Qi[%d][1]:%.4f  T[%d]:%.4f energy[i]:%.1f \n"
                    ,i,Qi[i][1],i,deriv_n[unit][i],energy[i]);
    }

  if (a) printf("unit:%d Qrad_outside:%.4f "
                "Qconva:%.4f  Qconvi:%.4f  Qrad:%.4f v-wind:%.4f\n"
                ,unit,Qrad_outside,Qconva,Qconvi,Qrad,v); 

  keff=0;
  for (i=2;i<=nodes_max;i++) keff+=(Qi[i][1] /* +Qi[i][2] */ );
    keff=keff/(A*h*(deriv_o[unit][1]-deriv_o[unit][nodes_max])
             *(nodes_max-1)); /* mean */
  if (a) printf("P9[%d]:%.1f keff_Qi:%.2f keff:%.2f k_wall:%.2f dT:%.1f\n"
              ,nodes_max-1
             ,(Qi[nodes_max-1][2]-Qi[nodes_max-1][1])/(A*h)
              ,(Qi[nodes_max+1][1]+Qi[1][1])/(2*A*h*(Tinside-T_amb)),keff,
              kA_wall/A,Tinside-T_amb /*Tinside*/ );



/*---------------set outputs------------------------------------------------*/

  if (a) printf("Wall unit %d,Touts:%.2f,Tins:%.2f,Qka:%.2f\n",
  unit,Tsurface_outside,   Tsurface_inside,QkAn);
  out_n[unit][1]=Tsurface_inside;   /* Temp. of wall surface inside */
  /*out_n[unit][1]=deriv_n[unit][1];*/
  out_n[unit][2]=alpha_i_conv;      /* Alpha convection to inside */
  out_n[unit][3]=eps_i_lw;          /* epsilon inside surface longwave */
  out_n[unit][4]=slope;             /* angle between horizontal and wall */
  out_n[unit][5]=orientation;       /* orientation 0=south -90=east +90=west */
  out_n[unit][6]=Qgain_node+heat_power*h; /* system gains [Ws]*/
  if (mode_wall_standalone) out_n[unit][6]-=Qconvi;/*no connect to AirRadNode*/

  out_n[unit][7]=/* -Qrad_outside-Qconva+Qloss_win;/ * system losses */
    -QkAn+Qloss_win;

  if (InsideWallFlag) out_n[unit][7]=Qloss_win;  
  /* total internal energy contents due to 0  Celsius */
  out_n[unit][8]=0;
  for (i=1;i<=nodes_max;i++) out_n[unit][8]+=energy[i];/*deriv_n[unit][i]*mc;*/
 
  if (InsideWallFlag)  out_n[unit][8]+= 0; /* Qsw_direct;  !!!!!!!!!!!*/ 
  /* out_n[unit][8]+=Q_sw_win; */
  out_n[unit][9] = Tsurface_outside;
  out_n[unit][10]= deriv_n[unit][nodes_max];
  out_n[unit][11]= Tsurface_inside_win;
  /*out_n[unit][12]=(Ibt+IdT);   Global solar radiation on wall */
  out_n[unit][12]=Igt; 
  out_n[unit][13]=Ibt; /* Beam radiation on wall */
  /* shortwave piped through,from window [Ws] to AirRadNode */
  /* in case of insid wall it's Qsw_direct piped twice
     from airRadnode sw_out_x through ideal window to 
     adjacent zone
  */
  if (InsideWallFlag) out_n[unit][14]=Qsw_direct; /* pipe through */
  else                out_n[unit][14]=Q_sw_beam_win;
                                  /* via AirRadNode into floor*/
  out_n[unit][15]=Q_sw_diff_win; /* diff shortwafe win to Tstar  [Ws] */
  out_n[unit][16]=Q_2ndary_win; /* addd up to Q_loss_win, then devided into
                               convectiv and radiativ part */
  out_n[unit][18]=kA[wall_index][0]; /* used in AirRadNode !!! */ 
  /*
    out_n[unit][19]=(+Qi[nodes_max-1][1]+Qi[nodes_max-1][2])/(h*A); 
  */
  out_n[unit][19]=(+Qi[nodes_max-1][2]/*+Qi[nodes_max-1][2]*/)/(h*A); 
  out_n[unit][19]=(+Qi[nodes_max-1][1]
                   -Qi[nodes_max  ][1] /* to 2nd nabour */
                   +Qi[nodes_max-1][2])/(h*A); 
  out_n[unit][19]=(+Qi[nodes_max-2][1]
                   -Qi[nodes_max-1][1])/(h*A); 
  out_n[unit][19]=(-Qi[nodes_max-1][1]
                   +Qi[nodes_max-2][2])/(h*A); 
  /* chriwi !!!!! */
  out_n[unit][20] = deriv_n[unit][1];
  out_n[unit][21]= 0.5*(deriv_n[unit][nodes_max-1]+deriv_n[unit][nodes_max-2]);
  /* (Qi[i][2]+Qi[i][1])/(A*h) */
  out_n[unit][22]=alpha_a_conv;
  out_n[unit][23]=eps_i_lw_win;
  out_n[unit][24]=alpha_i_conv_win;
  out_n[unit][47]=kA_wall; /*used by AirRadNode kA-value of wall incl.alpha */ 
  out_n[unit][48]=deriv_n[unit][nodes_max-1];
 
  return (1);
}
/*--------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------*/
int calculate_nodes(int unit,double **par,double A,int pcm_layer,int *pcm_node,
                    int wall_index,double h,
                    int init_flag,int layer_max,int nodes_max ,double **kA,
                    double **mc,double *k_wall_intern,double *mc_intern)

/*
double **par,double **out_n,double **out_o
                    ,int a1,double A,double h,double alpha_i_conv
                    ,double alpha_a_conv,double alpha_convi, int pcm_layer
                    ,double **k_node_malloc,int wall_index,int init_flag
                    ,int layer_max,int nodes_max
                    ,double **kA,double **mc)
*/
/*
calc nodes with max 4 layers and node_max 
return node_capacity 1..node_max kA[1..node_max+1] 
kA[wall_index][i3] wall_index is index to this wall ; i3 is from
                   0 up to node_max (from inside to outside)
*/
{  
int a=0;
int i,i1,i3;
double lamb_layer[LAYER_MAX+1];       /* lambda-value of layer [W/mK] */
double s_layer   [LAYER_MAX+1];       /* thickness of layer [m] */
double rho_layer [LAYER_MAX+1];       /* density of layer   [kg/m3] */
double cp_layer  [LAYER_MAX+1];       /* cp of layer        [kJ/kgK] */
double mc_layer  [LAYER_MAX+1];
int    layerWithMax_mc;               /* get max mc */
double mc_max=0;
int    NoOfNodesInLayer[LAYER_MAX+1]; /* divide layer into n subnodes */
double mc_layer_subnode[LAYER_MAX+1][NODES_MAX+1];/* divide layer into nodes */
double mc_check2=0;
double R_layer;
double R_check=0,R_check2=0;
double mc_check=0;
double R[NODES_MAX];
double summe=0;
double tau=1e12; /* timeconstant */

if (pcm_layer>0) a=1;

if ( (layer_max>LAYER_MAX) || (layer_max<1) ) {
   printf("wall[%d]:layer_max:%d LAYER_MAX:%d exit\n",
   unit,layer_max,LAYER_MAX); exit(-1); }
if (init_flag!=1) { printf("exit calculate_nodes()\n"); exit(-1); }
if (nodes_max>=NODES_MAX) {
   printf("wall[%d]:nodes_max:%d > NODES_MAX:%d exiting\n",unit,nodes_max
          ,NODES_MAX); exit (-1); }
if (nodes_max<layer_max) {
 printf("wall[%d]:layer_max:%d > nodes_max:%d exiting\n",unit,layer_max,
         nodes_max); exit (-1); }
for (i=1; i<=LAYER_MAX; i++) /* get parameters */
  {
  s_layer   [i]=PAR (12+i*4);    /* start layer 1 par 16 [m] */  
  cp_layer  [i]=PAR (13+i*4)*1e3; /* kJ/kgK --> Ws/kgK */
  lamb_layer[i]=PAR (14+i*4);
  rho_layer [i]=PAR (15+i*4);
  mc_layer  [i]=A*s_layer[i]*rho_layer[i]*cp_layer[i];
  if ( (mc_layer[i]<0) || (s_layer[i]<0) || (cp_layer<0) || (lamb_layer[i]<0)
      || (rho_layer[i]<0) ) {
    printf("wall[%d]:mc_layer[%d]:%.1f s:%.1f cp:%.1f lambda:%.1f \n"
            ,unit,i,mc_layer[i],s_layer[i],cp_layer[i],lamb_layer[i]);
    exit(-1); }

  summe+=mc_layer[i];
/* R_node[1]+=((a_glob[1][l])*s_layer[l])/lamb_layer[l]; kA=1/R*/


  if (a) { 
   printf("\nwall[%d] calculate_nodes: ######################\n",unit);
   printf("s[%d]=%.2fcm cp[%d]=%.2f lamb[%d]=%.2f "
          "rho[%d]=%.2f mc_layer[%d]=%.2f\n",
           i,s_layer[i]*100,i,cp_layer[i],i,lamb_layer[i],i,
           rho_layer[i],i,mc_layer[i]);
          };
  };
 printf("wall[%d],total capacity:%.4f kJ/K \n",unit,summe/1000);
for (i=1;i<=layer_max;i++) /* make nodes with integer number in one layer*/
 {
 NoOfNodesInLayer  [i]=1; /* begin with 1 subnode */
 mc_layer_subnode  [i][1]=mc_layer[i]; /* this one is still known */
 mc_check+=mc_layer[i];
 }
if (nodes_max>layer_max) /* create subnodes of LayerNodes */
 {
 for (i3=layer_max+1;i3<=nodes_max;i3++) /*add upper nodes with max-operator */
  {
  layerWithMax_mc=0; /* get max mc from mc_layer[] */
  mc_max=0;
  for (i=1;i<=layer_max;i++) 
   for (i1=1;i1<=NoOfNodesInLayer[i];i1++)
    if ( (mc_layer_subnode[i][i1]>mc_max) && (i!=pcm_layer) ) /*pcm just one
                                                                     layer!*/
     { 
     mc_max=mc_layer_subnode[i][i1]; 
     layerWithMax_mc=i; 
     if (a) printf("layer:%d subnode:%d mc_max:%.1f layerWithMax_mc:%d\n"
             ,i,i1,mc_max,layerWithMax_mc);
     }
    else if ( (i==pcm_layer) && (a) ) printf("pcm_layer:%d \n",i);
  /* split largest node into 2 seperate nodes */
  if (a) printf("layerWithMax_mc:%d\n",layerWithMax_mc);
  NoOfNodesInLayer[layerWithMax_mc]++; /* increment this one */
  for (i1=1;i1<=NoOfNodesInLayer[layerWithMax_mc];i1++)
    {
    mc_layer_subnode[layerWithMax_mc][i1]= mc_layer[layerWithMax_mc]
                                /NoOfNodesInLayer[layerWithMax_mc];
    if (a) printf("mc[%d][%d]:%.2f\n",layerWithMax_mc,i1
            ,mc_layer_subnode[layerWithMax_mc][i1]);
    }
  } /* end creating sub-nodes from i3 =layer_max to nodes_max */
 }

R_check = R_check2=0; /* reset check values */
mc_check=mc_check2=0;
i3=1; /* index kA[i],mc[i] */
for (i=1;i<=layer_max;i++) /* get kA[i3] and mc[i3] */
 {
 R_layer=s_layer[i]/(A*lamb_layer[i]); /* resistance of layer */
 R_check+=R_layer;                     /* res_wall */
 mc_check+=mc_layer[i];          /* mc_wall */

 printf("NoOfNodesInLayer[%d]:%d\n",i,NoOfNodesInLayer[i]);
 for (i1=1;i1<=NoOfNodesInLayer[i];i1++)
   {
   if (i1==1) R[0 ]=0; /* reset */

   mc[wall_index][i3  ]=mc_layer_subnode[i][i1];
   if ( (i==pcm_layer) && (i1==1) ) *pcm_node=i3; /* pcm_node */

   R[i3-1]+=0.5*R_layer/( (double) NoOfNodesInLayer[i]); 
   R[i3  ] =0.5*R_layer/( (double) NoOfNodesInLayer[i]);
   kA[wall_index][i3-1]=1/R[i3-1];
   kA[wall_index][i3  ]=1/R[i3  ];

   if (tau>mc[wall_index][i3]*R[i3]) tau=mc[wall_index][i3]*R[i3];
   if (i3==1) R_check2 =R[i3-1];
   else       R_check2+=R[i3-1]; /* last one is complete */
   if ( (i==layer_max) && (i1==NoOfNodesInLayer[i]) ) R_check2+=R[i3];

   mc_check2+=mc_layer_subnode[i][i1];
   if (a) 
   printf("wall[%d]: layer%2d subnode:%2d=node:%2d mc[%d][%d]=mc[%d]:%4.1f "
          "kg_H20 k[%d]:%.1f k_tmp[%d]:%.1f \n"
      ,unit,i,i1,i3,i,i1,i3,mc[wall_index][i3]/4190
      ,i3-1,kA[wall_index][i3-1]/A,i3,kA[wall_index][i3]/A );
   i3++;
   /* printf("kA[%d]:%.1f R1:%.4f k_layer:%.4f\n",i3-1,kA[wall_index][i3-1],
      R[i3],1/R_layer);*/
   }
 }

if (fabs(R_check-R_check2)>1e-8) {printf(
   "k_check:%.2f Rcheck:%.1f kA_check:%.1f kA_check2:%.1f mc_check:%.1f "
          "mc_check2:%.1f \n",
          1/(R_check*A),R_check,1/R_check,1/R_check2,mc_check,mc_check2);
   printf("wall[%d]:R_check:%.1f error\n", unit,R_check); exit (-1); 
   }
if (a) printf("#####wall[%d] has %d nodes k:%.2f A:%.2f kA:%.1f mc:%.1fkgH20\n"
       ,unit,i3-1,1/(R_check*A),A,1/R_check,mc_check/4190);


*k_wall_intern=kA[wall_index][nodes_max+1]=1/R_check;
*mc_intern=mc_check;

/* tau=*mc_intern*R_check/(nodes_max) ; */
if (fabs(tau)<10*h) {
 printf("tau error unit:%d tau:%.1fsec timestep h:%.1fsec\n" ,unit,tau,h); 
 
 /* exit(-1); */
}
if (fabs(mc_check2-mc_check)>1e-8) { printf("error mc_check2:%e \n",
       mc_check2-mc_check); exit(-1); }
/*-------------end multilayer.c----------------------------------------------*/
if (a) printf("pcm_node:%d\n\n",*pcm_node);
return(1);
} /* calculate_nodes */

int Get_Alpha_inside(double Tsurface,double Tair,double H,double L,double slope,
                     double *alpha_i_conv)
{
int a=0;
double pi=3.14159;
double rad=pi/180;
double g=9.81;                      /* m/s2 */
double Tm=0.5*(Tsurface+Tair);
double beta_luft=1/(Tm+273.15);
/*double beta_luft=(-0.01111e-3*Tm)+3.674e-3;*/
double Pr=(-1.316e-4*Tm)+0.7176;
double kinVis=(0.623e-7*Tm)+138.2e-7;
double lambda=(7.36e-5*Tm)+24.18e-3;

double Gr;       /* Grashof */
double Ra, Rac;  /* Rayleight */
double f1, f2;   /* factors for calculating Nusselt */
double Nu;       /* Nusselt */
double l;        /* characteristical length if horizontal, else use H */

if ((slope>180)||(slope<0))
  { printf("slope out of range! slope=%.1f\n",slope);
    exit (-1);
  }

/* vertcal surfaces:     VDI-Waermeatlas Fa 2 oder Feist: Therm. Geb'sim. */
/* tilted surfaces:      VDI-Waermeatlas Fa 3 */
/* horizontal surfaces : VDI-Waermeatlas Fa 4 */

if (Tsurface-Tair==0) { *alpha_i_conv=1e-12; 
 printf("Get_Alpha_inside:slope:%.1f Tsurface=Tair:%.4f alpha_i_conv:%.2f\n"
   ,slope,Tsurface,*alpha_i_conv); 
 return (1);
}
/* vertical wall */
if (slope==90)
#if 1       /* VDI: Churchill & Chu 1975 */
  { Gr=g*H*H*H*beta_luft*fabs(Tsurface-Tair)/(kinVis*kinVis); 
    Ra=fabs(Gr*Pr);
    f1=pow (1+pow( (0.492/Pr),0.5625 ), -1.77777);
    Nu=pow((0.825+0.387*pow((Ra*f1),0.1666)),2);
    *alpha_i_conv=Nu*lambda/H;
    if(a) printf("vertical wall, alpha_i_conv:%.1f\n",*alpha_i_conv);
  }
#endif
#if 0       /* Feist: Rat 1969 */
  { *alpha_i_conv=1/(0.3381+0.0012*(Tm+273.15))
                  *pow(fabs(Tsurface-Tair),(0.32+0.014*H));
    if(a) printf("vertical wall, alpha_i_conv:%.1f\n",*alpha_i_conv);
  }
#endif
/* horizontal floor */
if (slope==180)
  { l=(L*H)/(2*(L+H));
    Gr=g*l*l*l*beta_luft*fabs(Tsurface-Tair)/(kinVis*kinVis);
    Ra=fabs(Gr*Pr);
    if (Tsurface > Tair)                    /* Qp upwards from floor to air */
      { f2=pow(1+pow((0.322/Pr),0.55),-1.818);
        Nu=0.15*(pow((Ra*f2),0.333));
        if(a) printf("Qp upwards from floor to air\n");
      }
    else /* if (Tsurface < Tair) */       /* Qp downwards from air to floor */
      { f1=pow (1+pow( (0.492/Pr),0.5625 ), -1.77777);
        Nu=0.6*pow((Ra*f1),0.2);
        if(a) printf("Qp downwards from air to floor\n");
      }
    *alpha_i_conv=Nu*lambda/l;
    if(a) printf("horizontal floor, alpha_i_conv:%.1f\n",*alpha_i_conv);
  }

/* horizontal roof */
if (slope==0)
  { l=(L*H)/(2*(L+H));
    Gr=g*l*l*l*beta_luft*fabs(Tsurface-Tair)/(kinVis*kinVis);
    Ra=fabs(Gr*Pr);
    if (Tsurface < Tair)                     /* Qp upwards from air to roof */
      { /*=0.155*pow(Ra,0.333);*/
        f2=pow(1+pow((0.322/Pr),0.55),-1.818);
        Nu=0.15*(pow((Ra*f2),0.333));
        if(a) printf("Qp upwards from air to roof\n");
      }
    else /* if (Tsurface > Tair) */        /* Qp downwards from roof to air */
      { f1=pow (1+pow( (0.492/Pr),0.5625 ), -1.77777);
        Nu=0.6*pow((Ra*f1),0.2);
        if(a) printf("Qp downwards from roof to air\n");
      }
    *alpha_i_conv=Nu*lambda/l;
    if(a) printf("horizontal roof, alpha_i_conv:%.1f\n",*alpha_i_conv);
  }

/* tilted floor */
if ((slope>90)&&(slope<180))
  { Gr=g*H*H*H*beta_luft*fabs(Tsurface-Tair)/(kinVis*kinVis);
    Ra=fabs(Gr*Pr);
    if (Tsurface > Tair)                    /* Qp upwards from floor to air */
      { Rac=pow(10,(8.9-0.00178*pow((slope-90),1.82)));
        Nu=0.56*pow((Rac*sin(slope*rad)),0.25)
          +0.13*(pow(Ra,0.3333)-pow(Rac,0.3333));
        if (Nu<0)
          { f1=pow (1+pow( (0.492/Pr),0.5625 ), -1.77777);
            Nu=pow((0.825+0.387*pow((Ra*sin(slope*rad)*f1),0.1666)),2);
          }
        if(a) printf("Qp upwards from air to roof\n");
      }
    else /* if (Tsurface < Tair) */       /* Qp downwards from air to floor */
      { f1=pow (1+pow( (0.492/Pr),0.5625 ), -1.77777);
        Nu=pow((0.825+0.387*pow((Ra*sin(slope*rad)*f1),0.1666)),2);
        if(a) printf("Qp downwards from roof to air\n"); 
      }
    *alpha_i_conv=Nu*lambda/H;
    if(a) printf("tilted floor, alpha_i_conv:%.1f\n",*alpha_i_conv);  
  }

/* tilted roof */
if ((slope>0)&&(slope<90))
  { Gr=g*H*H*H*beta_luft*fabs(Tsurface-Tair)/(kinVis*kinVis);
    Ra=fabs(Gr*Pr);  
    if (Tsurface < Tair)                     /* Qp upwards from air to roof */
      { Rac=pow(10,(8.9-0.00178*pow((90-slope),1.82)));
        Nu=0.56*pow((Rac*sin(slope*rad)),0.25)
          +0.13*(pow(Ra,0.3333)-pow(Rac,0.3333));
        if (Nu<0)
          { f1=pow (1+pow( (0.492/Pr),0.5625 ), -1.77777);
            Nu=pow((0.825+0.387*pow((Ra*sin(slope*rad)*f1),0.1666)),2);
          }
        if(a) printf("Qp upwards from air to roof\n");
      }
    else /* if (Tsurface > Tair) */        /* Qp downwards from roof to air */
      { f1=pow (1+pow( (0.492/Pr),0.5625 ), -1.77777);
        Nu=pow((0.825+0.387*pow((Ra*sin(slope*rad)*f1),0.1666)),2);
        if(a) printf("Qp downwards from roof to air\n");
      }
    *alpha_i_conv=Nu*lambda/H;
    if(a) printf("tilted roof, alpha_i_conv:%.1f\n",*alpha_i_conv);  
  }

if ( (*alpha_i_conv<=0) || (*alpha_i_conv>10) )
  { printf("\nGet_Alpha_inside alpha_i_conv:%.1f error \n",*alpha_i_conv);
    printf("Get_Alpha_inside: Tsurface:%.1f Tair:%.1f H:%.1f L:%.1f\n"
              "slope:%.1f alpha_i_conv:%.4f Nu:%.2f \n",
              Tsurface,Tair,H,L,slope,*alpha_i_conv,Nu);
    exit (-1);
  }
if (a) printf(" alpha_i_conv:%.4f\n",*alpha_i_conv);
if (a) printf("Get_Alpha_inside: Tsurface:%.1f Tair:%.1f H:%.1f L:%.1f\n"
              "slope:%.1f alpha_i_conv:%.4f Nu:%.2f \n",
              Tsurface,Tair,H,L,slope,*alpha_i_conv,Nu);

return(1);
}

int GetQrad_outside(int unit, double *Qrad_outside,double *Tsurface_outside,
                    double *Qconva,double *Q_kA,double T_amb,double Tnode,
                    double eps_a_lw,double alpha_a_conv,double h,double A,
		    double kA,int mode_ti,double g_win,double k_earth,
                    double eps_a_sw,double Igt,double cc,int init_flag,double Rd)
 {
 int a=0; /* debug */
 int i=0,mute_flag=0;
 int no_sky_radiation=0;
 /* double cc=0; / * 0:clear sky, 1:cloudy */
 double Tsky_kelvin,T_environment=0,T_environment_4, T_so,T_store;
 double T_zero=273.15;
 double Qrad,QkA,Qconv;
 double a1,a2,b,sum; 
 double Qsol=0;

 T_store=*Tsurface_outside;
 if ( (cc>=0) && (cc<=1) )
   {
   Tsky_kelvin=pow(5.31e-13*pow(T_amb+T_zero,6)/SIGMA,0.25);
   Tsky_kelvin=(1-cc)*Tsky_kelvin+cc*(T_amb+T_zero);  /* cc=1 => Tsky = T_amb*/
   }
 else   Tsky_kelvin=pow(5.31e-13*pow(T_amb+T_zero,6)/SIGMA,0.25);
                                                    /* clear sky if cc=-1 */

 /* nach Clarke, Rural site: isolated alpha=0.5, beta+gamma=0.5 */
 T_environment_4=Rd*pow(Tsky_kelvin,4)+(1-Rd)*pow(T_amb+T_zero,4);
 T_environment=pow(T_environment_4,0.25)-T_zero;
 if (a) printf("cc:%.1f Tsky_kelvin:%.4f T_env.:%.4f T_amb%.4f wallunit:%d\n"
              ,cc,Tsky_kelvin,T_environment,T_amb,unit);

/* Tsurface_outside is determined from energy balance at outer wall surface: 
   Gains=losses => kA*h*(Tnode-Tsurface_outside)+Qsol=Qrad+Qconv;
   Gives nonlinear equation a1*pow(Tsurface_outside,4) + a2*Tsurface =b, which 
   is solved with Newtons method iteratively. See below for Qrad,Qconv.  */
if (!no_sky_radiation) Qsol=eps_a_sw*A*Igt*h;
if (Qsol<0) Qsol=0;
a1 = eps_a_lw*A*SIGMA;
a2 = alpha_a_conv*A + kA;
b  = Qsol/h + kA * (Tnode+T_zero) + a1 * T_environment_4  
   +  alpha_a_conv*A*(T_amb+T_zero);

/* If alpha_a_conv=0, we assume this is the floor. Calculate conductive heat 
   transfer to earth with k_earth=lambda_earth/d. d=mean penetration depth of 
   temperature variation into earth. */
if (alpha_a_conv==0)  
  { 
   *Tsurface_outside=(kA*Tnode + k_earth*A*T_amb)/(kA + k_earth*A);
   Qconv=0;
   Qrad=0;
   QkA= h*kA*(Tnode - *Tsurface_outside); 
if(a) printf("unit:%d,cc:%.1f,T_env:%.4f,Tamb:%.4f,Tso:%.4f,Tnode:%.4f(floor)\n"
          ,unit,cc,T_amb,T_amb,*Tsurface_outside,Tnode);/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  }
 else  /*Normal wall with radiative and convective heat exchange with ambient*/
{
 *Tsurface_outside+=T_zero; /* Rescale old Celsius value to Kelvin. */

if (a)  printf("Eingangstemp:Tnode:%.4f,Tso:%.4f,T_env:%.4f,Tamb:%.4f\n"
  ,Tnode,*Tsurface_outside-T_zero, T_environment,T_amb);

/* Iteration loop to determine new Tsurface_outside */
 do
 {
 i++;
 T_so = *Tsurface_outside;
 *Tsurface_outside = T_so - (a1 * pow(T_so,4) + a2*T_so - b)
                   / (4.0*a1*pow(T_so,3) + a2 );
 if (a) printf("Iterate: Old value of Tsurf_outside:%.4f,New value:%.4f i:%d\n"
               ,T_so-T_zero,*Tsurface_outside-T_zero,i);
 }
 while ((i<5)&&(fabs(*Tsurface_outside - T_so)>0.01)); /* max. 5 iterations*/

 *Tsurface_outside-=T_zero; /* Rescale to Celsius*/
  T_so-=T_zero;
if(a)printf("No of iteration steps=%d\n",i);
 /* Alternative without Qrad: *Tsurface_outside=(kA*Tnode+Qsol/h+
    alpha_a_conv*A*T_amb)/(alpha_a_conv*A + kA ); mute_flag=1;*/

 if (i>=5)  /* Iteration didnt converge, set reasonable value. */
  { printf("Iteration loop to determine Tsurface_outside didnt converge.\n");
    printf("Tsurface_outside set to 0.95*T_last_node.\n");
   *Tsurface_outside=Tnode+(T_environment-Tnode)*0.2;  
    printf("Tsurface_outside set to :%.4f\n",*Tsurface_outside);
    mute_flag=1;
  }

 if(a) printf("unit:%d,cc:%.1f,T_env:%.4f,Tamb:%.4f,Tso:%.4f,Tnode:%.4f\n"
          ,unit,cc,T_environment,T_amb,*Tsurface_outside,Tnode);
 if ((*Tsurface_outside<T_environment)&&(*Tsurface_outside<T_amb)
     &&(*Tsurface_outside<Tnode))
  { printf("Tsurface_outside<T_environment&&Tsurface_outside<Tamb"
           "&&Tsurface_outside<Tnode:"
           "T_env:%.4f,Tamb:%.4f,Tso:%.4f,Tnode:%.4f\n"
          ,T_environment,T_amb,*Tsurface_outside,Tnode);
    *Tsurface_outside=T_environment;
    if (*Tsurface_outside>T_amb) *Tsurface_outside=T_amb;
    if (*Tsurface_outside>Tnode) *Tsurface_outside=Tnode;
    mute_flag=1;  /* *Tsurface_outside set to minimum */
  }
 /* Check iteration results : */

 /*Thermal radiation losses: */ 
 Qrad = h*eps_a_lw*A*SIGMA*(pow(*Tsurface_outside+T_zero,4)-T_environment_4); 
 /*Convective heat losses: */
 Qconv= h*alpha_a_conv*A*(*Tsurface_outside-T_amb); 
 /* Heat transport from node to surface: */
 QkA  = h*kA*(Tnode - *Tsurface_outside); 
 if (a) printf("Qka:%.4f,Qsol:%.4f,Qrad:%.4f,Qconv:%.4f,T_env:%.4f\n",
        QkA,Qsol,Qrad,Qconv,T_environment); 

 sum=Qrad+Qconv+Qsol+QkA;
 if (a) printf("Qka:%.4f,Qsol:%.4f,Qrad:%.4f,Qconv:%.4f,T_env:%.4f sum:%.1f\n",
        QkA,Qsol,Qrad,Qconv,T_environment,sum);

 if ((mute_flag!=1)&&(fabs(sum)>10.0)) /* no need to check if mute_flag=1 */
 {
  if (fabs(Qrad+Qconv-Qsol-QkA)/ 
      fabs(0.5*sum) > 0.05)
  {   printf("Qrad+Qconv:%.4f,Qsol+kA*h*..:%.4f\n",Qrad+Qconv,Qsol+
          kA*h*(Tnode-*Tsurface_outside));
      printf("Warning: Iterat.method for determ.Tsurface_outside unprecise.");
  } 
 }
} /* End of if block: normal wall or floor */

 *Qconva=Qconv;
 *Qrad_outside=Qrad;
 *Q_kA=-QkA;  
 if (a) printf("GetQrad_outside:[%d] Qconva:%.2f Qrad_outside:%.2f Q_kA:%.2f "
               "sum:%.2f\n",unit,*Qconva,*Qrad_outside,*Q_kA,
               *Qconva+*Qrad_outside+*Q_kA);

            
 if (Tnode<-30)  { 
   printf("T_amb:%.1f T_env:%.1f Tsky:%.1f dT:%.1f S:%.1fW/m2 "
          "Tsurface_outside:%.1f Tnode:%.1f \n"
         ,T_amb,T_environment,Tsky_kelvin-T_zero,Tnode-T_environment,
          eps_a_lw*SIGMA*(T_environment_4-pow(Tnode+T_zero,4) ),
          *Tsurface_outside,Tnode) ;
   exit (-1);
   }

 return 1;
 } /* GetQrad_outside */

/*int GetTsurfaceInside(double *Tsurface_inside,double *Qconvi,
                      double *Qrad,double *Q_sw_inside,
                      double A,double alpha_i_conv,int wall_index,
                      double kA,double Tinside,double Tnode,
                      int mode_wall_standalone,double h)*/
int GetTsurfaceInside(double *Tsurface_inside,double *Qconvi,
                      double *Qrad,double Q_gain,
                      double A,double alpha_i_conv,int wall_index,
                      double kA,double Tinside,double Tnode,
                      int mode_wall_standalone,double h,double Tstar,
                      double eps_i_lw)
{
/* Tnode and Tinside are used to get Tsurface via potential-divider 
    U1=U*R1/(R1+R2) */
int a=0;
/*double kA_total=1.0/(1.0/(A*alpha_i_conv)+1.0/kA);*/

/* *Tsurface_inside=Tinside+( -Tinside + Tnode )*kA_total/(A*alpha_i_conv); */

/* Tsurface_inside with energy-balance at inside surface, stebus */
/* *Tsurface_inside=(kA*Tnode*h+alpha_i_conv*A*Tinside*h+*Qrad+*Q_sw_inside) 
    	 /(kA*h+alpha_i_conv*h);*/
#if 0
*Tsurface_inside=(*Qrad+*Q_sw_inside-*Qconvi)/(kA*h)+Tnode;
if (mode_wall_standalone)
{
 *Qconvi=kA_total*(Tnode-Tinside)*h;
 *Qrad=0;
 *Q_sw_inside=0;
 }
#endif
#if 1
int i=0;
int mute_flag;
double T_zero=273.15;
double a1,a2,b;
double Tstar_4;
double T_si;
double Qrad_check;
double Qconvi_check;
double QkA_check;
double sum;
double kA_total;
if (!mode_wall_standalone)
  {
*Tsurface_inside+=T_zero; /* Rescale old Celsius value to Kelvin. */
Tstar+=T_zero;
Tstar_4=Tstar*Tstar*Tstar*Tstar;
/* Tsurface_inside is determined from energy balance at inner wall surface: 
   Gains=losses => 
   kA*h*(Tsurface_inside-Tnode)+alpha_i_conv*A*h*(Tsurface_inside-Tinside)
    =eps_i_lw*SIGMA*A*h*(Tstar^4-Tsurface_inside^4)+Q_sw_win;
   if window:
   Q_2ndary_win-Qloss_win=eps_i_lw_win*SIGMA*A*h*(Tsurface_inside^4-Tstar^4)
                      +alpha_i_conv*A*h*(Tsurface_inside-Tinside);
   Gives nonlinear equation a1*pow(Tsurface_outside,4) + a2*Tsurface =b, which 
   is solved with Newtons method iteratively. See below for Qrad,Qconv.  */
a1 = eps_i_lw*A*SIGMA;
a2 = alpha_i_conv*A + kA;
b  = Q_gain/h + kA * (Tnode+T_zero) + a1 * Tstar_4  
   +  alpha_i_conv*A*(Tinside+T_zero);

if (a)printf("eps_i_lw:%.4f,A:%.4f,SIGMA:%.4f,alpha_i_conv:%.4f,kA:%.4f,\n"
       "Q_gain/h:%.4f,Tnode+T_zero:%.4f,Tinside+T_zero:%.4f\n",
       eps_i_lw,A,SIGMA,alpha_i_conv,kA,Q_gain/h,Tnode+T_zero,Tinside+T_zero);
if (a)printf("conv:%.1f a1:%.4f,a2:%.4f,b:%.4f\n",alpha_i_conv,a1,a2,b);
/* Iteration loop to determine new Tsurface_inside */
do
{
i++;
T_si = *Tsurface_inside;
*Tsurface_inside = T_si - (a1 * pow(T_si,4) + a2*T_si - b)
                  / (4.0*a1*pow(T_si,3) + a2 );
if (a) printf("Iterate: Old value of Tsurf_inside:%.4f,New value:%.4f i:%d\n"
              ,T_si-T_zero,*Tsurface_inside-T_zero,i);
}
while ((i<5)&&(fabs(*Tsurface_inside - T_si)>0.01)); /* max. 5 iterations*/
*Tsurface_inside-=T_zero; /* Rescale to Celsius*/
if (a)printf("No of iteration steps=%d\n",i);
if (i>=5)  /* Iteration didnt converge, set reasonable value. */
  { printf("Iteration loop to determine Tsurface_inside didnt converge.\n");
    printf("Tsurface_inside set to (*Qrad+*Q_sw_inside-*Qconvi)/(kA*h)+Tnode.
           \n");
    if (kA!=0)/* wall */
      *Tsurface_inside=(*Qrad+Q_gain-*Qconvi)/(kA*h)+Tnode;
    else /* window */
      *Tsurface_inside=Tinside-Q_gain/h*0.04/A;
    printf("Tsurface_inside set to :%.4f\n",*Tsurface_inside);
    mute_flag=1;
  }
/* Check iteration results : */
 /* radiativ heatflow from AirRad_node: */ 
Qrad_check  = h*eps_i_lw*A*SIGMA*(Tstar_4-pow(*Tsurface_inside+T_zero,4)); 
/* convective heatflow to AirRad_node: */
Qconvi_check= h*alpha_i_conv*A*(*Tsurface_inside-Tinside); 
/* Heat transport from surface to node: */
QkA_check   = h*kA*(*Tsurface_inside - Tnode);
 sum=Qrad_check+Q_gain+Qconvi_check+QkA_check;
if (a) printf("QkA:%.4f,Q_gain:%.4f,Qrad:%.4f,Qconv:%.4f,Tstar:%.4f "
              "sum:%.1f\n",QkA_check,Q_gain,Qrad_check,Qconvi_check,Tstar,sum);
if ((mute_flag!=1)&&(fabs(sum)>10.0)) /* no need to check if mute_flag=1 */
{
 if (fabs(Qrad_check+Q_gain-Qconvi_check-QkA_check)/ 
     fabs(0.5*sum) > 0.05)
 {   printf("Qrad+Qconv:%.4f,Q_gain+kA*h*..:%.4f\n",Qrad_check+Qconvi_check,
            Q_gain+kA*h*(Tnode-*Tsurface_inside));
     printf("Warning: Iterat.method for determ.Tsurface_inside unprecise.");
 } 
}
}/* !mode_wall_standalone  */
#endif
else/* mode_wall_standalone */
  {
   kA_total=1.0/(1.0/(A*alpha_i_conv)+1.0/kA);
   *Qconvi=kA_total*(Tnode-Tinside)*h;
   *Qrad=0;
   *Tsurface_inside=Tinside+( -Tinside + Tnode )
                     *kA_total/(A*alpha_i_conv);
   }

return (1);
}  /* GetTsurfaceInside */

int GetNodeTemp(double tsim,int unit,int wall_index,double **deriv_n,
                double **deriv_o,
                int i, double energy[NODE_MAX+1],double ***qp_sum,
                double Qi[NODE_MAX+2][3],
                int i1,double **init_energy,double *mc[WALL_MAX],
                int nodes_max,double Tsurface_outside,double h,
                int pcm_node,double T_melt,double H_melt_upper,double mcp2  )
{
/* GetNodeTemp() is a stabilized euler solver, which should ensure
   a solution on critical timeconstants tau. this results one
   arge lambda's or small capacities.
   first energy[i] is calculated with colsim qp_sum algorithm.
   dedicated node temperature deriv_n[unit][i] is calculated on
   the node energy. now ambient node temperatures are used to define 
   min and max values of all belonging node-temps.
   stab-criteria for the new node temp deriv_n[unit][i] is the 
   accomplishment to this range. otherwise dQ_too_much is substituted
   from the Qi[i][1] heatflows and energy balance is calculated for
   next time.
   Qi[i][2] is the heatgain to the node: this is calculated after
   stab-criteria, since temp-increase out of criteria range is 
   possible.
*/
int pedantic=1;
int k=0;
double min=1e16,max=-1e16;
double T_tmp=0,dQ_too_much;
/* Qi[i]2] are gains,first calc passive heatflows */
/* qp_sum[unit][i][1]+=Qi[i][1]; */

/*
for (k=1;k<=i1;k++) qp_sum[unit][i][k]+=Qi[i][k]; 
*/
qp_sum[unit][i][1]+=Qi[i][1]; /* QkA passiv heatflow */

energy[i]= init_energy[unit][i]
           +qp_sum[unit][i  ][1]  /* this one to the last node  */
           -qp_sum[unit][i+1][1]  /* this one to the next node i+1 */
           +qp_sum[unit][i  ][2]; /* gains of last time step */

deriv_n[unit][i]=energy[i]/mc[wall_index][i];
if (pcm_node==i) deriv_n[unit][i]=GetNodeTempPcm(deriv_n[unit][i],energy[i],
                                  T_melt,mc[wall_index][i],H_melt_upper,mcp2);
 
/* stability-check: check range of deriv_n[i] in deriv_o[i-1,i+1]*/

if ( (i>1) && (i<=nodes_max) ) /* i=1 is inside node which gets
                                 forced energy input */
 {
 /* try to reduce Q[i][1] from adj. node (dQ_too_much) */
 deriv_o[unit][nodes_max+1]=Tsurface_outside; /* trick */
 for (k=-1;k<=1;k++) if (deriv_o[unit][i+k]<min) min=deriv_o[unit][i+k];
 for (k=-1;k<=1;k++) if (deriv_o[unit][i+k]>max) max=deriv_o[unit][i+k];

 /* stab-criteria to adjacent nodes */
 if (   (deriv_n[unit][i]>max+0.01)  || (deriv_n[unit][i]<min-0.01) )
   {
   if (0) printf("stability-max:deriv_n[unit][%d]:%.2f min:%.1f max:%.1f \n",
          i,deriv_n[unit][i],min,max);
   if (deriv_n[unit][i]>max) T_tmp=max;
   if (deriv_n[unit][i]<min) T_tmp=min;

   dQ_too_much=(deriv_n[unit][i]-T_tmp)*mc[wall_index][i];
   if (1) printf("In wall unit:%d: dQ_too_much:%.1fWs T_tmp:%.3f "
                 "deriv_n:%.3f,Qi[%d][1]:%.1f\n"
                 ,unit,dQ_too_much,T_tmp,deriv_n[unit][i],i,Qi[i][1]);
   /*
   Qi[i][1]= -dQ_too_much;
   qp_sum[unit][i][1]+=Qi[i][1]; / * 2nd time ! * /
   */
   qp_sum[unit][i][1]-=Qi[i][1];
   Qi[i][1]-=dQ_too_much; /* correction */
   qp_sum[unit][i][1]+=Qi[i][1];
   
   energy[i]= init_energy[unit][i]
           +qp_sum[unit][i  ][1]  /* this one to the last node  */
           -qp_sum[unit][i+1][1]  /* this one to the next node i+1 */
           +qp_sum[unit][i  ][2]; /* gains */
   deriv_n[unit][i]=energy[i]/mc[wall_index][i];
   if (pcm_node==i) deriv_n[unit][i]=GetNodeTempPcm(deriv_n[unit][i],energy[i],
                                  T_melt,mc[wall_index][i],H_melt_upper,mcp2);
   
   if (0) printf("           :Qi[%d][1]:%e T[%d]:%.1f\n"
      ,i,Qi[i][1],i,deriv_n[unit][i]);
   if (pedantic) printf("tsim:%.1f GetNodeTemp[%d] T:%.1f->Tnew:%.1f "
                        "T1:%.1f T2:%.1f P:%.1f\n"
               ,tsim,i,deriv_o[unit][i],deriv_n[unit][i],
               deriv_o[unit][i+1],deriv_o[unit][i-1],dQ_too_much/h);
   
   } /* stabilization */

   if ( (deriv_n[unit][i]>100) || (deriv_n[unit][i]<-30) ) {
     printf("wall[%d] GetNodeTemp() node:%d Tn>100 || <-30"
            "min:%.2f max:%.2f Tnew:%.2f\n"
             ,unit,i,min,max,deriv_n[unit][i]);
     printf("     To[%d]=%.5fC To[%d]=%.5fC To[%d]=%.5fC \n"
           ,i-1,deriv_o[unit][i-1],i,deriv_o[unit][i],i+1,deriv_o[unit][i+1]); 
     printf("     T [%d]=%.5fC T [%d]=%.5fC T [%d]=%.5fC \n"
           ,i-1,deriv_n[unit][i-1],i,deriv_n[unit][i],i+1,deriv_n[unit][i+1]); 
     printf("     P1[%d][1]=%.5fC P1[%d][1]=%.5fC P1[%d]=%.5f\n"
            ,i-1,Qi[i-1][1]/h,i,Qi[i][1]/h,i+1,Qi[i+1][1]/h);
      exit (-1);  
     }

 } /* if n>1 stability check */ 
/* qp_sum[unit][i][1]+=Qi[i][2]; / * active heatflow */
qp_sum[unit][i][2]+=Qi[i][2]; /* active heatflow */
energy[i]= init_energy[unit][i]
           +qp_sum[unit][i  ][1]  /* this one to the last node  */
           -qp_sum[unit][i+1][1]  /* this one to the next node i+1 */
           +qp_sum[unit][i  ][2]; /* gains */
deriv_n[unit][i]=energy[i]/mc[wall_index][i];
if (pcm_node==i) deriv_n[unit][i]=GetNodeTempPcm(deriv_n[unit][i],energy[i],
                                  T_melt,mc[wall_index][i],H_melt_upper,mcp2);

return (1);
}; /* GetNodeTemp() */


int GetQrad_outside2(int unit,double *Qrad_outside,double *Tsurface_outside,
                    double *Qconva,double *Q_kA,double T_amb,double Tnode,
                    double eps_a_lw,double alpha_a_conv,double h,double A
                     ,double kA,int mode_ti,double g_win, double k_earth
                   ,double eps_a_sw,double Igt,double cc,int init_flag,double Rd)
 {
 int     a=0;
 int    no_sky_radiation=0;
 /* double cc=0; / * 1:cloudy 0:sunny , clearness clark */
 double Tsky_kelvin,T_environment,T_environment_4;
 double Qrad,QkA,Qconv;
 double error; /* used to define iteration */
 double factor_corr=1e-4;
 double a1,b; /* Qprad-fit-coeffs */
 double Qsol;
#define N_MAX 50


if ( (cc>=0) && (cc<=1) )
   {
   Tsky_kelvin=pow(5.31e-13*pow(T_amb+273,6)/SIGMA,0.25);
   Tsky_kelvin=(1-cc)*Tsky_kelvin+cc*(T_amb+273); /* cc=1 => Tsky = T_amb*/
   }
else   Tsky_kelvin=pow(5.31e-13*pow(T_amb+273,6)/SIGMA,0.25);
                                                      /* clear sky if cc=-1 */
 /* nach Clarke, Rural site: isolated alpha=0.5, beta+gamma=0.5 */
 T_environment_4=Rd*pow(Tsky_kelvin,4)+(1-Rd)*pow(T_amb+273,4);
 T_environment=pow(T_environment_4,0.25)-273;
 /* nonlinear equation in Tsurface: Qrad-Qconv_outside=kA(Tnode-Tsurface */
/* take a fit : generated by data-base from valid_twd/TsurfaceOutside.c 
   Qprad[cc][dT]=a*x+b*y=a*cc+b*dT (a=9.996 b=5.52297)
   execute projects/valid_twd/makefit_Qprad.ssc with parameter eps_a_lw 
*/
/* dT=(*Tsurface_outside - T_amb ); / * usually positive */
/* Qprad =a*cc+b*(Ts-Tamb)-Qpsol  Qpsol=constant 
   Qpconv=alpha*(Ts-Tamb)
   QpkA  =kA*(Tn-Ts)
   Ts*(b+alpha+kA)=kA*Tn+alpha*Tamb+b*Tamb-a*cc
*/
a1=9.996; b=5.52297; /* fit Qprad=a*cc+b*dT */

Qsol=eps_a_sw*A*Igt*h; /* solar gains */
if (no_sky_radiation) Qrad=Qsol=a1=b=Igt=0; /*  setting to null */
*Tsurface_outside= (kA*Tnode+alpha_a_conv*A*T_amb+b*T_amb-a1*cc-Qsol/h)
                  /(b+alpha_a_conv*A+kA);
if (*Tsurface_outside<T_environment)  /* instability */
 {
 *Tsurface_outside=T_environment;
 Qsol=kA*Tnode+alpha_a_conv*A*T_amb+b*T_amb-a1*cc- *Tsurface_outside*
          (b+alpha_a_conv*A+kA);
 Qsol*=h;
 }
/*
alpha_a_conv*A*(Tsurface_outside-T_amb)=kA*(Tnode-Tsurface_outside)
*/

/*
Qprad[cc][dT]=a*x+b*y=a*cc+b*dT
Q_solar-Qrad_outside-Qconv_outside=Qka  
Qrad =h*(a1*cc+b*(*Tsurface_outside - T_amb));
Qconv=h*alpha_a_conv*A*(*Tsurface_outside-T_amb);
QkA  =kA*h*( Tnode - *Tsurface_outside );
Qsol=eps_a_sw*A*IgT*h;
if Tnode>Tsurface>Tamb
*/


if (0) printf("*Tsurface_outside:%.2f Tnode:%.2f T_amb:%.2f Tenv:%.1f\n"
    ,*Tsurface_outside, Tnode,T_amb,T_environment);

if (!no_sky_radiation)  Qrad =h*(a1*cc+b*(*Tsurface_outside - T_amb))+Qsol;
 /* !!!!!!!!!!!!! 
                         Qrad =h*(a1*cc+b*(*Tsurface_outside - T_amb))-Qsol;
*/

Qconv=h*alpha_a_conv*A*(*Tsurface_outside-T_amb); /* negative */
QkA  =kA*h*( Tnode - *Tsurface_outside ); /* positive */

error=QkA-Qrad-Qconv;
if (0) printf(
"Qrad:%.1f Qconv:%.1f QkA:%.1f error:%.1f\n",Qrad,Qconv,QkA,error);
if (fabs(error)>1) {
       printf("Qrad:%.1f Qconv:%.1f QkA:%.1f Qsol:%.2f error:%.1f "
              "Tnode:%.1f Tsurface_outside:%.1f T_amb:%.1f \n"
           ,Qrad,Qconv,QkA,Qsol,error,Tnode,*Tsurface_outside,T_amb);
       exit (-1);
 };
if (*Tsurface_outside<T_environment) { printf("Ts error :Ts:%.1f Tev:%.1f \n"
       ,*Tsurface_outside,T_environment);exit (-1); }

#if 0
Qrad =h*(9.996*cc+5.52297*dT )
      +eps_a_sw*A*IgT*h; /* solar gains */

printf("T_amb:%.2f Ts:%.1f kA:%.2f\n",T_amb,*Tsurface_outside,kA); 
Qconv=h*alpha_a_conv*A*(T_amb- *Tsurface_outside); /* negative */
QkA  =kA*h*( Tnode - *Tsurface_outside ); /* positive */
QkA=+Qrad+Qconv; /* thats not equal ! */
#endif

 *Qconva=Qconv;
 *Qrad_outside=Qrad;
 *Q_kA=-QkA;

if (a) printf("error:%.2f QkA:%.1f/m2 Qconv:%.1f Qrad:%.1f Tnode:%.2f "
       "Tsurf_outside:%.1f,%.1f\n",
   error,*Q_kA/(h*A),Qconv/(A*h),Qrad/(A*h),Tnode,
    *Tsurface_outside-error*factor_corr,*Tsurface_outside);


/* *Qrad_outside=h*eps_a_lw*A*SIGMA*(T_environment_4-pow(Tnode+273,4)); */
/* if (*Qrad_outside< (-1)*h*100 ) *Qrad_outside=(-1)*h*100 ; / *limit rad  */
/* Qconva=h*alpha_a_conv*A*(T_amb-deriv_o[unit][nodes_max]); */

 if (Tnode<-30)  {
   printf("wall[%d]: GetQrad_outside2:\n",unit);
   printf(" T_amb:%.1f T_env:%.1f Tsky:%.1f dT:%.1f S:%.1fW/m2 "
          "Tsurface_outside:%.1f Tnode:%.1f \n"
         ,T_amb,T_environment,Tsky_kelvin-273,Tnode-T_environment,
          eps_a_lw*SIGMA*(T_environment_4-pow(Tnode+273,4) ),
          *Tsurface_outside,Tnode) ;
   exit (-1);
   }

 return 1;
 } /* GetQrad_outside2 */


double GetNodeTempPcm(double T,double energy,double T_melt,double mcp1,
                      double H_melt_upper,double mcp2)
{
int a=0;
double deriv_n=T; /* new temp */
/*
deriv_n[unit][i]=GetNodeTempPcm(energy[i],T_melt,mc[wall_index][i],
                 H_melt_upper[wall_index],m_pcm*cp2_pcm);
*/

if (energy>=T_melt*mcp1) { /* T>=T_melt pcm */ 
  if (energy>H_melt_upper) /* upper T_range */
       deriv_n=T_melt+(energy-H_melt_upper)/(mcp2);
  else deriv_n=T_melt; /* melting range of pcm material */
} /* T>=T_melt */
if (a) printf("GetNode:energy:%.2f T_melt:%.1f mcp1:%.1f H_melt_upper:%.1f "
       "mcp2:%.1f T:%.2f\n",
       energy,T_melt,mcp1,H_melt_upper,mcp2,deriv_n);
return (deriv_n);
}

